function [n,bornes,ref,ldr,nr,ldi,ni]=retindice(ld,parm,method,n1)
%  function [n,bornes,ref,ldr,nr,ldi,ni]=retindice(ld,parm,method);
%  indice de materiaux fonction de la longueur d'onde ld EN MICRONS
%  bornes:domaine de validite (quand interploation retourne nan a l'exterieur)
%  ref:references(chaine de characteres)
%  ldr,nr,ldi,ni:valeurs tabulees dans le cas de tabulations
%--------------------------------------------------------------------------------------------------------------------------
%   parm  | milieu  | bornes en lambda | references
%--------------------------------------------------------------------------------------------------------------------------
%    1    | argent  |  .2      10      | P.Lalanne:  Electronic Handbook of Optical Constant of solid version 1.0 HOC
%
%    1.28 |         | .000124  .04592  | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%    1.35 |         | .04592   .3647   | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%    1.33 |         | .3757    2.066   | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%    1.14 |         | 1.265    9.919   | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%    1.4  |         |  .75     .9      | modele de Drude 'comment' (assez mediocre ...)
%    1.5  |         |  .75     .9      | modele de Drude benchmark
%    1.7  |         |  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 4370 (1972)
%--------------------------------------------------------------------------------------------------------------------------
%    2    |  or     | .15      15      | P.Lalanne:  Palick et Electronic Handbook of Optical Constant of solid version 1.0 HOC
%
%    2.01 |         | .5166    .9537   | JT:    M.C. Theye, Phys. Rev. B2, 3060 (1970)
%    2.02 |         | .521     .985    | obsolete voir 2.7     JT:    P.B. Johnson. R.W. Christy. Phys. Rev. B6. 4370 (1972)
%    2.03 |         | .5       .95     | JT:    K. Weiss. Z. Naturforscher 3a. 143 (1948)	
%    2.04 |         | .55      .95     | JT:    Corn   http://www.sopra   sa.com/index2.htm 
%    2.05 |         | .51662   .99191  | JT:    http://www.sopra   sa.com/index2.htm
%    2.06 |         | .4       .85     | Guillaume sat
%
%    2.28 |         | .000125  .0124   | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%    2.39 |         | .01409   .04768  | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%    2.10 |         | .047     .2      | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%    2.27 |         | .2066    2.066   | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%    2.14 |         | 1.265    4       | JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC
%
%    2.2  |         | .20664   2.479   | H.Sauer:    ref='???????? ';
%    2.3  |         | .5       1       |  approximation polynomiale de degre 4 de Corn   (2.04)
% 
%    2.4  |         |   ?      ?       | modele de Drude (mauvaise approximation)
%
%    2.7  |         |  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 4370 (1972)
%--------------------------------------------------------------------------------------------------------------------------
%    3    | chrome  | 0.20664   1.239  | 
%    20.7 | Chrome  |  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 5059
%    3.7  |         | 0.0100     30.995| ????  Concatenation of: Cr_llnl_cxro + Cr_palik   http://www.luxpop.com????? pas compatible avec  3 et 20.7
%--------------------------------------------------------------------------------------------------------------------------
%    4    | verre   |    0   inf       |  sans dispersion 
%    4.1  | SF11    |                  | 
%    4.2  | SF10    |                  | 
%    4.3  | BK7     |                  |
%    4.4  | SiO2    |                  | extrait de la bibliothque CodeV
%    4.45 |MoSiO2   |0.0001239   .41326|  Sbastien de Rossi   .Calcul  partir des 'atomic scattering factors' de  
%         |(6.31g/cm3)|                |CXRO <http://www-cxro.lbl.gov> et LLNL <http://www-phys.llnl.gov/V_Div/scattering/asf.html>.
%    4.5  |Silicium |   1.1    5.6     | Frey Leviton Madison,Proc,SPIE Vol,6273,6272J'  
%    4.52 |Sili_cr  |0.0001239   3332. |  Sbastien de Rossi      Si_llnl_cxro + Si_palik   
%    4.51 |Germanium|   1.9    5.5     | Frey Leviton Madison,Proc,SPIE Vol,6273,6272J'  
%    4.6  |AlxGA1-xAs|                 | Optolectronique, Emmanuel Rosencher et Borge Vinter, Masson, chapitre 7B2, P264'; 
%    4.61 |PxAs1-xGa|                  | Optolectronique, Emmanuel Rosencher et Borge Vinter, Masson, chapitre 7B2, P264'; 
%    4.62 |GaxIn1-xP|                  | Optolectronique, Emmanuel Rosencher et Borge Vinter, Masson, chapitre 7B2, P264'; 
%    4.71 |Saphir no| 1.1098 1.6932    | Laurent FREY Handbook of Optical   Constants of Solids III, Ed. D. Palik (1998) transmis par P Lalanne
%    4.72 |Saphir ne|                  |
%    4.7  |GaAs     |.08856      17.46 | indice de  GaAs Palik p 434 440
%    4.701|GaAs     |  .3     1.91     | S.Collin transmis par Ch.Sauvan 2_2010
%--------------------------------------------------------------------------------------------------------------------------
%    5    | eau     |                  | 
%    5.1  | eau SD  |    0   inf       |  sans dispersion 
%--------------------------------------------------------------------------------------------------------------------------
%    6    | air     |    0   inf       |  sans dispersion 
%    7    | bio     |    0   inf       |  sans dispersion 
%    8    | rsine  |    0   inf       |  sans dispersion 
%    9    |polypyrrole|  0   inf       |  sans dispersion 
%--------------------------------------------------------------------------------------------------------------------------
%    10   |aluminium| 0.00415  30.99   | http://cvs.savannah.nongnu.org/viewvc/freesnell/al.nk?root=freesnell&view=markup  project freesnell - View of /freesnell/al.nk 23 Octobre 2004 (Transmis par Inbal Friedler et P.Lalanne)
%--------------------------------------------------------------------------------------------------------------------------
%    11   |  titane | 0.000123  200    | Electronic HandBook of Optical Constants of Solids (HOC) transmis par JC Rodier
%--------------------------------------------------------------------------------------------------------------------------
%    12   |  cuivre |.000137756,9.53692| |  A VERIFIER | Handbook of Optical Constants of Solids, Ed. by Edward D. Palik,http://www.luxpop.com
%    12.7 |  cuivre |  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 4370 (1972)
%--------------------------------------------------------------------------------------------------------------------------
%    13.7 | Tungsten|0.000012398,24.7960| |  A VERIFIER | 'W_llnl_cxro + W_palik'; http://www.luxpop.com 
%--------------------------------------------------------------------------------------------------------------------------
%    14.7 | Fer     |  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 5059
%--------------------------------------------------------------------------------------------------------------------------
%    15.7 | Cobalt  |  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 5059
%--------------------------------------------------------------------------------------------------------------------------
%    16.7 | Nickel  |  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 5059
%--------------------------------------------------------------------------------------------------------------------------
%    17.7 |Palladium|  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 5059
%--------------------------------------------------------------------------------------------------------------------------
%    18.7 |  Titane |  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 5059
%--------------------------------------------------------------------------------------------------------------------------
%    19.7 | Vanadium|  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 5059
%--------------------------------------------------------------------------------------------------------------------------
%    21.7 |Manganese|  0.1879,1.9372   | P.B. Johnson. R.W. Christy. Phys. Rev. B6. 5059
%--------------------------------------------------------------------------------------------------------------------------
%    22   |MgO      |  12.2, 43.4      | Handbook of Optical Constants of Solids, Ed. by Edward D. Palik, transmis par Mondher
%--------------------------------------------------------------------------------------------------------------------------
%    23   |SiC      |                  | modele de Drude J le Gall M Olivier JJ Greffet Phys Rev B vol 55 number 15 15 april 1997
%--------------------------------------------------------------------------------------------------------------------------
%
%
%  method :methode d'interpolation dans le cas de tables. 
% ATTENTION Par defaut:method='spline' (voir interp1)
% il est plus prudent d'utiliser 'linear' qui conserve le signe de la partie imaginaire !
% LES VERSIONS 'OFFICIELLES' DE L'OR ET L'ARGENT SONT 2 ET 1 avec method='spline' (option par defaut) 
%
%  dans le cas 4.5 (Silicium) peut etre remplacee par la temperature en Kelvin  (293 par defaut)
%  dans le cas 4.6 (AlxGA1-xAs) peut etre remplacee par x  (0 par defaut)
%% TESTS DES DONNEES: 
%  si pas de variable en sortie, trace de la courbe sauf si ld a une seule valeur
%  si parm est un tableau, trace comparatif des paramatres
%     si en plus ll=[],trace de la courbe entre les bornes les plus grandes des parametres choisis 
%   (dans le cas de tabulation,les points tabules sont marques) 
%
% forme particuliere: n=retindice(ld,d,n2,n1) :indice effectif du mode fondamental transverse 
% d'une fibre circulaire de diamtre d d'indice n2 dans un milieu d'indice n1
% d et ld ont la meme longueur,(ou bien l'un des 2 a un seul element)  n2 et n1 sont reels   n2/n1 peut varier entre 1.18 et 5
%   ( formule approchee (n-n1)/(n2-n1) = (1-1/(a3*x^2+a4)) * exp(-exp(a1*x+a2)) , x=n2*d./ld , puis recherche precise avec retmarcuse )
%
%% EXEMPLES de traces:
%  retindice([],1);
%  retindice(linspace(.2,2,1000),[2.2,2.27,2.14,2,2.4]);title('OR');
%  retindice([],[2.06,2.2,2.27,2.14,2,2.01,2.02,2.03,2.04,2.05,2.7]);title('OR');
%  retindice(linspace(.5,1,1000),[2.2,2.27,2.14,2,2.01,2.02,2.03,2.04,2.05,2.3]);title('OR');
%  retindice([],[1,1.28,1.35,1.33,1.14,1.7]);title('ARGENT');
%  retindice([],10);title('ALUMINIUM');
%  retindice([],3);
%
% See also: RETHELP_POPOV RETDRUDE

if nargin<3;method='spline';end;if nargin<2;parm=1;end;
if nargin>3;[n,bornes,ref]=indicefibre(ld,parm,method,n1);[ldr,nr,ldi,ni]=deal([]);return;end;
%  COMPARAISONS avec superposition de courbes
if length(parm)>1; 
m=length(parm);n=cell(1,m);ldr=cell(1,m);ldi=cell(1,m);nr=cell(1,m);ni=cell(1,m);
if isempty(ld);ld=[inf,-inf];% construction de ld
for ii=1:m;[prv,bornes]=retindice([],parm(ii));if ~isfinite(bornes(2));bornes(2)=15;end;ld(1)=min(ld(1),bornes(1));ld(2)=max(ld(2),bornes(2));end;
ld=linspace(ld(1),ld(2),10000);
end;
reel=1;for ii=1:m;[n{ii},bornes,ref,ldr{ii},nr{ii},ldi{ii},ni{ii}]=retindice(ld,parm(ii),method);if ~isreal(n{ii});reel=0;end;end;
ma={'.k','*r','xb','+g','oc','*m','sm','dc','vg','^b','>r','<k'};maa={'-k','--r','-b','--g','c','m','-m','--c','-g','--b','r','k'};
figure;hold on;leg='lg=legend(';
for ii=1:m;jj=1+mod(ii-1,min(length(ma),length(maa)));
f=find(isfinite(n{ii})&~isnan(n{ii}));n{ii}=n{ii}(f);LD=ld(f);
f=find((ldr{ii}<max(ld))&(ldr{ii}>min(ld)));nr{ii}=nr{ii}(f);ldr{ii}=ldr{ii}(f);
f=find((ldi{ii}<max(ld))&(ldi{ii}>min(ld)));ni{ii}=ni{ii}(f);ldi{ii}=ldi{ii}(f);

if reel;
plot(LD,n{ii},maa{jj},ldr{ii},nr{ii},ma{jj});grid on;;ylabel('n');    
else;    
subplot(2,1,1);hold on;plot(LD,real(n{ii}),maa{jj},ldr{ii},nr{ii},ma{jj});grid on;ylabel('real(n)');
subplot(2,1,2);hold on;plot(LD,imag(n{ii}),maa{jj},ldi{ii},ni{ii},ma{jj});grid on;xlabel('longueur d''onde en microns');ylabel('imag(n)');
end;
if ~isempty(LD);leg=[leg,'''',num2str(parm(ii)),''','];end;
if ~isempty(ldr{ii});leg=[leg,''''','];end;
end;

leg=[leg(1:end-1),');'];eval(leg);set(lg,'fontsize',7);xlabel('longueur d''onde en microns');
return
end;

% CALCUL
switch parm;
case 1;   [n,bornes,ref,ldr,nr,ldi,ni]=indice_philippe(ld,1,method);  
case {1.14,1.28,1.33,1.35}; [n,bornes,ref,ldr,nr,ldi,ni]=indice_jp(ld,parm,method);  
case 2;   [n,bornes,ref,ldr,nr,ldi,ni]=indice_philippe(ld,2,method); 
case {2.10,2.14,2.27,2.28,2.39}; [n,bornes,ref,ldr,nr,ldi,ni]=indice_jp(ld,parm,method); 
case {2.01,2.02,2.03,2.04,2.05}; [n,bornes,ref,ldr,nr,ldi,ni]=indice_jt(ld,parm,method); 
case 4.7;  [n,bornes,ref,ldr,nr,ldi,ni]=indice_GaAs(ld,parm,method);
case {1.7,2.7,12.7,14.7,15.7,16.7,17.7,18.7,19.7,20.7,21.7};[n,bornes,ref,ldr,nr,ldi,ni]=indice_Johnson(ld,parm,method);   
case 2.06; [n,bornes,ref,ldr,nr,ldi,ni]=indice_g(ld,parm,method); 
case 2.2; [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('or',ld,method);  
case 2.3; [n,bornes,ref,ldr,nr,ldi,ni]=indiceor(ld);
case 2.4; [n,bornes,ref,ldr,nr,ldi,ni]=indiceor_drude(ld);
case 1.4; [n,bornes,ref,ldr,nr,ldi,ni]=indiceag_drude(ld);
case 1.5; [n,bornes,ref,ldr,nr,ldi,ni]=indiceag_drude_benchmark(ld);
case 4;   [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('verre',ld,method);    
case 3;   [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('chrome',ld,method); 
case 3.7;[n,bornes,ref,ldr,nr,ldi,ni]=indice_Cr(ld,parm,method);  
case 4;   [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('verre',ld,method);    
case 4.1; [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('SF11',ld,method);    
case 4.2; [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('SF10',ld,method);    
case 4.3; [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('BK7',ld,method);    
case 4.4; [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('SiO2',ld,method);   
case 4.45;[n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('MoSiO2',ld,method);   
case 4.5; [n,bornes,ref,ldr,nr,ldi,ni]=indice_Si(ld,method,0);   
case 4.52;[n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('Si',ld,method);  
case 4.51;[n,bornes,ref,ldr,nr,ldi,ni]=indice_Si(ld,method,1);   
case 4.6; [n,bornes,ref,ldr,nr,ldi,ni]=indice_algaas(0,ld,method);   
case 4.61;[n,bornes,ref,ldr,nr,ldi,ni]=indice_algaas(1,ld,method);   
case 4.62;[n,bornes,ref,ldr,nr,ldi,ni]=indice_algaas(2,ld,method);   
case 4.71;[n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('Saphir_no',ld,method);   
case 4.72;[n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('Saphir_ne',ld,method);   
case 5;   [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('eau',ld,method);    
case 5.1; [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('eauSD',ld,method);    
case 6;   [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('air',ld,method);    
case 7;   [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('bio',ld,method);    
case 8;   [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('rsine',ld,method);    
case 9;   [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('polypyrrole',ld,method);    
case 10;  [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('Aluminium',ld,method);    
case 11;  [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('Titane',ld,method);    
case 12;  [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('Cuivre',ld,method);    
case 13;  [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('Tungsten',ld,method);    
case 22;  [n,bornes,ref,ldr,nr,ldi,ni]=calcul_indice('MgO',ld,method);    
case 23;  [n,bornes,ref,ldr,nr,ldi,ni]=indiceSiC_drude(ld);    
otherwise;error(['parm=',num2str(parm),'  inconnu']);
end;

% TRACE 
if (nargout==0)&(length(ld)~=1);
if isempty(ld);if ~isfinite(bornes(2));bornes(2)=15;end;ld=linspace(max(0,bornes(1)),bornes(2),10000);[n,bornes,ref,ldr,nr,ldi,ni]=retindice(ld,parm,method);end;    
figure;
f=find(isfinite(n)&~isnan(n));n=n(f);ld=ld(f);
if isreal(n);
plot(ld,n,ldr,nr,'.k');axis([min(ld),max(ld),min(n),max(n)]);grid;title(['parm=',num2str(parm),'  ',ref],'fontsize',7);xlabel(['longueur d''onde en microns   bornes=',num2str(bornes)]);ylabel('n');    
else    
subplot(2,1,1);plot(ld,real(n),ldr,nr,'.k');axis([min(ld),max(ld),min(real(n)),max(real(n))]);grid;title(['parm=',num2str(parm),'  ',ref],'fontsize',7);ylabel('real(n)');
subplot(2,1,2);plot(ld,imag(n),ldi,ni,'.k');axis([min(ld),max(ld),min(imag(n)),max(imag(n))]);grid;xlabel(['longueur d''onde en microns   bornes=',num2str(bornes),' method=',method]);ylabel('imag(n)');
end;
end;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [n,bornes,ref,ldr,nr,ldi,ni]=indice_philippe(ld,parm,method);

switch parm;
case 1;% indice de l'argent   Electronic Handbook of Optical Constant of solid version 1.0 HOC
lld=[10 9.919 8.856 7.293 5.636 4.133 3.100 2.296 1.59 1.409 1.265 1.127  1.033    0.9537 .8856 .8266 0.7749 0.7293     .6526 .6199 .5904 .5391 .4959 .4592 0.40 0.3875 0.3757 0.3647 0.3542 .3397 0.3306 .3263 .3179 0.31  0.2988 0.2883 0.2755 0.2638 0.248 .2214 0.2066 0.2]; %en m
nn=[13.30 13.11  10.69 7.461 4.425 2.446 1.387 0.823 .485 0.421 0.375 0.251  0.226    0.198  .163  .1450 0.1430 0.1480     .140  0.131 .1210 .1290 .1300 .144  .173 0.192  0.20   0.186  0.209 0.259 0.371 0.526  0.932 1.323 1.522  1.502  1.441  1.372  1.298 1.208 1.125  1.072]...
+i*[54.10 53.7   49.40 42.50 34.00 25.10 18.80 14.00 9.57 8.370 7.780 7.670  6.990    6.43   5.95  5.500 5.090  4.74        4.15 3.88   3.66 3.250 2.88  2.56  1.95 1.81   1.67   1.61   1.44  1.12  0.813 0.663  0.504 0.647 0.992  1.19   1.31   1.35   1.35  1.3   1.27   1.24 ];
bornes=[  .2     10   ];ref='ARGENT  Electronic Handbook of Optical Constant of solid version 1.0 HOC adapt par P Lalanne';
% la valeur a 10 a te ajoute par extrapolation (retdrude) en 3 2010 (sinon la derniere valeur etait 9.919)
case 2;% indice de l'or   ref Palick  et Electronic Handbook of Optical Constant of solid version 1.0 HOC celle de Philippe 
lld=[0.15 .16 0.17 .18 .2 .2254 .248 .2755 .3024 .3179 .3444 .3757 0.3875 .4 0.4133 0.4275 0.4428 0.4592 0.4769 0.4959 0.5166 0.5391 0.5636 0.6526 0.6888 0.7293 0.7749 0.8266 0.8856 ...
        0.9537 1.512 2 2.48 2.952 3.444 4 4.592 4.959 5.636 6.199 7.085 8 9.184 9.919 15];% en m
nn=[1.419 1.483 1.519 1.47 1.427 1.452 1.484 1.648 1.812 1.84 1.766 1.696 1.674 1.658 1.636 1.616 1.562 1.426 1.242 .910 .608 .402 .306 .166 .160 .164 .174 .188 .210 ...
        .236 .537 .85 1.205 1.598 2.046 2.6 3.289 3.748 4.611 5.423 6.937 8.5 10.84 12.24 23]...
+i*[1.102 1.106 1.07 1.085 1.215 1.442 1.636 1.852 1.92 1.904 1.846 1.906 1.936 1.956 1.958 1.94 1.904 1.840 1.790 1.840 2.120 2.54 2.88 3.15 3.80 4.35 4.86 5.39 5.88 ...
        6.47 9.58 12.6 15.5 18.3 21.3 24.6 28.2 30.5 34.3 37.5 42.0 46.4 51.6 54.7 75];
bornes=[ .15       15     ];ref='OR   Palick  et Electronic Handbook of Optical Constant of solid version 1.0 HOC adapt par P Lalanne';

end  % switch parm

if isempty(ld);n=[];else;n=interp1(lld,nn,ld,method,NaN*(1+1i));end;
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [n,bornes,ref,ldr,nr,ldi,ni]=indice_Cr(ld,parm,method);
% probleme ????????????????????

ld_nCr_niCr=[99.795159       0.95964873      0.021441625;...
       100.60878       0.95915020      0.021808840;...
       101.42903       0.95864534      0.022182217;...
       102.25597       0.95815641      0.022562129;...
       103.08966       0.95766205      0.022948411;...
       103.93014       0.95718875      0.023285276;...
       104.77747       0.95670984      0.023626104;...
       105.63171       0.95617220      0.023948541;...
       106.49292       0.95562656      0.024275716;...
       107.36114       0.95505328      0.024607042;...
       108.23645       0.95447105      0.024943215;...
       109.11889       0.95386561      0.025283665;...
       110.00852       0.95325020      0.025629078;...
       110.90541       0.95261114      0.025978882;...
       111.80961       0.95196028      0.026333764;...
       112.72119       0.95127823      0.026693183;...
       113.64019       0.95058172      0.027058482;...
       114.56669       0.94984513      0.027433321;...
       115.50074       0.94909627      0.027823919;...
       116.44241       0.94833102      0.028279409;...
       117.39175       0.94755753      0.028742360;...
       118.34883       0.94679010      0.029212911;...
       119.31372       0.94601144      0.029691166;...
       120.28647       0.94522329      0.030177250;...
       121.26716       0.94442231      0.030671287;...
       122.25584       0.94360650      0.031173389;...
       123.25258       0.94277387      0.031683718;...
       124.25744       0.94191512      0.032202423;...
       125.27050       0.94103389      0.032730560;...
       126.29182       0.94011384      0.033269941;...
       127.32147       0.93917940      0.033848770;...
       128.35951       0.93823078      0.034512251;...
       129.40601       0.93728186      0.035188252;...
       130.46104       0.93635036      0.035877893;...
       131.52468       0.93541168      0.036580640;...
       132.59699       0.93447278      0.037297574;...
       133.67804       0.93352484      0.038028192;...
       134.76790       0.93257187      0.038773484;...
       135.86665       0.93160929      0.039533045;...
       136.97436       0.93063989      0.040307752;...
       138.09110       0.92966057      0.041095675;...
       139.21694       0.92867329      0.041896807;...
       140.35196       0.92767153      0.042711687;...
       141.49624       0.92665515      0.043540418;...
       142.64984       0.92562357      0.044385111;...
       143.81285       0.92457657      0.045246303;...
       144.98535       0.92351223      0.046124121;...
       146.16740       0.92243013      0.047018976;...
       147.35909       0.92132671      0.047931235;...
       148.56049       0.92020167      0.048861172;...
       149.77169       0.91903665      0.049809231;...
       150.99277       0.91783336      0.050775541;...
       152.22379       0.91657361      0.051820228;...
       153.46486       0.91526346      0.052933239;...
       154.71604       0.91399337      0.054106496;...
       155.97743       0.91274810      0.055330616;...
       157.24910       0.91150771      0.056583403;...
       158.53114       0.91026439      0.057863404;...
       159.82362       0.90901332      0.059173652;...
       161.12665       0.90775010      0.060512145;...
       162.44030       0.90646489      0.061882492;...
       163.76466       0.90515880      0.063282110;...
       165.09982       0.90379091      0.064737966;...
       166.44586       0.90238168      0.066235685;...
       167.80288       0.90098756      0.067925422;...
       169.17096       0.89958822      0.069718729;...
       170.55019       0.89826801      0.071564875;...
       171.94067       0.89696745      0.073453911;...
       173.34249       0.89570725      0.075399315;...
       174.75573       0.89444989      0.077389302;...
       176.18050       0.89319599      0.079439241;...
       177.61688       0.89193150      0.081535432;...
       179.06498       0.89072244      0.083865989;...
       180.52487       0.88951680      0.086292850;...
       181.99668       0.88849209      0.088810231;...
       183.48048       0.88750036      0.091387173;...
       184.97638       0.88669220      0.094055264;...
       186.48447       0.88591523      0.096784002;...
       188.00486       0.88541680      0.099551338;...
       189.53765       0.88496472       0.10237124;...
       191.08293       0.88463402       0.10497244;...
       192.64081       0.88432492       0.10759030;...
       194.21139       0.88405509       0.11028599;...
       195.79478       0.88379699       0.11303599;...
       197.39107       0.88369140       0.11586842;...
       199.00038       0.88360797       0.11875717;...
       200.62281       0.88381413       0.12173345;...
       202.25847       0.88405122       0.12476809;...
       203.90747       0.88477102       0.12785926;...
       205.56990       0.88553165       0.13100002;...
       207.24590       0.88650966       0.13343721;...
       208.93555       0.88752439       0.13591690;...
       210.63898       0.88851021       0.13844539;...
       212.35630       0.88955547       0.14101814;...
       214.08762       0.89095864       0.14364149;...
       215.83306       0.89247303       0.14627591;...
       217.59272       0.89471939       0.14857838;...
       219.36674       0.89699385       0.15078468;...
       221.15521       0.89899405       0.15191709;...
       222.95827       0.90099901       0.15305738;...
       224.77603       0.90268189       0.15420695;...
       226.60860       0.90441419       0.15536454;...
       228.45612       0.90617411       0.15653140;...
       230.31870       0.90803739       0.15770645;...
       232.19647       0.91017676       0.15889092;...
       234.08954       0.91246646       0.15986333;...
       235.99805       0.91516161       0.15997108;...
       237.92212       0.91774270       0.15996035;...
       239.86188       0.91974226       0.15958661;...
       241.81745       0.92174898       0.15919802;...
       243.78896       0.92365731       0.15882548;...
       245.77655       0.92568686       0.15843972;...
       247.78034       0.92794598       0.15806859;...
       249.80047       0.93037140       0.15724110;...
       251.83707       0.93310393       0.15536758;...
       253.89028       0.93552369       0.15330421;...
       255.96022       0.93719111       0.15091060;...
       258.04704       0.93888977       0.14851737;...
       260.15087       0.94060406       0.14619569;...
       262.27186       0.94276608       0.14387969;...
       264.41013       0.94566731       0.14162807;...
       266.56584       0.94819448       0.13713347;...
       268.73913       0.95009257       0.12916604;...
       270.93013       0.95039662       0.12138765;...
       273.13900       0.94841676       0.11396278;...
       275.36588       0.94556566       0.10689559;...
       277.61091       0.94160197       0.10034291;...
       279.87424       0.93688441      0.094132420;...
       282.15603       0.93129663      0.088350510;...
       284.45642       0.92470726      0.083282920;...
       286.77557       0.91705451      0.078985732;...
       289.11362       0.90964354      0.076220431;...
       291.47073       0.90245936      0.074938993;...
       293.84707       0.89555716      0.073682477;...
       296.24277       0.88888251      0.072442343;...
       298.65801       0.88216379      0.071726865;...
       301.09294       0.87537765      0.071425217;...
       303.54772       0.86885264      0.071283198;...
       306.02252       0.86247592      0.071253210;...
       308.51749       0.85597983      0.071296200;...
       311.03281       0.84935530      0.071384033;...
       313.56863       0.84278670      0.072021151;...
       316.12512       0.83619226      0.072986449;...
       318.70246       0.82967406      0.074028881;...
       321.30081       0.82313073      0.075119946;...
       323.92034       0.81664074      0.076459095;...
       326.56123       0.81010436      0.077928882;...
       329.22366       0.80344131      0.079508847;...
       331.90778       0.79664399      0.081152317;...
       334.61380       0.78999054      0.083425200;...
       337.34187       0.78330616      0.085964704;...
       340.09219       0.77689789      0.088594840;...
       342.86492       0.77049566      0.091290915;...
       345.66027       0.76422987      0.094084772;...
       348.47840       0.75791611      0.096947364;...
       351.31951       0.75169676      0.099915055;...
       354.18379       0.74541075       0.10295426;...
       357.07142       0.73920122       0.10609935;...
       359.98258       0.73291486       0.10931713;...
       362.91749       0.72667475       0.11264364;...
       365.87632       0.72034628       0.11604484;...
       368.85927       0.71405471       0.11957468;...
       371.86655       0.70766832       0.12318425;...
       374.89834       0.70131284       0.12693177;...
       377.95485       0.69485724       0.13076203;...
       381.03628       0.68841138       0.13474132;...
       384.14283       0.68185884       0.13880668;...
       387.27471       0.67527181       0.14309204;...
       390.43212       0.66857035       0.14747163;...
       393.61528       0.66198120       0.15211311;...
       396.82438       0.65528133       0.15685370;...
       400.05966       0.64867625       0.16179450;...
       403.32130       0.64196001       0.16683817;...
       406.60954       0.63531865       0.17209200;...
       409.92459       0.62857073       0.17745763;...
       413.26667       0.62189182       0.18304471;...
       420.271      0.780000      0.210000;...
       430.486      0.770000      0.220000;...
       439.645      0.750000      0.230000;...
       449.203      0.730000      0.250000;...
       459.185      0.720000      0.260000;...
       469.621      0.710000      0.280000;...
       480.543      0.700000      0.310000;...
       490.040      0.680000      0.330000;...
       499.919      0.680000      0.360000;...
       510.206      0.670000      0.390000;...
       518.745      0.680000      0.430000;...
       529.829      0.680000      0.450000;...
       539.043      0.690000      0.480000;...
       551.022      0.700000      0.500000;...
       560.995      0.710000      0.520000;...
       568.716      0.720000      0.540000;...
       579.346      0.730000      0.560000;...
       590.381      0.740000      0.580000;...
       598.937      0.740000      0.600000;...
       610.739      0.750000      0.620000;...
       619.900      0.770000      0.640000;...
       629.340      0.780000      0.650000;...
       639.072      0.800000      0.670000;...
       649.110      0.820000      0.680000;...
       659.468      0.830000      0.680000;...
       670.162      0.840000      0.690000;...
       681.209      0.850000      0.700000;...
       688.778      0.870000      0.700000;...
       700.452      0.870000      0.710000;...
       708.457      0.880000      0.720000;...
       720.814      0.890000      0.720000;...
       729.294      0.900000      0.730000;...
       737.976      0.900000      0.740000;...
       751.394      0.910000      0.750000;...
       760.614      0.920000      0.760000;...
       770.062      0.920000      0.770000;...
       789.682      0.950000      0.790000;...
       799.871      0.960000      0.800000;...
       810.327      0.970000      0.800000;...
       832.081       1.00000      0.820000;...
       837.703       1.01000      0.820000;...
       849.178       1.03000      0.820000;...
       866.993       1.06000      0.820000;...
       891.942       1.09000      0.820000;...
       898.406       1.10000      0.820000;...
       918.370       1.12000      0.800000;...
       932.180       1.13000      0.800000;...
       946.412       1.15000      0.770000;...
       968.594       1.15000      0.750000;...
       991.840       1.15000      0.730000;...
       1007.97       1.14000      0.710000;...
       1033.17       1.13000      0.700000;...
       1059.66       1.10000      0.690000;...
       1078.09       1.09000      0.690000;...
       1097.17       1.07000      0.690000;...
       1116.94       1.05000      0.690000;...
       1147.96       1.03000      0.700000;...
       1180.76       1.01000      0.720000;...
       1203.69       1.00000      0.720000;...
       1239.80      0.980000      0.730000;...
       1270.29      0.950000      0.730000;...
       1299.58      0.920000      0.740000;...
       1340.32      0.860000      0.770000;...
       1369.94      0.830000      0.810000;...
       1389.91      0.810000      0.830000;...
       1420.16      0.780000      0.870000;...
       1439.95      0.760000      0.890000;...
       1460.31      0.740000      0.920000;...
       1490.14      0.720000      0.970000;...
       1510.11      0.710000       1.00000;...
       1530.62      0.690000       1.04000;...
       1549.75      0.680000       1.07000;...
       1569.37      0.680000       1.11000;...
       1589.49      0.670000       1.15000;...
       1610.13      0.670000       1.19000;...
       1629.17      0.660000       1.23000;...
       1648.67      0.670000       1.27000;...
       1679.95      0.690000       1.33000;...
       1700.69      0.700000       1.36000;...
       1719.56      0.710000       1.39000;...
       1738.85      0.720000       1.42000;...
       1768.62      0.740000       1.45000;...
       1799.42      0.760000       1.49000;...
       1820.56      0.750000       1.51000;...
       1850.45      0.780000       1.57000;...
       1878.48      0.800000       1.59000;...
       1910.32      0.820000       1.63000;...
       1940.22      0.850000       1.66000;...
       1971.07      0.880000       1.68000;...
       1999.68      0.890000       1.69000;...
       2029.13      0.920000       1.72000;...
       2069.78      0.940000       1.73000;...
       2101.36      0.960000       1.73000;...
       2141.28      0.970000       1.74000;...
       2171.28      0.970000       1.75000;...
       2209.98      0.950000       1.74000;...
       2250.09      0.940000       1.76000;...
       2300.19      0.930000       1.80000;...
       2339.25      0.900000       1.83000;...
       2379.65      0.870000       1.87000;...
       2430.98      0.860000       1.94000;...
       2479.60      0.850000       2.01000;...
       2530.20      0.860000       2.07000;...
       2582.92      0.860000       2.13000;...
       2637.87      0.860000       2.21000;...
       2695.22      0.880000       2.28000;...
       2755.11      0.890000       2.35000;...
       2817.73      0.900000       2.42000;...
       2869.91      0.910000       2.49000;...
       2937.91      0.940000       2.58000;...
       3001.94      0.980000       2.67000;...
       3076.43       1.02000       2.76000;...
       3162.76       1.06000       2.85000;...
       3237.08       1.12000       2.95000;...
       3332.80       1.18000       3.04000;...
       3415.43       1.26000       3.12000;...
       3512.18       1.33000       3.18000;...
       3625.15       1.39000       3.24000;...
       3723.12       1.43000       3.31000;...
       3850.31       1.44000       3.40000;...
       3961.02       1.48000       3.54000;...
       4091.75       1.54000       3.71000;...
       4245.89       1.65000       3.89000;...
       4380.92       1.80000       4.06000;...
       4558.09       1.99000       4.22000;...
       4714.07       2.22000       4.36000;...
       4900.40       2.49000       4.44000;...
       5123.14       2.75000       4.46000;...
       5321.03       2.98000       4.45000;...
       5584.68       3.18000       4.41000;...
       5820.66       3.34000       4.38000;...
       6107.39       3.48000       4.36000;...
       7004.52       3.84000       4.37000;...
       8156.58       4.23000       4.34000;...
       8265.33       4.27000       4.33000;...
       8491.78       4.31000       4.32000;...
       8609.72       4.33000       4.32000;...
       8855.71       4.38000       4.31000;...
       9116.18       4.42000       4.30000;...
       9392.42       4.47000       4.29000;...
       9685.94       4.49000       4.28000;...
       9998.39       4.50000       4.28000;...
       10331.7       4.52000       4.29000;...
       10687.9       4.53000       4.30000;...
       11069.6       4.53000       4.31000;...
       11479.6       4.53000       4.34000;...
       11921.2       4.51000       4.36000;...
       12398.0       4.47000       4.43000;...
       12914.6       4.47000       4.50000;...
       13476.1       4.45000       4.56000;...
       13775.6       4.43000       4.60000;...
       14416.3       4.35000       4.66000;...
       15119.5       4.24000       4.81000;...
       15894.9       4.13000       5.03000;...
       16754.1       4.06000       5.30000;...
       17711.4       4.01000       5.59000;...
       18784.8       3.96000       5.95000;...
       19996.8       4.01000       6.31000;...
       21375.9       4.01000       6.65000;...
       22959.3       3.92000       7.06000;...
       24796.0       3.83000       7.58000;...
       26952.2       3.72000       8.20000;...
       29519.0       3.47000       8.97000;...
       32626.3       3.09000       10.2000;...
       36464.7       2.89000       12.0000;...
       41326.7       3.15000       14.3000;...
       47684.6       3.91000       17.1000;...
       56354.5       5.30000       20.6000;...
       68877.8       8.73000       25.4000;...
       88557.1       15.3000       26.4000;...
       123980.       11.8000       29.8000;...
       137756.       11.8000       33.9000;...
       154975.       13.6000       42.2000;...
       206633.       21.2000       42.0000;...
       309950.       14.9000       65.2000];


lld=ld_nCr_niCr(:,1)/10000;nn=ld_nCr_niCr(:,2)+i*ld_nCr_niCr(:,3);
if isempty(ld);n=[];else;n=interp1(lld,nn,ld,method,NaN*(1+1i));end;
bornes=[min(lld),max(lld)];ref='Concatenation of: Cr_llnl_cxro + Cr_palik   http://www.luxpop.com';

ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [n,bornes,ref,ldr,nr,ldi,ni]=indice_GaAs(ld,parm,method);
% indice de  GaAs Palik p 434 440

ld_nGaAs=...
[.08856 .840+.602i;...
.09537 .864+.709i;...
.1033 .895+.791i;...
.1148 .923+.881i;...
.1240 .913+.974i;...
.1378 .901+1.136i;...
.1550 .899+1.435i;...
.1771 1.063+1.838i;...
.1879 1.247+2.047i;...
.1968 1.441+1.988i;...
.2000 1.424+1.976i;...
.2066 1.264+2.472i;...
.2087 1.287+2.523i;...
.2101 1.288+2.556i;...
.2123 1.304+2.592i;...
.2138 1.311+2.625i;...
.2160 1.321+2.673i;...
.2175 1.325+2.710i;...
.2198 1.339+2.772i;...
.2214 1.349+2.815i;...
.2238 1.367+2.885i;...
.2254 1.383+2.936i;...
.2279 1.410+3.019i;...
.2296 1.430+3.079i;...
.2322 1.468+3.181i;...
.2339 1.499+3.255i;...
.2366 1.552+3.384i;...
.2384 1.599+3.484i;...
.2412 1.699+3.660i;...
.2431 1.802+3.795i;...
.2460 2.044+3.992i;...
.2480 2.273+4.084i;...
.2510 2.654+4.106i;...
.2530 2.890+4.047i;...
.2562 3.187+3.898i;...
.2583 3.342+3.770i;...
.2616 3.511+3.574i;...
.2638 3.596+3.452i;...
.2672 3.708+3.280i;...
.2695 3.768+3.169i;...
.2731 3.850+3.018i;...
.2755 3.913+2.919i;...
.2792 4.004+2.715i;...
.2818 4.015+2.563i;...
.2857 3.981+2.368i;...
.2883 3.939+2.260i;...
.2924 3.864+2.132i;...
.2952 3.810+2.069i;...
.2995 3.736+2.001i;...
.3024 3.692+1.969i;...
.3069 3.634+1.935i;...
.3100 3.601+1.920i;...
.3147 3.559+1.907i;...
.3179 3.538+1.904i;...
.3229 3.512+1.905i;...
.3263 3.501+1.909i;...
.3315 3.488+1.920i;...
.3351 3.485+1.931i;...
.3406 3.489+1.950i;...
.3444 3.495+1.965i;...
.3502 3.513+1.992i;...
.3542 3.531+2.013i;...
.3563 3.541+2.024i;...
.3583 3.553+2.036i;...
.3604 3.566+2.049i;...
.3625 3.580+2.062i;...
.3647 3.596+2.076i;...
.3668 3.614+2.091i;...
.3690 3.635+2.107i;...
.3712 3.657+2.123i;...
.3734 3.681+2.142i;...
.3757 3.709+2.162i;...
.3780 3.740+2.183i;...
.3803 3.776+2.207i;...
.3827 3.818+2.232i;...
.3850 3.871+2.260i;...
.3875 3.938+2.288i;...
.3899 4.023+2.307i;...
.3924 4.126+2.304i;...
.3949 4.229+2.270i;...
.3974 4.313+2.212i;...
.4000 4.373+2.146i;...
.4025 4.413+2.082i;...
.4052 4.439+2.029i;...
.4078 4.462+1.988i;...
.4105 4.483+1.961i;...
.4133 4.509+1.948i;...
.4161 4.550+1.952i;...
.4189 4.626+1.967i;...
.4217 4.755+1.960i;...
.4246 4.917+1.885i;...
.4275 5.052+1.721i;...
.4305 5.107+1.529i;...
.4335 5.102+1.353i;...
.4366 5.065+1.206i;...
.4397 5.015+1.088i;...
.4428 4.959+.991i;...
.4460 4.902+.912i;...
.4492 4.845+.846i;...
.4525 4.793+.789i;...
.4558 4.741+.739i;...
.4592 4.694+.696i;...
.4626 4.649+.659i;...
.4661 4.605+.626i;...
.4696 4.567+.595i;...
.4732 4.525+.569i;...
.4769 4.492+.539i;...
.4806 4.456+.517i;...
.4843 4.423+.497i;...
.4881 4.392+.476i;...
.4920 4.362+.458i;...
.4959 4.333+.441i;...
.4999 4.305+.426i;...
.5040 4.279+.411i;...
.5081 4.254+.398i;...
.5123 4.229+.385i;...
.5166 4.205+.371i;...
.5209 4.183+.359i;...
.5254 4.162+.347i;...
.5299 4.141+.337i;...
.5344 4.120+.327i;...
.5391 4.100+.320i;...
.5438 4.082+.308i;...
.5486 4.063+.301i;...
.5535 4.045+.294i;...
.5585 4.029+.285i;...
.5636 4.013+.276i;...
.5687 3.998+.266i;...
.5740 3.983+.257i;...
.5794 3.968+.251i;...
.5848 3.954+.245i;...
.5904 3.940+.240i;...
.5961 3.927+.232i;...
.6019 3.914+.228i;...
.6078 3.902+.223i;...
.6138 3.890+.213i;...
.6199 3.878+.211i;...
.6262 3.867+.203i;...
.6326 3.856+.196i;...
.6391 3.846+.187i;...
.6458 3.836+.183i;...
.6526 3.826+.179i;...
.6595 3.817+.173i;...
.6666 3.809+.173i;...
.6738 3.799+.168i;...
.6812 3.792+.158i;...
.6888 3.785+.151i;...
.6965 3.779+.152i;...
.7045 3.772+.134i;...
.7126 3.762+.127i;...
.7208 3.752+.118i;...
.7293 3.742+.112i;...
.7380 3.734+.105i;...
.7469 3.725+.101i;...
.7560 3.716+.097i;...
.7653 3.707+.093i;...
.7749 3.700+.091i;...
.7847 3.693+.089i;...
.7948 3.685+.087i;...
.8051 3.679+.085i;...
.8157 3.672+.083i;...
.8266 3.666+.080i;...
.8321 3.614+.0728i;...
.8377 3.614+.0686i;...
.8434 3.614+.0664i;...
.8492 3.614+.0628i;...
.8551 3.614+.0612i;...
.8610 3.614+.0568i;...
.8640 3.614+.0557i;...
.8670 3.614+.0572i;...
.8700 3.614+.0554i;...
.8731 3.614+.0271i;...
.8793 3.614+.00595i;...
.8856 3.614+.00169i;...
.8920 3.614+.000568i;...
.8984 3.614+.000250i;...
.9050 3.614+.000133i;...
.9184 3.5690;...
.9537 3.5388;...
.9919 3.5138;...
1.033 3.4920;...
1.078 3.4724;...
1.127 3.4546;...
1.181 3.4383;...
1.240 3.4232;...
1.305 3.4094;...
1.378 3.3965;...
1.459 3.3847;...
1.550 3.3737;...
1.653 3.3636;...
1.771 3.3543;...
1.907 3.3457;
2.066 3.3378;...
2.254 3.3306;...
2.480 3.3240;...
2.755 3.3180;...
3.100 3.3125;...
3.542 3.3075;...
4.130 3.3027;...
4.275 3.3017;...
4.428 3.3008;...
4.592 3.2998;...
4.768 3.2988;...
4.959 3.2978;...
5.166 3.2968;...
5.391 3.2954;...
5.636 3.2946;...
5.904 3.2934;...
6.199 3.2921;...
6.526 3.2907;...
6.888 3.2891;...
7.293 3.2874;...
7.749 3.2854;...
8.266 3.2831;...
8.856 3.2803;...
9.537 3.2769;...
10.33 3.2727;...
11.27 3.2671;...
12.40 3.2597+4.93e-6i;...
12.65 3.2597+1.61e-5i;...
12.91 3.2597+3.49e-5i;...
13.05 3.2597+3.63e-5i;...
13.19 3.2597+2.72e-5i;...
13.48 3.2597+1.61e-5i;...
13.78 3.2493+1.64e-5i;...
14.09 3.2493+2.24e-5i;...
14.42 3.2493+2.40e-5i;...
14.76 3.2493+2.10e-5i;...
15.12 3.2493+2.16e-5i;...
15.50 3.2336+2.83e-5i;...
15.90 3.2336+4.55e-5i;...
16.31 3.2336+7.600e-5i;...
16.75 3.2336+9.46e-5i;...
16.98 3.2336+1.00e-4i;...
17.22 3.2336+1.50e-4i;...
17.46 3.2336+2.05e-4i];

lld=ld_nGaAs(:,1);nn=ld_nGaAs(:,2);
if isempty(ld);n=[];else;n=interp1(lld,nn,ld,method,NaN*(1+1i));end;
bornes=[min(lld),max(lld)];ref=['  indice de  GaAs Palik p 434 440'];

ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [n,bornes,ref,ldr,nr,ldi,ni]=indice_Johnson(ld,parm,method);
% indice du Cuivre Argent Or fer Cobalt Nickel Palladium Titane Vanadium Chrome  Manganese  P.B. Johnson. R.W. Christy. Phys. Rev. B6. 4370  et  5059
                         V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg=...
[   .64   1.09+13.43i      .24+14.08i     .92+13.78i    3.17+6.12i   3.87+7.79i   3.47+9.09i   3.34+9.89i   3.51+5.19i   2.79+7.90i   3.71+5.04i   3.89+5.95i;...
    .77    .76+11.12i      .15+11.85i     .56+11.21i    3.11+5.39i   3.61+7.26i   3.14+7.96i   3.01+8.59i   3.69+4.70i   2.77+6.34i   3.66+4.31i   3.78+5.41i;...
    .89    .60+9.439i      .13+10.10i     .43+9.519i    3.09+4.83i   3.42+6.77i   2.96+7.08i   2.80+7.65i   3.67+4.37i   2.64+5.18i   3.69+3.84i   3.65+5.02i;...
   1.02    .48+8.245i      .09+8.828i     .35+8.145i    3.03+4.39i   3.17+6.31i   2.79+6.43i   2.66+6.90i   3.62+4.15i   2.70+4.33i   3.67+3.60i   3.48+4.74i;...
   1.14    .36+7.217i      .04+7.795i     .27+7.150i    2.97+4.06i   2.94+5.88i   2.65+5.93i   2.52+6.33i   3.50+4.02i   2.87+3.78i   3.58+3.58i   3.30+4.53i;...
   1.26    .32+6.421i      .04+6.992i     .22+6.350i    2.92+3.79i   2.78+5.50i   2.48+5.55i   2.34+5.89i   3.35+3.97i   2.94+3.50i   3.41+3.57i   3.10+4.35i;...
   1.39    .30+5.768i      .04+6.312i     .17+5.663i    2.96+3.56i   2.65+5.16i   2.40+5.23i   2.23+5.50i   3.29+3.96i   3.12+3.34i   3.30+3.52i   2.97+4.18i;...
   1.51    .26+5.180i      .04+5.727i     .16+5.083i    2.94+3.39i   2.53+4.88i   2.26+4.97i   2.06+5.19i   3.21+4.01i   3.16+3.25i   3.20+3.48i   2.83+4.03i;...
   1.64    .24+4.665i      .03+5.242i     .14+4.542i    2.87+3.28i   2.40+4.64i   2.13+4.73i   1.95+4.89i   3.00+4.01i   3.20+3.20i   3.08+3.42i   2.70+3.91i;...
   1.76    .21+4.205i      .04+4.838i     .13+4.103i    2.86+3.19i   2.31+4.45i   2.06+4.50i   1.86+4.65i   2.86+3.96i   3.18+3.15i   3.05+3.39i   2.62+3.78i;...
   1.88    .22+3.747i      .05+4.483i     .14+3.697i    2.92+3.10i   2.25+4.27i   1.99+4.26i   1.80+4.42i   2.76+3.84i   3.25+3.09i   3.09+3.34i   2.56+3.65i;...
   2.01    .30+3.205i      .06+4.152i     .21+3.272i    2.88+3.05i   2.19+4.11i   1.99+4.02i   1.75+4.21i   2.67+3.72i   3.54+3.02i   3.17+3.30i   2.51+3.54i;...
   2.13    .70+2.704i      .05+3.858i     .29+2.863i    2.94+2.99i   2.13+3.96i   1.96+3.80i   1.68+4.02i   2.60+3.58i   3.83+3.00i   3.22+3.30i   2.47+3.43i;...
   2.26   1.02+2.577i      .06+3.586i     .43+2.455i    2.95+2.93i   2.05+3.82i   1.92+3.61i   1.64+3.84i   2.54+3.43i   4.00+3.04i   3.18+3.33i   2.39+3.33i;...
   2.38   1.18+2.608i      .05+3.324i     .62+2.081i    2.86+2.91i   1.97+3.68i   1.85+3.42i   1.57+3.68i   2.44+3.30i   3.86+3.18i   2.94+3.33i   2.32+3.23i;...
   2.50   1.22+2.564i      .05+3.093i    1.04+1.833i    2.74+2.88i   1.88+3.55i   1.82+3.25i   1.52+3.54i   2.36+3.19i   3.92+3.26i   2.75+3.30i   2.25+3.14i;...
   2.63   1.25+2.483i      .05+2.869i    1.31+1.849i    2.67+2.82i   1.81+3.41i   1.78+3.09i   1.46+3.39i   2.32+3.10i   3.81+3.38i   2.51+3.24i   2.19+3.06i;...
   2.75   1.24+2.397i      .04+2.657i    1.38+1.914i    2.59+2.77i   1.74+3.28i   1.73+2.95i   1.41+3.26i   2.27+3.04i   3.59+3.46i   2.33+3.14i   2.11+2.98i;...
   2.88   1.25+2.305i      .04+2.462i    1.45+1.948i    2.48+2.71i   1.67+3.17i   1.71+2.82i   1.37+3.14i   2.21+3.01i   3.52+3.49i   2.19+3.04i   2.06+2.90i;...
   3.00   1.28+2.207i      .05+2.275i    1.46+1.958i    2.35+2.65i   1.61+3.05i   1.70+2.69i   1.33+3.03i   2.14+2.98i   3.31+3.49i   2.08+2.93i   2.00+2.82i;...
   3.12   1.32+2.116i      .05+2.070i    1.47+1.952i    2.24+2.58i   1.57+2.93i   1.72+2.57i   1.30+2.93i   2.08+2.95i   3.07+3.48i   2.00+2.83i   1.96+2.74i;...
   3.25   1.33+2.045i      .05+1.864i    1.46+1.933i    2.12+2.50i   1.53+2.82i   1.72+2.48i   1.26+2.83i   1.99+2.93i   2.87+3.45i   1.92+2.74i   1.92+2.67i;...
   3.37   1.36+1.975i      .07+1.657i    1.48+1.895i    2.02+2.43i   1.50+2.71i   1.70+2.40i   1.24+2.74i   1.90+2.90i   2.68+3.40i   1.87+2.69i   1.89+2.59i;...
   3.50   1.37+1.916i      .10+1.419i    1.50+1.866i    1.93+2.35i   1.49+2.61i   1.74+2.32i   1.23+2.65i   1.82+2.87i   2.51+3.34i   1.84+2.64i   1.89+2.51i;...
   3.62   1.36+1.864i      .14+1.142i    1.48+1.871i    1.85+2.27i   1.48+2.52i   1.78+2.26i   1.22+2.57i   1.72+2.82i   2.42+3.29i   1.76+2.58i   1.87+2.45i;...
   3.74   1.34+1.821i      .17+0.829i    1.48+1.883i    1.78+2.19i   1.46+2.44i   1.84+2.22i   1.20+2.50i   1.61+2.74i   2.37+3.22i   1.69+2.53i   1.86+2.38i;...
   3.87   1.38+1.783i      .81+0.392i    1.54+1.898i    1.74+2.12i   1.45+2.37i   1.93+2.19i   1.21+2.42i   1.55+2.66i   2.28+3.15i   1.65+2.47i   1.86+2.32i;...
   3.99   1.38+1.729i     1.13+0.616i    1.53+1.893i    1.69+2.06i   1.44+2.31i   2.01+2.18i   1.21+2.35i   1.50+2.57i   2.17+3.06i   1.58+2.40i   1.86+2.25i;...
   4.12   1.40+1.679i     1.34+0.964i    1.53+1.889i    1.67+2.00i   1.44+2.25i   2.02+2.18i   1.20+2.29i   1.45+2.46i   2.07+2.95i   1.53+2.34i   1.86+2.19i;...
   4.24   1.42+1.633i     1.39+1.161i    1.49+1.878i    1.65+1.94i   1.44+2.19i   2.03+2.20i   1.18+2.23i   1.40+2.36i   1.93+2.84i   1.48+2.28i   1.85+2.14i;...
   4.36   1.45+1.633i     1.41+1.264i    1.47+1.869i    1.64+1.88i   1.44+2.14i   2.03+2.23i   1.18+2.18i   1.35+2.26i   1.87+2.73i   1.45+2.21i   1.85+2.08i;...
   4.49   1.46+1.646i     1.41+1.331i    1.43+1.847i    1.62+1.84i   1.44+2.09i   2.01+2.26i   1.19+2.13i   1.30+2.17i   1.81+2.63i   1.43+2.15i   1.86+2.03i;...
   4.61   1.45+1.668i     1.38+1.372i    1.38+1.803i    1.59+1.79i   1.44+2.04i   1.96+2.29i   1.16+2.10i   1.27+2.07i   1.77+2.55i   1.39+2.08i   1.85+1.99i;...
   4.74   1.41+1.691i     1.35+1.387i    1.35+1.749i    1.56+1.75i   1.44+2.01i   1.89+2.30i   1.13+2.07i   1.27+1.99i   1.70+2.45i   1.38+2.03i   1.84+1.94i;...
   4.86   1.41+1.741i     1.33+1.393i    1.33+1.688i    1.53+1.70i   1.45+1.97i   1.82+2.32i   1.11+2.03i   1.26+1.91i   1.64+2.37i   1.37+1.97i   1.83+1.91i;...
   4.98   1.37+1.783i     1.31+1.389i    1.33+1.631i    1.51+1.66i   1.45+1.93i   1.73+2.31i   1.08+1.99i   1.27+1.83i   1.60+2.29i   1.36+1.91i   1.82+1.86i;...
   5.11   1.34+1.799i     1.30+1.378i    1.32+1.577i    1.50+1.61i   1.46+1.91i   1.65+2.29i   1.04+1.95i   1.28+1.77i   1.57+2.20i   1.36+1.85i   1.82+1.82i;...
   5.23   1.28+1.802i     1.28+1.367i    1.32+1.536i    1.48+1.57i   1.47+1.89i   1.57+2.25i   1.00+1.91i   1.30+1.72i   1.58+2.14i   1.38+1.80i   1.81+1.79i;...
   5.36   1.23+1.792i     1.28+1.357i    1.30+1.497i    1.48+1.53i   1.47+1.87i   1.49+2.20i   0.97+1.86i   1.31+1.68i   1.59+2.07i   1.40+1.77i   1.78+1.76i;...
   5.48   1.18+1.768i     1.26+1.344i    1.31+1.460i    1.47+1.49i   1.45+1.86i   1.43+2.15i   0.94+1.81i   1.32+1.66i   1.62+2.01i   1.43+1.74i   1.74+1.73i;...
   5.60   1.13+1.737i     1.25+1.342i    1.30+1.427i    1.47+1.47i   1.43+1.85i   1.38+2.09i   0.92+1.76i   1.32+1.66i   1.63+1.98i   1.45+1.73i   1.73+1.70i;...
   5.73   1.08+1.699i     1.22+1.336i    1.30+1.387i    1.47+1.44i   1.41+1.84i   1.34+2.02i   0.91+1.70i   1.32+1.67i   1.65+1.95i   1.47+1.72i   1.72+1.67i;...
   5.85   1.04+1.651i     1.20+1.325i    1.30+1.350i    1.47+1.43i   1.38+1.82i   1.32+1.96i   0.89+1.65i   1.31+1.68i   1.66+1.94i   1.46+1.72i   1.70+1.64i;...
   5.98   1.01+1.599i     1.18+1.312i    1.30+1.304i    1.49+1.41i   1.36+1.78i   1.29+1.89i   0.87+1.60i   1.31+1.69i   1.68+1.94i   1.46+1.71i   1.67+1.61i;...
   6.10    .99+1.550i     1.15+1.296i    1.33+1.277i    1.47+1.40i   1.32+1.75i   1.28+1.82i   0.86+1.55i   1.27+1.69i   1.65+1.94i   1.43+1.70i   1.63+1.58i;...
   6.22    .98+1.493i     1.14+1.277i    1.33+1.251i    1.45+1.40i   1.29+1.71i   1.28+1.75i   0.85+1.50i   1.25+1.68i   1.62+1.94i   1.39+1.70i   1.62+1.55i;...
   6.35    .97+1.440i     1.12+1.255i    1.34+1.226i    1.42+1.39i   1.26+1.67i   1.29+1.69i   0.85+1.46i   1.22+1.66i   1.58+1.93i   1.35+1.68i   1.59+1.52i;...
   6.47    .95+1.388i     1.10+1.232i    1.32+1.203i    1.35+1.37i   1.21+1.63i   1.29+1.64i   0.85+1.42i   1.16+1.64i   1.54+1.92i   1.31+1.65i   1.55+1.50i;...
   6.60    .94+1.337i     1.07+1.212i    1.28+1.188i    1.29+1.35i   1.16+1.59i   1.26+1.60i   0.84+1.38i   1.10+1.62i   1.47+1.90i   1.28+1.64i   1.48+1.47i];

	

lld=1.239829488174885./V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,1);
switch parm;
case 12.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,2);ref='Cu';
case 1.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,3);ref='ARGENT';
case 2.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,4);ref='OR';
case 14.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,5);ref='Fer';
case 15.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,6);ref='Cobalt';
case 16.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,7);ref='Nickel';
case 17.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,8);ref='Palladium';
case 18.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,9);ref='Titane';
case 19.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,10);ref='Vanadium';
case 20.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,11);ref='Chrome';
case 21.7;nn=V_nCu_nAG_nOr_nFer_nCo_nNi_nPa_nTi_nVa_nCr_nMg(:,12);ref='Manganese';
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(ld);n=[];else;n=interp1(1./lld,nn,1./ld,method,NaN*(1+1i));end;% pour compatibilite avec Collin on interpole en Ev
bornes=[min(lld),max(lld)];ref=[ref,' P.B. Johnson. R.W. Christy. Phys. Rev. B6. 4370  et  5059'];
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [n,bornes,ref,ldr,nr,ldi,ni]=indice_jp(ld,parm,method);
% indice de l'ARGENT
%  14   B.Dold,R.Mecke.Optik 22,435(1965)
%  28   H.J.Hagemann,W.Gudat,C.Kunz.DESY Report Sr-74/7,Hamburg,(1974) (voir aussi 40 ?)
%  33   P.Winsemius,F.F.van Kampen,H.P.Lengkeek,C.G.van Went.J.Phys.F6,158351976)
%  35   G.Leveque,C.G.Olson,D.W.Lynch.Phys.Rev.B 24,4654 (1983)
ldnag=[...
[.0001240 , 0.999+1.29e-6i,28];...
[.0001550 , 1.000+2.92e-6i,28];...
[.0002066 , 1.002+8.36e-6i,28];...
[.0002480 , 1.001+1.52e-5i,28];...
[.0002755 , 1.001+2.21e-5i,28];...
[.0003099 , 1.003+3.57e-5i,28];...
[.0003444 , 1.003+3.24e-5i,28];...
[.0003542 , 1.003+1.72e-5i,28];...
[.0004133 , 1.001+1.43e-5i,28];...
[.0004959 , 1.001+2.40e-5i,28];...
[.0006199 , 1.001+4.72e-5i,28];...
[.0008265 , 0.999+0.000121i,28];...
[.00124 , 0.998+0.000523i,28];...
[.00155 , 0.997+0.00108i,28];...
[.001675 , 0.998+0.00142i,28];...
[.002066 , 0.997+0.00269i,28];...
[.002480 , 0.997+0.00457i,28];...
[.003099 , 0.998+0.00882i,28];...
[.003351 , 1.007+0.00823i,28];...
[.003542 , 1.001+0.00168i,28];...
[.004133 , 0.996+0.00232i,28];...
[.004428 , 0.994+0.00270i,28];...
[.004768 , 0.992+0.00311i,28];...
[.005166 , 0.989+0.00350i,28];...
[.005635 , 0.987+0.00345i,28];...
[.006199 , 0.982+0.00377i,28];...
[.006888 , 0.978+0.00431i,28];...
[.007293 , 0.973+0.00442i,28];...
[.007749 , 0.968+0.00438i,28];...
[.008265 , 0.963+0.00407i,28];...
[.008856 , 0.955+0.00354i,28];...
[.009537 , 0.943+0.00366i,28];...
[.01033 , 0.926+0.00622i,28];...
[.01127 , 0.902+0.0172i,28];...
[.01181 , 0.89+0.0261i,28];...
[.01240 , 0.876+0.0382i,28];...
[.01305 , 0.861+0.0565i,28];...
[.01378 , 0.849+0.0824i,28];...
[.01459 , 0.846+0.111i,28];...
[.01550 , 0.848+0.139i,28];...
[.01589 , 0.851+0.149i,28];...
[.01631 , 0.853+0.158i,28];...
[.01675 , 0.853+0.166i,28];...
[.01722 , 0.851+0.181i,28];...
[.01771 , 0.859+0.197i,28];...
[.01823 , 0.871+0.208i,28];...
[.01878 , 0.883+0.211i,28];...
[.01937 , 0.885+0.210i,28];...
[.02000 , 0.881+0.211i,28];...
[.02066 , 0.873+0.221i,28];...
[.02138 , 0.868+0.237i,28];...
[.02214 , 0.871+0.259i,28];...
[.02296 , 0.885+0.274i,28];...
[.02339 , 0.890+0.275i,28];...
[.02384 , 0.889+0.276i,28];...
[.02431 , 0.886+0.281i,28];...
[.02480 , 0.884+0.290i,28];...
[.02530 , 0.885+0.300i,28];...
[.02583 , 0.888+0.309i,28];...
[.02638 , 0.893+0.317i,28];...
[.02695 , 0.896+0.323i,28];...
[.02818 , 0.899+0.334i,28];...
[.02952 , 0.897+0.349i,28];...
[.03099 , 0.896+0.368i,28];...
[.03179 , 0.895+0.378i,28];...
[.03263 , 0.892+0.388i,28];...
[.03351 , 0.885+0.400i,28];...
[.03444 , 0.876+0.418i,28];...
[.03542 , 0.865+0.454i,28];...
[.03646 , 0.879+0.489i,28];...
[.03755 , 0.899+0.514i,28];...
[.03874 , 0.921+0.528i,28];...
[.03999 , 0.932+0.534i,28];...
[.04132 , 0.931+0.541i,28];...
[.04275 , 0.919+0.557i,28];...
[.04350 , 0.911+0.572i,28];...
[.04428 , 0.902+0.590i,28];...%[.04508 , 0.851+0.616i,28];...% elimine  ?
[.04592 , 0.886+0.650i,28];...
[.04592 , 0.882+0.399i,35];...
[.04679 , 0.886+0.421i,35];...
[.04769 , 0.887+0.444i,35];...
[.04862 , 0.891+0.468i,35];...
[.05061 , 0.906+0.522i,35];...
[.05276 , 0.943+0.584i,35];...
[.05391 , 0.980+0.614i,35];...
[.0551 , 1.029+0.624i,35];...
[.05636, 1.075+0.615i,35];...
[.05767, 1.104+0.584i,35];...
[.05904, 1.109+0.561i,35];...
[.06048, 1.105+0.549i,35];...
[.06199, 1.098+0.548i,35];...
[.06526, 1.096+0.565i,35];...
[.06888, 1.101+0.593i,35];...
[.07293, 1.121+0.635i,35];...
[.07749, 1.167+0.676i,35];...
[.07999, 1.202+0.691i,35];...
[.08266, 1.243+0.693i,35];...
[.08551, 1.281+0.679i,35];...
[.08856, 1.308+0.656i,35];...
[.09184, 1.322+0.628i,35];...
[.09537, 1.322+0.603i,35];...
[.09919, 1.315+0.588i,35];...
[.1025, 1.308+0.581i,35];...
[.1051, 1.304+0.578i,35];...
[.1078, 1.300+0.573i,35];...
[.1107, 1.293+0.566i,35];...
[.1137, 1.280+0.560i,35];...
[.1170, 1.265+0.560i,35];...
[.1204, 1.252+0.564i,35];...
[.1240, 1.241+0.568i,35];...
[.1278, 1.229+0.566i,35];...
[.1348, 1.182+0.550i,35];...
[.1378, 1.149+0.552i,35];...
[.1409, 1.112+0.563i,35];...
[.1442, 1.073+0.581i,35];...
[.1476, 1.032+0.610i,35];...
[.1512, 0.993+0.653i,35];...
[.1550, 0.962+0.706i,35];...
[.1590, 0.940+0.770i,35];...
[.1631, 0.935+0.832i,35];...
[.1675, 0.936+0.892i,35];...
[.1722, 0.942+0.951i,35];...
[.1771, 0.953+1.01i,35];...
[.1823, 0.969+1.07i,35];...
[.1879, 0.995+1.13i,35];...
[.1937, 1.028+1.18i,35];...
[.1968, 1.048+1.21i,35];...
[.2000, 1.072+1.24i,35];...
[.2033, 1.098+1.26i,35];...
[.2066, 1.125+1.27i,35];...
[.2138, 1.173+1.29i,35];...
[.2214, 1.208+1.30i,35];...
[.2296, 1.238+1.31i,35];...
[.2384, 1.265+1.33i,35];...
[.2480, 1.298+1.35i,35];...
[.2530, 1.320+1.35i,35];...
[.2583, 1.343+1.35i,35];...
[.2638, 1.372+1.35i,35];...
[.2695, 1.404+1.33i,35];...
[.2755, 1.441+1.31i,35];...
[.2818, 1.476+1.26i,35];...
[.2883, 1.502+1.19i,35];...
[.2952, 1.519+1.08i,35];...
[.2988, 1.522+0.992i,35];...
[.3024, 1.496+0.882i,35];...
[.3061, 1.432+0.766i,35];...
[.3100, 1.323+0.647i,35];...
[.3115, 1.246+0.586i,35];...
[.3139, 1.149+0.540i,35];...
[.3155, 1.044+0.514i,35];...
[.3179, 0.932+0.504i,35];...
[.3195, 0.815+0.526i,35];...
[.3220, 0.708+0.565i,35];...
[.3237, 0.616+0.609i,35];...
[.3263, 0.526+0.663i,35];...
[.3306, 0.371+0.813i,35];...
[.3324, 0.321+0.902i,35];...
[.3351, 0.294+0.986i,35];...
[.3397, 0.259+1.12i,35];...
[.3444, 0.238+1.24i,35];...
[.3542, 0.209+1.44i,35];...
[.3647, 0.186+1.61i,35];...
[.3757, 0.200+1.67i,33];...
[.3875, 0.192+1.81i,33];...
[.4000, 0.173+1.95i,33];...
[.4133, 0.173+2.11i,33];...
[.4275, 0.160+2.26i,33];...
[.4428, 0.157+2.40i,33];...
[.4592, 0.144+2.56i,33];...
[.4769, 0.132+2.72i,33];...
[.4959, 0.130+2.88i,33];...
[.5166, 0.130+3.07i,33];...
[.5391, 0.129+3.25i,33];...
[.5636, 0.120+3.45i,33];...
[.5904, 0.121+3.66i,33];...
[.6199, 0.131+3.88i,33];...
[.6526, 0.140+4.15i,33];...
[.6888, 0.140+4.44i,33];...
[.7293, 0.148+4.74i,33];...
[.7749, 0.143+5.09i,33];...
[.8266, 0.145+5.50i,33];...
[.8856, 0.163+5.95i,33];...
[.9537, 0.198+6.43i,33];...
[1.033, 0.226+6.99i,33];...
[1.127, 0.251+7.67i,33];...
[1.240, 0.329+8.49i,33];...
[1.265, 0.375+7.78i,14];...
[1.291, 0.383+7.92i,14];...
[1.305, 0.358+8.95i,33];...% 33 au lieu de14 ?
[1.319, 0.392+8.06i,14];...
[1.348, 0.401+8.21i,14];...
[1.378, 0.411+8.37i,14];...
[1.378, 0.396+9.48i,33];...
[1.409, 0.421+8.37i,14];...
[1.442, 0.431+8.70i,14];...
[1.459, 0.446+10.1i,33];...
[1.476, 0.442+8.88i,14];...
[1.512, 0.455+9.08i,14];...
[1.550, 0.469+9.32i,14];...
[1.550, 0.514+10.8i,33];...
[1.59, 0.485+9.57i,14];... % 14 au lieu de33 ?
[1.631, 0.501+9.84i,14];...% 14 au lieu de33 ?
[1.653, 0.624+11.5i,33];...
[1.675, 0.519+10.1i,14];...
[1.722, 0.537+10.4i,14];...
[1.771, 0.557+10.7i,14];...
[1.771, 0.844+12.2i,33];...
[1.823, 0.578+11.1i,14];...
[1.879, 0.600+11.4i,14];...
[1.907, 0.873+13.3i,33];...
[1.937, 0.624+11.8i,14];...
[2.000, 0.650+12.2i,14];...
[2.066, 0.668+12.6i,14];...
[2.066 1.064+14.4i,33];...
[2.138, 0.729+13.0i,14];...
[2.214, 0.774+13.5i,14];...
[2.296, 0.823+14.0i,14];...
[2.384, 0.878+14.5i,14];...
[2.480, 0.939+15.1i,14];...
[2.583, 1.007+15.7i,14];...
[2.695, 1.083+16.4i,14];...
[2.818, 1.168+17.1i,14];...
[2.952, 1.265+17.9i,14];...
[3.100, 1.387+18.8i,14];...
[3.263, 1.536+19.8i,14];...
[3.444, 1.710+20.9i,14];...
[3.647, 1.915+22.1i,14];...
[3.875, 2.160+23.5i,14];...
[4.133, 2.446+25.1i,14];...
[4.428, 2.786+26.9i,14];...
[4.769, 3.202+29.0i,14];...
[5.166, 3.732+31.3i,14];...
[5.636, 4.425+34.0i,14];...
[6.199, 5.355+37.0i,14];...
[6.526, 5.960+38.6i,14];...
[6.888, 6.670+40.4i,14];...
[7.293, 7.461+42.5i,14];...
[7.749, 8.376+44.8i,14];...
[8.266,9.441+47.1i,14];...
[8.856, 10.69+49.4i,14];...
[9.537, 12.21+52.2i,14];...
[9.919, 13.11+53.7i,14]];

% indice de l'OR
%  10   L.R.Canfield,G.Hass,W.R.Hunter.J.Physique 25,124(1964).
%  14   B.Dold,R.Mecke.Optik 22,435(1965)
%  27   M.L.Theye.Phys.Rev.B2,3060(1970)
%  28   P.O.Nilsson.Phys.Kondens.Mater.11,1(1970)
%  39   H.J.Hagemann,W.Gudat,C.Kunz.DESY Report Sr-74/7,Hamburg,(1974) (voir aussi 55 ? )

ldnor=[...
[.000125, 0.9999719+2.369e-6i,28];...
[.000135,0.9999665+3.13e-6i,28];...
[.000145,0.9999608+3.13e-6i,28];...
[.000150,0.9999558+3.13e-6i,28];...
[.000175,0.9999415+7.78e-6i,28];...
[.00020,0.9999227+1.25e-5i,28];...
[.00025,0.999879+2.75e-5i,28];...
[.00030,0.999831+5.25e-5i,28];...
[.00035,0.999794+9.07e-5i,28];...
[.00040,0.999769+0.00012,28];...
[.00050,0.999706+0.000236i,28];...
[.00060,0.999637+9.88e-5i,28];...
[.00070,0.999395+0.000168i,28];...
[.00080,0.999156+0.000267i,28];...
[.000834,0.999085+0.00031i,28];...
[.000989,0.99885+0.00044i,28];...
[.001044,0.99871+0.000533i,28];...
[.001225,0.9982+0.00094i,28];...
[.001334,0.99788+0.00123i,28];...
[.001456,0.99759+0.00157i,28];...
[.001597,0.998734+0.00186i,28];...
[.001759,0.99708+0.00234i,28];...
[.001945,0.99665+0.00268i,28];...
[.002164,0.99652+0.00285i,28];...
[.002362,0.99624+0.00376i,28];...
[.002478,0.99600+0.0056i,28];...
[.002742,0.99592+0.0061i,28];...
[.003136,0.99525+0.00615i,28];...
[.004133,0.991+0.00895i,28];...
[.004769,0.988+0.0106i,28];...
[.005636,0.986+0.0118i,28];...
[.006199,0.983+0.0115i,28];...
[.006888,0.979+0.0107i,28];...
[.008265,0.963+0.0101i,28];...
[.01033,0.929+0.0178i,28];...
[.01127,0.911+0.0283i,28];...
[.01181,0.900+0.0353i,28];...
[.0124,0.889+0.0454i,28];...
[.01409,0.889+0.0554i,39];...
[.01417,0.888+0.0558i,39];...
[.01425,0.887+0.0576i,39];...
[.01442,0.886+0.0613i,39];...
[.01459,0.887+0.0648i,39];...
[.01467,0.888+0.0654i,39];...
[.01476,0.888+0.0633i,39];...
[.01485,0.885+0.0637i,39];...
[.01494,0.884+0.0653i,39];...
[.01512,0.881+0.0682i,39];...
[.01550,0.878+0.0752i,39];...
[.01589,0.875+0.0818i,39];...
[.01631,0.873+0.0871i,39];...
[.01675,0.869+0.0927i,39];...
[.01722,0.862+0.0997i,39];...
[.01771,0.857+0.112i,39];...
[.01823,0.855+0.124i,39];...
[.01878,0.856+0.136i,39];...
[.01937,0.857+0.147i,39];...
[.02000,0.859+0.156i,39];...
[.02066,0.863+0.162i,39];...
[.02138,0.861+0.166i,39];...
[.02214,0.856+0.171i,39];...
[.02296,0.848+0.180i,39];...
[.02384,0.839+0.190i,39];...
[.02480,0.828+0.204i,39];...
[.02530,0.821+0.213i,39];...
[.02583,0.814+0.225i,39];...
[.02638,0.809+0.238i,39];...
[.02695,0.805+0.254i,39];...
[.02755,0.805+0.270i,39];...
[.02818,0.807+0.282i,39];...
[.02883,0.807+0.292i,39];...
[.02952,0.804+0.303i,39];...
[.03024,0.799+0.317i,39];...
[.03099,0.795+0.334i,39];...
[.03179,0.793+0.353i,39];...
[.03263,0.791+0.371i,39];...
[.03351,0.788+0.391i,39];...
[.03444,0.786+0.414i,39];...
[.03542,0.783+0.439i,39];...
[.03646,0.779+0.473i,39];...
[.03757,0.781+0.515i,39];...
[.03874,0.805+0.570i,39];...
[.03999,0.851+0.596i,39];...
[.04065,0.870+0.601i,39];...
[.04133,0.887+0.603i,39];...
[.04203,0.900+0.601i,39];...
[.04275,0.905+0.599i,39];...
[.04428,0.901+0.607i,39];...
[.04592,0.895+0.635i,39];...
[.04700,0.855+0.548i,10];...
[.04768,0.896+0.679i,39];...
[.04800,0.846+0.565i,10];...
[.04900,0.846+0.600i,10];...
[.050,0.85+0.645i,10];...
[.051,0.860+0.695i,10];...
[.052,0.872+0.740i,10];...
[.053,0.890+0.795i,10];...
[.054,0.915+0.825i,10];...
[.055,0.950+0.840i,10];...
[.056,0.985+0.848i,10];...
[.057,1.022+0.850i,10];...
[.058,1.055+0.842i,10];...
[.059,1.085+0.830i,10];...
[.060,1.113+0.813i,10];...
[.061,1.134+0.795i,10];...
[.062,1.146+0.77i,10];...
[.063,1.153+0.75i,10];...
[.064,1.157+0.73i,10];...
[.065,1.155+0.71i,10];...
[.066,1.140+0.70i,10];...
[.067,1.125+0.694i,10];...
[.068,1.107+0.687i,10];...
[.069,1.088+0.680i,10];...
[.070,1.075+0.678i,10];...
[.071,1.06+0.680i,10];...
[.072,1.05+0.685i,10];...
[.073,1.042+.690i,10];...
[.074,1.038+0.697i,10];...
[.075,1.033+.704i,10];...
[.076,1.030+0.713i,10];...
[.077,1.029+0.720i,10];...
[.078,1.028+0.730i,10];...
[.079,1.028+0.739i,10];...
[.080,1.029+0.745i,10];...
[.081,1.030+0.752i,10];...
[.082,1.033+0.759i,10];...
[.083,1.037+0.765i,10];...
[.084,1.041+0.770i,10];...
[.085,1.048+0.775i,10];...
[.086,1.053+0.780i,10];...
[.087,1.061+0.784i,10];...
[.088,1.070+0.789i,10];...
[.089,1.080+0.793i,10];...
[.090,1.090+0.798i,10];...
[.091,1.100+0.801i,10];...
[.092,1.110+0.806i,10];...
[.093,1.121+0.809i,10];...
[.094,1.133+0.812i,10];...
[.095,1.146+0.815i,10];...
[.096,1.159+0.819i,10];...
[.097,1.170+0.823i,10];...
[.098,1.180+0.826i,10];...
[.099,1.190+0.831i,10];...
[.100,1.200+0.836i,10];...
[.101,1.207+0.842i,10];...
[.102,1.210+0.848i,10];...
[.103,1.213+0.853i,10];...
[.104,1.215+0.860i,10];...
[.105,1.217+0.865i,10];...
[.106,1.217+0.870i,10];...
[.107,1.218+0.878i,10];...
[.108,1.218+0.885i,10];...
[.109,1.220+0.893i,10];...
[.110,1.222+0.900i,10];...
[.111,1.223+0.907i,10];...
[.112,1.225+0.914i,10];...
[.113,1.228+0.922i,10];...
[.114,1.232+0.927i,10];...
[.115,1.237+0.932i,10];...
[.116,1.242+0.935i,10];...
[.117,1.247+0.942i,10];...
[.118,1.250+0.950i,10];...
[.119,1.255+0.955i,10];...
[.120,1.260+0.962i,10];...
[.121,1.265+0.967i,10];...
[.122,1.270+0.975i,10];...
[.123,1.275+0.982i,10];...
[.124,1.280+0.987i,10];...
[.125,1.285+0.994i,10];...
[.126,1.290+1.000i,10];...
[.127,1.295+1.005i,10];...
[.128,1.300+1.012i,10];...
[.129,1.304+1.017i,10];...
[.130,1.308+1.020i,10];...
[.131,1.313+1.027i,10];...
[.132,1.318+1.032i,10];...
[.133,1.323+1.038i,10];...
[.134,1.328+1.045i,10];...
[.135,1.333+1.050i,10];...
[.136,1.338+1.053i,10];...
[.137,1.345+1.058i,10];...
[.138,1.350+1.063i,10];...
[.139,1.355+1.067i,10];...
[.140,1.360+1.072i,10];...
[.145,1.386+1.089i,10];...
[.150,1.419+1.102i,10];...
[.155,1.450+1.108i,10];...
[.160,1.483+1.106i,10];...
[.165,1.512+1.093i,10];...
[.170,1.519+1.070i,10];...
[.175,1.500+1.070i,10];...
[.180,1.470+1.085i,10];...
[.185,1.442+1.107i,10];...
[.190,1.427+1.135i,10];...
[.195,1.424+1.170i,10];...
[.200,1.427+1.215i,10];...
[.2066,1.422+1.306i,27];...
[.2101,1.430+1.334i,27];...
[.2138,1.432+1.364i,27];...
[.2175,1.438+1.388i,27];...
[.2214,1.442+1.418i,27];...
[.2254,1.452+1.442i,27];...
[.2296,1.454+1.478i,27];...
[.2339,1.462+1.510i,27];...
[.2384,1.470+1.550i,27];...
[.2431,1.478+1.590i,27];...
[.2480,1.484+1.636i,27];...
[.2530,1.490+1.698i,27];...
[.2583,1.504+1.748i,27];...
[.2638,1.546+1.784i,27];...
[.2695,1.598+1.822i,27];...
[.2755,1.648+1.852i,27];...
[.2818,1.690+1.882i,27];...
[.2883,1.742+1.900i,27];...
[.2952,1.776+1.918i,27];...
[.3024,1.812+1.920i,27];...
[.3100,1.83+1.916i,27];...
[.3179,1.84+1.904i,27];...
[.3263,1.824+1.878i,27];...
[.3351,1.798+1.86i,27];...
[.3444,1.766+1.846i,27];...
[.3542,1.74+1.848i,27];...
[.3647,1.716+1.862i,27];...
[.3757,1.696+1.906i,27];...
[.3875,1.674+1.936i,27];...
[.4000,1.658+1.956i,27];...
[.4133,1.636+1.958i,27];...
[.4275,1.616+1.940i,27];...
[.4428,1.562+1.904i,27];...
[.4592,1.426+1.846i,27];...
[.4769,1.242+1.796i,27];...
[.4959,0.916+1.840i,27];...
[.5166,0.608+2.12i,27];...
[.5391,0.402+2.54i,27];...
[.5636,0.306+2.88i,27];...
[.5904,0.236+2.98i,27];...% partie imaginaire absente 
[.6199,0.194+2.98i,27];...% partie imaginaire absente 
[.6526,0.166+3.15i,27];...
[.6888,0.160+3.80i,27];...
[.7293,0.164+4.35i,27];...
[.7749,0.174+4.86i,27];...
[.8266,0.188+5.39i,27];...
[.8856,0.210+5.88i,27];...
[.9537,0.236+6.47i,27];...
[1.033,0.272+7.07i,27];...
[1.127,0.312+7.93i,27];...
[1.240,0.372+8.77i,27];...
[1.265,0.389+8.09i,14];...
[1.291,0.403+8.25i,14];...
[1.319,0.419+8.42i,14];...%[1.384,0.436+8.59i,14];...% elimine
[1.378,0.454+8.77i,14];...
[1.378,0.458+9.72i,27];...
[1.409,0.473+8.96i,14];...
[1.442,0.493+9.15i,14];...
[1.476,0.515+9.36i,14];...
[1.512,0.537+9.58i,14];...
[1.550,0.559+9.81i,14];...
[1.550,0.550+11.5i,27];...
[1.590,0.583+10.1i,14];...
[1.631,0.609+10.3i,14];...
[1.675,0.636+10.6i,14];...
[1.722,0.665+10.9i,14];...
[1.771,0.696+11.2i,14];...
[1.823,0.730+11.5i,14];...
[1.879,0.767+11.9i,14];...
[1.937,0.807+12.2i,14];...
[2.000,0.850+12.6i,14];...
[2.066,0.896+13.0i,14];...
[2.066,0.976+15.1i,27];...
[2.138,0.947+13.4i,14];...
[2.214,1.002+13.9i,14];...
[2.296,1.063+14.4i,14];...
[2.384,1.130+14.9i,14];...
[2.480,1.205+15.5i,14];...
[2.583,1.287+16.1i,14];...
[2.695,1.379+16.8i,14];...
[2.818,1.482+17.5i,14];...
[2.952,1.598+18.3i,14];...
[3.100,1.728+19.2i,14];...
[3.179,1.800+19.7i,14];...
[3.263,1.876+20.2i,14];...
[3.351,1.958+20.7i,14];...
[3.444,2.046+21.3i,14];...
[3.542,2.141+21.9i,14];...
[3.647,2.242+22.5i,14];...
[3.757,2.352+23.1i,14];...
[3.875,2.471+23.9i,14];...
[4.000,2.600+24.6i,14]];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch parm;
case 1.14;f=find(ldnag(:,3)==14);lld=ldnag(f,1);nn=ldnag(f,2);ref='ARGENT';
case 1.28;f=find(ldnag(:,3)==28);lld=ldnag(f,1);nn=ldnag(f,2);ref='ARGENT';
case 1.33;f=find(ldnag(:,3)==33);lld=ldnag(f,1);nn=ldnag(f,2);ref='ARGENT';
case 1.35;f=find(ldnag(:,3)==35);lld=ldnag(f,1);nn=ldnag(f,2);ref='ARGENT';

case 2.10;f=find(ldnor(:,3)==10);lld=ldnor(f,1);nn=ldnor(f,2);ref='OR';
case 2.14;f=find(ldnor(:,3)==14);lld=ldnor(f,1);nn=ldnor(f,2);ref='OR';
case 2.27;f=find(ldnor(:,3)==27);lld=ldnor(f,1);nn=ldnor(f,2);ref='OR';
case 2.28;f=find(ldnor(:,3)==28);lld=ldnor(f,1);nn=ldnor(f,2);ref='OR';
case 2.39;f=find(ldnor(:,3)==39);lld=ldnor(f,1);nn=ldnor(f,2);ref='OR';
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isempty(ld);n=[];else;n=interp1(lld,nn,ld,method,NaN*(1+1i));end;
bornes=[min(lld),max(lld)];ref=[ref,'  JPH:   Electronic Handbook of Optical Constant of solid version 1.0 HOC'];

ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [n,bornes,ref,ldr,nr,ldi,ni]=indice_g(ld,parm,method);
% indice de l'OR  transmis par Guillaume

ldn=...
[[3.100 400.00 1.593671 1.927484 ];...
[3.092 401.00 1.593113 1.928917];... 
[3.084 402.00 1.592094 1.928374];... 
[3.077 403.00 1.592971 1.931185];... 
[3.069 404.00 1.590616 1.933102];... 
[3.062 405.00 1.589634 1.931455 ];...
[3.054 406.00 1.587825 1.930266];... 
[3.047 407.00 1.586347 1.930694];... 
[3.039 408.00 1.584491 1.931619];... 
[3.032 409.00 1.582791 1.931123];... 
[3.024 410.00 1.582391 1.934107];... 
[3.017 411.00 1.581230 1.932698];... 
[3.010 412.00 1.580891 1.935256];... 
[3.002 413.00 1.577796 1.934884];...
[2.995 414.00 1.577589 1.933468];... 
[2.988 415.00 1.575351 1.934037 ];...
[2.981 416.00 1.573016 1.931807];... 
[2.973 417.00 1.571844 1.933889];... 
[2.966 418.00 1.568827 1.932858];... 
[2.959 419.00 1.567370 1.933212];... 
[2.952 420.00 1.565154 1.932961];... 
[2.945 421.00 1.564147 1.931504];... 
[2.938 422.00 1.561194 1.931643];... 
[2.931 423.00 1.560094 1.931826];... 
[2.924 424.00 1.556955 1.930842 ];...
[2.917 425.00 1.555231 1.930512];... 
[2.911 426.00 1.552635 1.929849];... 
[2.904 427.00 1.549864 1.930258];... 
[2.897 428.00 1.547097 1.928511];... 
[2.890 429.00 1.546624 1.929637];... 
[2.884 430.00 1.541944 1.927093];... 
[2.877 431.00 1.541342 1.927904];... 
[2.870 432.00 1.538111 1.926468];... 
[2.864 433.00 1.534878 1.925035];... 
[2.857 434.00 1.530758 1.923546];... 
[2.850 435.00 1.529725 1.924425];... 
[2.844 436.00 1.526496 1.921477];... 
[2.837 437.00 1.523095 1.920838];... 
[2.831 438.00 1.518725 1.918658];... 
[2.824 439.00 1.516732 1.917444];... 
[2.818 440.00 1.512954 1.916988];... 
[2.812 441.00 1.509703 1.914347];... 
[2.805 442.00 1.505250 1.912582];... 
[2.799 443.00 1.502423 1.909941];... 
[2.793 444.00 1.495469 1.907444];... 
[2.786 445.00 1.492977 1.907344];... 
[2.780 446.00 1.488706 1.903870];... 
[2.774 447.00 1.484704 1.901493];... 
[2.768 448.00 1.479942 1.899697];... 
[2.762 449.00 1.476751 1.895640];... 
[2.755 450.00 1.470546 1.893338];... 
[2.749 451.00 1.465892 1.891525];... 
[2.743 452.00 1.463412 1.889254];... 
[2.737 453.00 1.456993 1.886589];... 
[2.731 454.00 1.453675 1.884653];... 
[2.725 455.00 1.447147 1.881304];... 
[2.719 456.00 1.440418 1.877338];... 
[2.713 457.00 1.436858 1.875308];... 
[2.707 458.00 1.429352 1.872100];... 
[2.701 459.00 1.422702 1.870504];... 
[2.695 460.00 1.415321 1.866307];... 
[2.690 461.00 1.409168 1.862298];... 
[2.684 462.00 1.401504 1.858060];... 
[2.678 463.00 1.394799 1.856113];... 
[2.672 464.00 1.388138 1.852316];... 
[2.667 465.00 1.379506 1.848740];... 
[2.661 466.00 1.373321 1.846043];... 
[2.655 467.00 1.365142 1.841828];... 
[2.649 468.00 1.356725 1.836752];... 
[2.644 469.00 1.346298 1.831977];... 
[2.638 470.00 1.335905 1.829526];... 
[2.633 471.00 1.327293 1.827489];... 
[2.627 472.00 1.315189 1.822407];... 
[2.621 473.00 1.305462 1.820136];... 
[2.616 474.00 1.295665 1.817515];... 
[2.610 475.00 1.282073 1.812655];... 
[2.605 476.00 1.270647 1.809266];... 
[2.599 477.00 1.256634 1.804940];... 
[2.594 478.00 1.245636 1.804216];... 
[2.589 479.00 1.232832 1.799570];... 
[2.583 480.00 1.219217 1.795319];... 
[2.578 481.00 1.206148 1.794829];... 
[2.572 482.00 1.191123 1.790686];... 
[2.567 483.00 1.179044 1.790717];... 
[2.562 484.00 1.164037 1.789762];... 
[2.557 485.00 1.144842 1.788467 ];...
[2.551 486.00 1.125800 1.785852];... 
[2.546 487.00 1.108534 1.787382];... 
[2.541 488.00 1.093250 1.786468];... 
[2.536 489.00 1.074229 1.788020];... 
[2.530 490.00 1.055052 1.791455];... 
[2.525 491.00 1.036412 1.793800];... 
[2.520 492.00 1.018347 1.795617];... 
[2.515 493.00 0.997824 1.800482];... 
[2.510 494.00 0.977971 1.802902];... 
[2.505 495.00 0.958958 1.810583];... 
[2.500 496.00 0.940198 1.817862];... 
[2.495 497.00 0.920489 1.823847];... 
[2.490 498.00 0.899418 1.831755];... 
[2.485 499.00 0.884265 1.840033];... 
[2.480 500.00 0.863933 1.847510];... 
[2.475 501.00 0.847976 1.857077];... 
[2.470 502.00 0.830925 1.868255];... 
[2.465 503.00 0.814742 1.879394];... 
[2.460 504.00 0.800136 1.890633];... 
[2.455 505.00 0.781407 1.899501];... 
[2.450 506.00 0.765912 1.914794];... 
[2.446 507.00 0.748925 1.927159];...
[2.441 508.00 0.732637 1.940351];... 
[2.436 509.00 0.717865 1.954522];... 
[2.431 510.00 0.701345 1.968982];... 
[2.426 511.00 0.688186 1.983109];... 
[2.422 512.00 0.672163 1.998982];... 
[2.417 513.00 0.660007 2.013553];...
[2.412 514.00 0.650393 2.030219];... 
[2.408 515.00 0.635045 2.042308];... 
[2.403 516.00 0.623902 2.058305];... 
[2.398 517.00 0.612440 2.073833];... 
[2.394 518.00 0.601408 2.088335];... 
[2.389 519.00 0.590708 2.101004];... 
[2.384 520.00 0.580311 2.117123];... 
[2.380 521.00 0.572809 2.138225];... 
[2.375 522.00 0.562755 2.154438];... 
[2.371 523.00 0.554375 2.167739];... 
[2.366 524.00 0.545645 2.183308];...
[2.362 525.00 0.535852 2.199821];... 
[2.357 526.00 0.527059 2.215241];... 
[2.353 527.00 0.518679 2.228434];...
[2.348 528.00 0.512999 2.241984];... 
[2.344 529.00 0.504889 2.253566];... 
[2.339 530.00 0.497041 2.269471];... 
[2.335 531.00 0.489981 2.284963];... 
[2.331 532.00 0.483728 2.300141];... 
[2.326 533.00 0.474575 2.314838];... 
[2.322 534.00 0.470269 2.330123];... 
[2.318 535.00 0.463890 2.344642];... 
[2.313 536.00 0.458242 2.357497];... 
[2.309 537.00 0.452369 2.371128];... 
[2.305 538.00 0.447667 2.388256];... 
[2.300 539.00 0.440701 2.399820];... 
[2.296 540.00 0.435879 2.413273];... 
[2.292 541.00 0.430834 2.426745];... 
[2.288 542.00 0.426859 2.437803];... 
[2.283 543.00 0.420608 2.452195];... 
[2.279 544.00 0.416152 2.466743];... 
[2.275 545.00 0.411381 2.478523];... 
[2.271 546.00 0.407034 2.493859];... 
[2.267 547.00 0.403116 2.506907];... 
[2.263 548.00 0.399317 2.522195];... 
[2.259 549.00 0.395405 2.533874];... 
[2.254 550.00 0.388214 2.551338];... 
[2.250 551.00 0.384985 2.561694];... 
[2.246 552.00 0.380549 2.578089];... 
[2.242 553.00 0.376800 2.591675];... 
[2.238 554.00 0.374070 2.605248];...
[2.234 555.00 0.369807 2.617803];... 
[2.230 556.00 0.366279 2.629261];... 
[2.226 557.00 0.361413 2.641695];... 
[2.222 558.00 0.357111 2.654864];... 
[2.218 559.00 0.355489 2.665419];... 
[2.214 560.00 0.350450 2.678368];... 
[2.210 561.00 0.348594 2.690380];... 
[2.206 562.00 0.345763 2.702805];... 
[2.202 563.00 0.344789 2.715993];... 
[2.198 564.00 0.341387 2.729949];... 
[2.195 565.00 0.338171 2.741493];... 
[2.191 566.00 0.333157 2.752770];... 
[2.187 567.00 0.331961 2.763918];... 
[2.183 568.00 0.328759 2.777848];... 
[2.179 569.00 0.327322 2.793328];... 
[2.175 570.00 0.322714 2.803717];... 
[2.171 571.00 0.321045 2.817095];... 
[2.168 572.00 0.317551 2.827938];... 
[2.164 573.00 0.314706 2.844317];... 
[2.160 574.00 0.311691 2.855312];... 
[2.156 575.00 0.309222 2.866065];... 
[2.153 576.00 0.304238 2.879656];... 
[2.149 577.00 0.303712 2.891499];... 
[2.145 578.00 0.301669 2.899828];... 
[2.141 579.00 0.298419 2.910320];... 
[2.138 580.00 0.295411 2.921623];... 
[2.134 581.00 0.294075 2.932979];... 
[2.130 582.00 0.293127 2.943047];... 
[2.127 583.00 0.290237 2.955236];... 
[2.123 584.00 0.290037 2.966244];... 
[2.120 585.00 0.288510 2.978746];... 
[2.116 586.00 0.283763 2.989273];... 
[2.112 587.00 0.282330 3.001693];... 
[2.109 588.00 0.281616 3.012581];... 
[2.105 589.00 0.280961 3.027651];... 
[2.102 590.00 0.275382 3.037069];... 
[2.098 591.00 0.274711 3.047540];... 
[2.094 592.00 0.270655 3.059783];... 
[2.091 593.00 0.271515 3.067625];... 
[2.087 594.00 0.268723 3.084897];... 
[2.084 595.00 0.267457 3.094033];... 
[2.080 596.00 0.266088 3.103324];... 
[2.077 597.00 0.262394 3.115880];... 
[2.073 598.00 0.260439 3.127023];... 
[2.070 599.00 0.257502 3.137481];... 
[2.067 600.00 0.257530 3.148729];... 
[2.063 601.00 0.254807 3.159875];... 
[2.060 602.00 0.251282 3.171539];... 
[2.056 603.00 0.252606 3.181605];... 
[2.053 604.00 0.248937 3.190853];... 
[2.049 605.00 0.245569 3.201313];... 
[2.046 606.00 0.246515 3.215434];... 
[2.043 607.00 0.243281 3.221170];... 
[2.039 608.00 0.243721 3.232858];... 
[2.036 609.00 0.240863 3.241942];... 
[2.033 610.00 0.239200 3.253121];... 
[2.029 611.00 0.237076 3.267163];... 
[2.026 612.00 0.236673 3.279275];... 
[2.023 613.00 0.234164 3.290091];... 
[2.019 614.00 0.231621 3.299918];... 
[2.016 615.00 0.232449 3.311861];... 
[2.013 616.00 0.230734 3.321350];... 
[2.010 617.00 0.225837 3.329295];... 
[2.006 618.00 0.227348 3.340760];...
[2.003 619.00 0.223024 3.351846];... 
[2.000 620.00 0.224788 3.362431];... 
[1.997 621.00 0.223511 3.368997];... 
[1.993 622.00 0.219493 3.382350];... 
[1.990 623.00 0.224078 3.390975];... 
[1.987 624.00 0.219844 3.399506];... 
[1.984 625.00 0.217044 3.410813];... 
[1.981 626.00 0.216434 3.419421];... 
[1.978 627.00 0.217411 3.428201];... 
[1.974 628.00 0.215620 3.441248];... 
[1.971 629.00 0.213193 3.456316];... 
[1.968 630.00 0.209530 3.466627];... 
[1.965 631.00 0.209542 3.474260];... 
[1.962 632.00 0.206028 3.484845];... 
[1.959 633.00 0.206790 3.497732];...
[1.956 634.00 0.203841 3.504128];... 
[1.953 635.00 0.205855 3.512858];... 
[1.950 636.00 0.203769 3.530010];... 
[1.947 637.00 0.200818 3.532797];... 
[1.943 638.00 0.204074 3.546462];... 
[1.940 639.00 0.202214 3.554164];... 
[1.937 640.00 0.197087 3.563407];... 
[1.934 641.00 0.197527 3.572345];... 
[1.931 642.00 0.193942 3.576513];... 
[1.928 643.00 0.198386 3.594846];... 
[1.925 644.00 0.194914 3.606684];... 
[1.922 645.00 0.196531 3.613454];... 
[1.919 646.00 0.190462 3.625293];... 
[1.916 647.00 0.193959 3.631903];... 
[1.913 648.00 0.189086 3.642803];... 
[1.911 649.00 0.191849 3.653893];... 
[1.908 650.00 0.192610 3.664268];... 
[1.905 651.00 0.186065 3.672255];... 
[1.902 652.00 0.184510 3.686883];... 
[1.899 653.00 0.183782 3.695544];... 
[1.896 654.00 0.186451 3.707954];... 
[1.893 655.00 0.183397 3.713734];... 
[1.890 656.00 0.185383 3.727188];...
[1.887 657.00 0.182224 3.738469];... 
[1.884 658.00 0.176249 3.747513];... 
[1.882 659.00 0.178932 3.756946];... 
[1.879 660.00 0.178915 3.761751];... 
[1.876 661.00 0.179987 3.777913];... 
[1.873 662.00 0.175988 3.784384];... 
[1.870 663.00 0.177228 3.794509];... 
[1.867 664.00 0.175769 3.807612];... 
[1.865 665.00 0.174209 3.816823];... 
[1.862 666.00 0.171048 3.822455];... 
[1.859 667.00 0.171596 3.830573];... 
[1.856 668.00 0.173436 3.842074];... 
[1.853 669.00 0.170383 3.854170];... 
[1.851 670.00 0.169338 3.858789];... 
[1.848 671.00 0.169298 3.870112];... 
[1.845 672.00 0.174364 3.884445];... 
[1.842 673.00 0.171478 3.893698];... 
[1.840 674.00 0.166808 3.908215];... 
[1.837 675.00 0.165783 3.913102];... 
[1.834 676.00 0.168923 3.924663];... 
[1.832 677.00 0.166458 3.934153];... 
[1.829 678.00 0.161219 3.947047];... 
[1.826 679.00 0.167394 3.951577];... 
[1.823 680.00 0.159737 3.962788];... 
[1.821 681.00 0.168204 3.969158];... 
[1.818 682.00 0.160549 3.984203];... 
[1.815 683.00 0.162326 3.992988];... 
[1.813 684.00 0.163734 3.997588];... 
[1.810 685.00 0.158800 4.004302];... 
[1.807 686.00 0.157024 4.015259];... 
[1.805 687.00 0.159951 4.027359];... 
[1.802 688.00 0.160385 4.040764];... 
[1.800 689.00 0.160229 4.045763];... 
[1.797 690.00 0.160427 4.051242];... 
[1.794 691.00 0.157699 4.065629];... 
[1.792 692.00 0.159985 4.072367];... 
[1.789 693.00 0.161655 4.078894];... 
[1.787 694.00 0.157633 4.092705];... 
[1.784 695.00 0.157910 4.105058];... 
[1.782 696.00 0.156232 4.111215];... 
[1.779 697.00 0.153133 4.125681];... 
[1.776 698.00 0.156074 4.132669];... 
[1.774 699.00 0.157629 4.136887];... 
[1.771 700.00 0.153497 4.153078];... 
[1.769 701.00 0.160290 4.162845];... 
[1.766 702.00 0.151914 4.171862];... 
[1.764 703.00 0.150128 4.170959];... 
[1.761 704.00 0.153322 4.191521];... 
[1.759 705.00 0.157929 4.189008];... 
[1.756 706.00 0.145832 4.195519];... 
[1.754 707.00 0.154993 4.210654];... 
[1.751 708.00 0.159755 4.224669];... 
[1.749 709.00 0.150536 4.239537];... 
[1.746 710.00 0.146312 4.240366];... 
[1.744 711.00 0.164909 4.254405];... 
[1.741 712.00 0.160179 4.262927];... 
[1.739 713.00 0.155788 4.269157];... 
[1.737 714.00 0.161875 4.279467];... 
[1.734 715.00 0.146716 4.283455];... 
[1.732 716.00 0.143560 4.295346];... 
[1.729 717.00 0.144965 4.300545];... 
[1.727 718.00 0.155382 4.323688];... 
[1.725 719.00 0.155639 4.329686];... 
[1.722 720.00 0.149287 4.334716];... 
[1.720 721.00 0.151999 4.345601];... 
[1.717 722.00 0.148130 4.360134];... 
[1.715 723.00 0.159144 4.361280];... 
[1.713 724.00 0.154360 4.362339];... 
[1.710 725.00 0.151218 4.380526];... 
[1.708 726.00 0.138939 4.393470];... 
[1.706 727.00 0.148207 4.402582];... 
[1.703 728.00 0.145026 4.403858];... 
[1.701 729.00 0.152401 4.405949];... 
[1.699 730.00 0.147495 4.433494];... 
[1.696 731.00 0.161894 4.428474];... 
[1.694 732.00 0.148437 4.443736];... 
[1.692 733.00 0.147360 4.446643];... 
[1.689 734.00 0.157855 4.449702];... 
[1.687 735.00 0.157061 4.460729];... 
[1.685 736.00 0.146474 4.474935];... 
[1.682 737.00 0.155803 4.480049];... 
[1.680 738.00 0.144358 4.493005];... 
[1.678 739.00 0.171011 4.493769];... 
[1.676 740.00 0.143190 4.507913];... 
[1.673 741.00 0.144871 4.524017];... 
[1.671 742.00 0.157946 4.513075];... 
[1.669 743.00 0.140970 4.527813];... 
[1.667 744.00 0.152090 4.539129];... 
[1.664 745.00 0.150365 4.543836];... 
[1.662 746.00 0.159452 4.560083];... 
[1.660 747.00 0.158825 4.556755];... 
[1.658 748.00 0.151103 4.566083];... 
[1.655 749.00 0.156594 4.579892];... 
[1.653 750.00 0.146798 4.586127];... 
[1.651 751.00 0.153646 4.604581];... 
[1.649 752.00 0.172929 4.622896];... 
[1.647 753.00 0.147824 4.619045];... 
[1.644 754.00 0.166068 4.625078];... 
[1.642 755.00 0.159833 4.627674];... 
[1.640 756.00 0.163153 4.633606];... 
[1.638 757.00 0.166420 4.646453];... 
[1.636 758.00 0.156807 4.654216];... 
[1.634 759.00 0.156620 4.654689];... 
[1.631 760.00 0.165704 4.673548];... 
[1.629 761.00 0.158549 4.693209];... 
[1.627 762.00 0.160804 4.692290];... 
[1.625 763.00 0.160317 4.700821];... 
[1.623 764.00 0.167775 4.713758];... 
[1.621 765.00 0.159241 4.712398];... 
[1.619 766.00 0.175359 4.718823];... 
[1.617 767.00 0.164167 4.718728];... 
[1.614 768.00 0.153628 4.737121];...
[1.612 769.00 0.164405 4.746668];... 
[1.610 770.00 0.179060 4.740813];... 
[1.608 771.00 0.159112 4.755075];... 
[1.606 772.00 0.160428 4.765811];... 
[1.604 773.00 0.156360 4.777171];... 
[1.602 774.00 0.154334 4.791327];... 
[1.600 775.00 0.172501 4.780602];... 
[1.598 776.00 0.156002 4.809043];... 
[1.596 777.00 0.164268 4.810368];... 
[1.594 778.00 0.186337 4.805020];... 
[1.592 779.00 0.154670 4.815806];... 
[1.590 780.00 0.154980 4.806052];... 
[1.588 781.00 0.172074 4.846431];... 
[1.586 782.00 0.160533 4.823400];... 
[1.584 783.00 0.161487 4.853376];... 
[1.582 784.00 0.150319 4.854792];... 
[1.580 785.00 0.180243 4.865839];... 
[1.578 786.00 0.159462 4.886735];... 
[1.576 787.00 0.163483 4.874329];... 
[1.574 788.00 0.146676 4.901243];... 
[1.572 789.00 0.178780 4.903180];... 
[1.570 790.00 0.184696 4.921706];... 
[1.568 791.00 0.185515 4.924265];... 
[1.566 792.00 0.157371 4.924045];... 
[1.564 793.00 0.189697 4.921422];... 
[1.562 794.00 0.149380 4.955803];... 
[1.560 795.00 0.147878 4.957354];... 
[1.558 796.00 0.157749 4.970846];... 
[1.556 797.00 0.152233 4.986849];... 
[1.554 798.00 0.146987 4.973865];... 
[1.552 799.00 0.167440 4.972242];... 
[1.550 800.00 0.161282 5.007711];... 
[1.000 805.00 0.1600   5.0397];...
[1.00  810.00 0.1600   5.0794];...
[1.00  815.00 0.1600   5.119];...
[1.000 820.0  0.1600   5.15 ];...  
[1.000 825.00 0.1600   5.198];...
[1.00  830.0  0.1600   5.238];...
[1.00  835.00 0.1600   5.277];...
[1.00  840.00 0.1600   5.317];...
[1.00  845.00 0.1600   5.357];...
[1.000 850.0  0.1600   5.39]];

lld=ldn(:,2)/1000;nn=ldn(:,3)+i*ldn(:,4);


if isempty(ld);n=[];else;n=interp1(lld,nn,ld,method,NaN*(1+1i));end;
bornes=[min(lld),max(lld)];ref=['  guillaume sat'];
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [n,bornes,ref,ldr,nr,ldi,ni]=indice_jt(ld,parm,method);
% indice de l'OR  transmis par Jean Taboury


%M.C. Theye, Phys. Rev. B2, 3060 (1970)						
	
	ldnr1=[[.5166	0.608];...	
	[.5391	0.402];...	
	[.5636	0.306];...	%[.5904	0.236];...	[.6199	0.194];...	
	[.6526	0.166];...	
	[.6888	0.16];...
	[.7293	0.164];...	
	[.7749	0.174];...	
	[.8266	0.188];...	
	[.8856	0.21];...	
	[.9537	0.236]];
							
	ldni1=[[.5166	   2.12];...							
	[.5391	   2.54];...								
 	[.5636	   2.88];...%[.5904	[.6199								
 	[.6526	   3.15];...								
	[.6888	   3.8];...								
	[.7293	   4.35];...								
	[.7749	   4.86];...								
	[.8266	   5.39];...								
	[.8856	   5.88];...								
	[.9537	   6.47]];							
%P.B. Johnson. R.W. Christy. Phys. Rev. B6. 4370 (1972)
				

	ldnr2=[[.521		0.62];...
	[.549		0.43];...					
	[.582		0.29];...					
	[.617		0.21];...					
	[.660		0.14];...					
	[.705		0.13];...					
	[.757		0.14];...					
	[.822		0.16];...					
	[.893		0.17];...					
	[.985		0.22]];				


	ldni2=[[.521		   2.081];...							
	[.549		   2.455];...							
	[.582		   2.863];...							
	[.617		   3.272];...							
	[.660		   3.697];...							
	[.705		   4.103];...							
	[.757		   4.542];...							
	[.822		   5.083];...							
	[.893		   5.663];...							
	[.985		   6.35]];						
%K. Weiss. Z. Naturforscher 3a. 143 (1948)
						


	ldnr3=[[.500			0.84];...				
	[.550			0.331];...				
	[.600			0.2	];...			
	[.650			0.142];...
	[.700			0.131];...
	[.750			0.14];...
	[.800			0.149];...
	[.850			0.157];...
	[.900			0.166];...					
	[.950			0.174]];			


	ldni3=[[.500			   1.84];...						
	[.550			   2.324];...						
	[.600			   2.897];...						
	[.650			   3.374];...						
	[.700			   3.842];...						
	[.750			   4.266];...						
	[.800			   4.654];...						
	[.850			   4.993];...						
	[.900			   5.335];...						
	[.950			   5.691]];						
%http://corndog.chem.wisc.edu/fresnel/audata.txt					(Corn)				0.004998306

									
ldnr4=[[0.55		0.421	];...
[0.56				0.3788  ];...
[0.58				0.3046	];...
[0.6	   			0.2438  ];...
[0.62				0.1963  ];...
[0.63				0.1776  ];...
[0.64				0.1622  ];...
[0.65				0.1501  ];...
[0.66				0.1387  ];...
[0.67				0.1372  ];...
[0.68				0.136   ];...
[0.7	   			0.1347  ];...
[0.72				0.1347  ];...
[0.74				0.136   ];...
[0.76				0.1385	];...
[0.78				0.142	];...
[0.8  				0.1465	];...
[0.82				0.1519	];...
[0.84				0.1581	];...
[0.86				0.165	];...
[0.88				0.1726	];...
[0.9	  			0.1807	];...
[0.92				0.1893	];...
[0.94				0.1984	];...
[0.96				0.2078  ];...
[0.98				0.2176  ]];


	ldni4=[[.550		   2.5059	   ];...				
	[.551				   2.518	   ];...				
	[.552				   2.53	       ];...				
	[.553				   2.5419	   ];...				
	[.554				   2.5539	   ];...				
	[.555				   2.5658	   ];...				
	[.556				   2.5777	   ];...				
	[.557				   2.5896	   ];...				
	[.558				   2.6014	   ];...				
	[.559				   2.6132	   ];...				
	[.560				   2.625	   ];...				
	[.561				   2.6368	   ];...				
	[.562				   2.6485	   ];...				
	[.563				   2.6603	   ];...				
	[.564				   2.6719	   ];...				
	[.565				   2.6836	   ];...				
	[.566				   2.6952	   ];...				
	[.567				   2.7068	   ];...				
	[.568				   2.7184	   ];...				
	[.569				   2.73	       ];...				
	[.570				   2.7415	   ];...				
	[.571				   2.753	   ];...				
	[.572				   2.7645	   ];...				
	[.573				   2.776	   ];...				
	[.574				   2.7874	   ];...				
	[.575				   2.7988	   ];...				
	[.576				   2.8102	   ];...				
	[.577				   2.8215	   ];...				
	[.578				   2.8329	   ];...				
	[.579				   2.8442	   ];...				
	[.580				   2.8555	   ];...				
	[.581				   2.8667	   ];...				
	[.582				   2.878	   ];...				
	[.583				   2.8892	   ];...				
	[.584				   2.9004	   ];...				
	[.585				   2.9115	   ];...				
	[.586				   2.9227	   ];...				
	[.587				   2.9338	   ];...				
	[.588				   2.9449	   ];...				
	[.589				   2.956	   ];...				
	[.590				   2.967	   ];...				
	[.591				   2.978	   ];...				
	[.592				   2.989	   ];...				
	[.593				   3	       ];...			
	[.594				   3.011	   ];...				
	[.595				   3.0219	   ];...				
	[.596				   3.0328	   ];...				
	[.597				   3.0437	   ];...				
	[.598				   3.0546	   ];...				
	[.599				   3.0654	   ];...				
	[.600				   3.0763	   ];...				
	[.601				   3.0871	   ];...				
	[.602				   3.0978	   ];...				
	[.603				   3.1086	   ];...				
	[.604				   3.1193	   ];...				
	[.605				   3.13	       ];...				
	[.606				   3.1407	   ];...				
	[.607				   3.1514	   ];...				
	[.608				   3.1621	   ];...				
	[.609				   3.1727	   ];...				
	[.610				   3.1833	   ];...				
	[.611				   3.1939	   ];...				
	[.612				   3.2045	   ];...				
	[.613				   3.215	   ];...				
	[.614				   3.2255	   ];...				
	[.615				   3.236	   ];...				
	[.616				   3.2465	   ];...				
	[.617				   3.257	   ];...				
	[.618				   3.2674	   ];...				
	[.619				   3.2779	   ];...				
	[.620				   3.2883	   ];...				
	[.621				   3.2986	   ];...				
	[.622				   3.309	   ];...				
	[.623				   3.3194	   ];...				
	[.624				   3.3297	   ];...				
	[.625				   3.34	       ];...				
	[.626				   3.3503	   ];...				
	[.627				   3.3605	   ];...				
	[.628				   3.3708	   ];...				
	[.629				   3.381	   ];...				
	[.630				   3.3912	   ];...				
	[.631				   3.4014	   ];...				
	[.632				   3.4116	   ];...				
	[.633				   3.4218	   ];...				
	[.634				   3.4319	   ];...				
	[.635				   3.442	   ];...				
	[.636				   3.4521	   ];...				
	[.637				   3.4622	   ];...				
	[.638				   3.4723	   ];...				
	[.639				   3.4823	   ];...				
	[.640				   3.4923	   ];...				
	[.641				   3.5023	   ];...				
	[.642				   3.5123	   ];...				
	[.643				   3.5223	   ];...				
	[.644				   3.5323	   ];...				
	[.645				   3.5422	   ];...				
	[.646				   3.5521	   ];...				
	[.647				   3.562	   ];...				
	[.648				   3.5719	   ];...				
	[.649				   3.5818	   ];...				
	[.650				   3.5916	   ];...				
	[.651				   3.6014	   ];...				
	[.652				   3.6113	   ];...				
	[.653				   3.6211	   ];...				
	[.654				   3.6308	   ];...				
	[.655				   3.6406	   ];...				
	[.656				   3.6504	   ];...				
	[.657				   3.6601	   ];...				
	[.658				   3.6698	   ];...				
	[.659				   3.6795	   ];...				
	[.660				   3.6892	   ];...				
	[.661				   3.6989	   ];...				
	[.662				   3.7085	   ];...				
	[.663				   3.7182	   ];...				
	[.664				   3.7278	   ];...				
	[.665				   3.7374	   ];...				
	[.666				   3.747	   ];...				
	[.667				   3.7565	   ];...				
	[.668				   3.7661	   ];...				
	[.669				   3.7756	   ];...				
	[.670				   3.7852	   ];...				
	[.671				   3.7947	   ];...				
	[.672				   3.8042	   ];...				
	[.673				   3.8137	   ];...				
	[.674				   3.8231	   ];...				
	[.675				   3.8326	   ];...				
	[.676				   3.842	   ];...				
	[.677				   3.8514	   ];...				
	[.678				   3.8608	   ];...				
	[.679				   3.8702	   ];...				
	[.680				   3.8796	   ];...				
	[.681				   3.889	   ];...				
	[.682				   3.8983	   ];...				
	[.683				   3.9077	   ];...				
	[.684				   3.917	   ];...				
	[.685				   3.9263	   ];...				
	[.686				   3.9356	   ];...				
	[.687				   3.9449	   ];...				
	[.688				   3.9541	   ];...				
	[.689				   3.9634	   ];...				
	[.690				   3.9726	   ];...				
	[.691				   3.9818	   ];...				
	[.692				   3.9911	   ];...				
	[.693				   4.0003	   ];...				
	[.694				   4.0094	   ];...				
	[.695				   4.0186	   ];...				
	[.696				   4.0278	   ];...				
	[.697				   4.0369	   ];...				
	[.698				   4.046	   ];...				
	[.699				   4.0552	   ];...				
	[.700				   4.0643	   ];...				
	[.701				   4.0734	   ];...				
	[.702				   4.0824	   ];...				
	[.703				   4.0915	   ];...				
	[.704				   4.1006	   ];...				
	[.705				   4.1096	   ];...				
	[.706				   4.1186	   ];...				
	[.707				   4.1276	   ];...				
	[.708				   4.1366	   ];...				
	[.709				   4.1456	   ];...				
	[.710				   4.1546	   ];...				
	[.711				   4.1636	   ];...				
	[.712				   4.1725	   ];...				
	[.713				   4.1815	   ];...				
	[.714				   4.1904	   ];...				
	[.715				   4.1993	   ];...				
	[.716				   4.2082	   ];...				
	[.717				   4.2171	   ];...				
	[.718				   4.226	   ];...				
	[.719				   4.2349	   ];...				
	[.720				   4.2438	   ];...				
	[.721				   4.2526	   ];...				
	[.722				   4.2614	   ];...				
	[.723				   4.2703	   ];...				
	[.724				   4.2791	   ];...				
	[.725				   4.2879	   ];...				
	[.726				   4.2967	   ];...				
	[.727				   4.3055	   ];...				
	[.728				   4.3142	   ];...				
	[.729				   4.323	   ];...				
	[.730				   4.3318	   ];...				
	[.731				   4.3405	   ];...				
	[.732				   4.3492	   ];...				
	[.733				   4.3579	   ];...				
	[.734				   4.3667	   ];...				
	[.735				   4.3754	   ];...				
	[.736				   4.384	   ];...				
	[.737				   4.3927	   ];...				
	[.738				   4.4014	   ];...				
	[.739				   4.41	       ];...				
	[.740				   4.4187	   ];...				
	[.741				   4.4273	   ];...				
	[.742				   4.436	   ];...				
	[.743				   4.4446	   ];...				
	[.744				   4.4532	   ];...				
	[.745				   4.4618	   ];...				
	[.746				   4.4704	   ];...				
	[.747				   4.479	   ];...				
	[.748				   4.4875	   ];...				
	[.749				   4.4961	   ];...				
	[.750				   4.5046	   ];...				
	[.751				   4.5132	   ];...				
	[.752				   4.5217	   ];...				
	[.753				   4.5302	   ];...				
	[.754				   4.5387	   ];...				
	[.755				   4.5472	   ];...				
	[.756				   4.5557	   ];...				
	[.757				   4.5642	   ];...				
	[.758				   4.5727	   ];...				
	[.759				   4.5812	   ];...				
	[.760				   4.5896	   ];...				
	[.761				   4.5981	   ];...				
	[.762				   4.6065	   ];...				
	[.763				   4.615	   ];...				
	[.764				   4.6234	   ];...				
	[.765				   4.6318	   ];...				
	[.766				   4.6402	   ];...				
	[.767				   4.6486	   ];...				
	[.768				   4.657	   ];...				
	[.769				   4.6654	   ];...				
	[.770				   4.6738	   ];...				
	[.771				   4.6821	   ];...				
	[.772				   4.6905	   ];...				
	[.773				   4.6988	   ];...				
	[.774				   4.7072	   ];...				
	[.775				   4.7155	   ];...				
	[.776				   4.7238	   ];...				
	[.777				   4.7322	   ];...				
	[.778				   4.7405	   ];...				
	[.779				   4.7488	   ];...				
	[.780				   4.7571	   ];...				
	[.781				   4.7654	   ];...				
	[.782				   4.7737	   ];...				
	[.783				   4.7819	   ];...				
	[.784				   4.7902	   ];...				
	[.785				   4.7985	   ];...				
	[.786				   4.8067	   ];...				
	[.787				   4.815	   ];...				
	[.788				   4.8232	   ];...				
	[.789				   4.8314	   ];...				
	[.790				   4.8396	   ];...				
	[.791				   4.8479	   ];...				
	[.792				   4.8561	   ];...				
	[.793				   4.8643	   ];...				
	[.794				   4.8725	   ];...				
	[.795				   4.8807	   ];...				
	[.796				   4.8888	   ];...				
	[.797				   4.897	   ];...				
	[.798				   4.9052	   ];...				
	[.799				   4.9134	   ];...				
	[.800				   4.9215	   ];...				
	[.801				   4.9297	   ];...				
	[.802				   4.9378	   ];...				
	[.803				   4.9459	   ];...				
	[.804				   4.9541	   ];...				
	[.805				   4.9622	   ];...				
	[.806				   4.9703	   ];...				
	[.807				   4.9784	   ];...				
	[.808				   4.9865	   ];...				
	[.809				   4.9946	   ];...				
	[.810				   5.0027	   ];...				
	[.811				   5.0108	   ];...				
	[.812				   5.0189	   ];...				
	[.813				   5.027	   ];...				
	[.814				   5.035	   ];...				
	[.815				   5.0431	   ];...				
	[.816				   5.0512	   ];...				
	[.817				   5.0592	   ];...				
	[.818				   5.0673	   ];...				
	[.819				   5.0753	   ];...				
	[.820				   5.0833	   ];...				
	[.821				   5.0914	   ];...				
	[.822				   5.0994	   ];...				
	[.823				   5.1074	   ];...				
	[.824				   5.1154	   ];...				
	[.825				   5.1235	   ];...				
	[.826				   5.1315	   ];...				
	[.827				   5.1395	   ];...				
	[.828				   5.1474	   ];...				
	[.829				   5.1554	   ];...				
	[.830				   5.1634	   ];...				
	[.831				   5.1714	   ];...				
	[.832				   5.1794	   ];...				
	[.833				   5.1873	   ];...				
	[.834				   5.1953	   ];...				
	[.835				   5.2033	   ];...				
	[.836				   5.2112	   ];...				
	[.837				   5.2192	   ];...				
	[.838				   5.2271	   ];...				
	[.839				   5.2351	   ];...				
	[.840				   5.243	   ];...				
	[.841				   5.2509	   ];...				
	[.842				   5.2589	   ];...				
	[.843				   5.2668	   ];...				
	[.844				   5.2747	   ];...				
	[.845				   5.2826	   ];...				
	[.846				   5.2905	   ];...				
	[.847				   5.2984	   ];...				
	[.848				   5.3063	   ];...				
	[.849				   5.3142	   ];...				
	[.850				   5.3221	   ];...				
	[.851				   5.33	       ];...				
	[.852				   5.3379	   ];...				
	[.853				   5.3458	   ];...				
	[.854				   5.3537	   ];...				
	[.855				   5.3615	   ];...				
	[.856				   5.3694	   ];...				
	[.857				   5.3773	   ];...				
	[.858				   5.3851	   ];...				
	[.859				   5.393	   ];...				
	[.860				   5.4008	   ];...				
	[.861				   5.4087	   ];...				
	[.862				   5.4165	   ];...				
	[.863				   5.4244	   ];...				
	[.864				   5.4322	   ];...				
	[.865				   5.44	       ];...				
	[.866				   5.4479	   ];...				
	[.867				   5.4557	   ];...				
	[.868				   5.4635	   ];...				
	[.869				   5.4713	   ];...				
	[.870				   5.4792	   ];...				
	[.871				   5.487	   ];...				
	[.872				   5.4948	   ];...				
	[.873				   5.5026	   ];...				
	[.874				   5.5104	   ];...				
	[.875				   5.5182	   ];...				
	[.876				   5.526	   ];...				
	[.877				   5.5338	   ];...				
	[.878				   5.5416	   ];...				
	[.879				   5.5494	   ];...				
	[.880				   5.5572	   ];...				
	[.881				   5.5649	   ];...				
	[.882				   5.5727	   ];...				
	[.883				   5.5805	   ];...				
	[.884				   5.5883	   ];...				
	[.885				   5.5961	   ];...				
	[.886				   5.6038	   ];...				
	[.887				   5.6116	   ];...				
	[.888				   5.6194	   ];...				
	[.889				   5.6271	   ];...				
	[.890				   5.6349	   ];...				
	[.891				   5.6426	   ];...				
	[.892				   5.6504	   ];...				
	[.893				   5.6581	   ];...				
	[.894				   5.6659	   ];...				
	[.895				   5.6736	   ];...				
	[.896				   5.6814	   ];...				
	[.897				   5.6891	   ];...				
	[.898				   5.6968	   ];...				
	[.899				   5.7046	   ];...				
	[.900				   5.7123	   ];...				
	[.901				   5.72	       ];...				
	[.902				   5.7278	   ];...				
	[.903				   5.7355	   ];...				
	[.904				   5.7432	   ];...				
	[.905				   5.7509	   ];...				
	[.906				   5.7587	   ];...				
	[.907				   5.7664	   ];...				
	[.908				   5.7741	   ];...				
	[.909				   5.7818	   ];...				
	[.910				   5.7895	   ];...				
	[.911				   5.7972	   ];...				
	[.912				   5.8049	   ];...				
	[.913				   5.8126	   ];...				
	[.914				   5.8203	   ];...				
	[.915				   5.828	   ];...				
	[.916				   5.8357	   ];...				
	[.917				   5.8434	   ];...				
	[.918				   5.8511	   ];...				
	[.919				   5.8588	   ];...				
	[.920				   5.8665	   ];...				
	[.921				   5.8742	   ];...				
	[.922				   5.8819	   ];...				
	[.923				   5.8896	   ];...				
	[.924				   5.8973	   ];...				
	[.925				   5.905	   ];...				
	[.926				   5.9126	   ];...				
	[.927				   5.9203	   ];...				
	[.928				   5.928	   ];...				
	[.929				   5.9357	   ];...				
	[.930				   5.9434	   ];...				
	[.931				   5.951	   ];...				
	[.932				   5.9587	   ];...				
	[.933				   5.9664	   ];...				
	[.934				   5.9741	   ];...				
	[.935				   5.9817	   ];...				
	[.936				   5.9894	   ];...				
	[.937				   5.9971	   ];...				
	[.938				   6.0047	   ];...				
	[.939				   6.0124	   ];...				
	[.940				   6.0201	   ];...				
	[.941				   6.0277	   ];...				
	[.942				   6.0354	   ];...				
	[.943				   6.043	   ];...				
	[.944				   6.0507	   ];...				
	[.945				   6.0584	   ];...				
	[.946				   6.066	   ];...				
	[.947				   6.0737	   ];...				
	[.948				   6.0813	   ];...				
	[.949				   6.089	   ];...				
	[.950				   6.0966	   ]];	
		
f=find(ismember(ldnr4(:,1),ldni4(:,1)));ldnr44=ldnr4(f,:);
f=find(ismember(ldni4(:,1),ldnr44(:,1)));ldni44=ldni4(f,:);


%http://www.sopra   sa.com/index2.htm
						
	ldnr5=[[.991912		   			    0.253125];...
	[.9537615385					0.236];...
	[.918437037					    0.222125];...
	[.8856357143					0.21];...
	[.8550965517					0.198125];...
	[.8265933333					0.188];...
	[.7999290323					0.181375];...
	[.77493125				    	0.176];...
	[.7514484848					0.1695];...
	[.7293470588					0.164];...
	[.7085085714					0.160875];...
	[.6888277778					0.16];...
	[.6702108108					0.161];...
	[.6525736842					0.166];...
	[.6358410256					0.17775];...
	[.619945					    0.194];...
	[.6048243902					0.212375];...
	[.5904238095					0.236];...
	[.5766930233					0.267625];...
	[.5635863636					0.306];...
	[.5510622222					0.3455];...
	[.5390826087					0.402];...
	[.527612766					    0.49175];...
	[.5166208333					0.608]];




	ldni5=[[.991912				   6.75313];...				
	[.9537615					   6.47];...				
	[.918437					   6.16813];...				
	[.8856357					   5.88];...				
	[.8550966					   5.63125];...				
	[.8265933					   5.39];...				
	[.799929					   5.12581];...				
	[.7749313					   4.86];...				
	[.7514485					   4.61019];...				
	[.7293471					   4.357];...				
	[.7085086					   4.08769];...				
	[.6888278					   3.8];...				
	[.6702108					   3.44581];...				
	[.6525737					   3.15];...				
	[.635841					   3.07];...				
	[.619945					   3.06	];...			
	[.6048244					   3.015];...				
	[.5904238					   2.97];...				
	[.576693					   2.94063];...				
	[.5635864					   2.88];...				
	[.5510622					   2.73063];...				
	[.5390826					   2.54	];...			
	[.5276128					   2.32625];...				
	[.5166208					   2.12]];			

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
switch parm;
case 2.01;lld=ldnr1(:,1);nn=ldnr1(:,2)+i*ldni1(:,2);ref='JT:  OR   M.C. Theye, Phys. Rev. B2, 3060 (1970)	';
case 2.02;lld=ldnr2(:,1);nn=ldnr2(:,2)+i*ldni2(:,2);ref='JT:  OR   P.B. Johnson. R.W. Christy. Phys. Rev. B6. 4370 (1972)	';
case 2.03;lld=ldnr3(:,1);nn=ldnr3(:,2)+i*ldni3(:,2);ref='JT:  OR   K. Weiss. Z. Naturforscher 3a. 143 (1948)	';
case 2.04;lld=ldnr44(:,1);nn=ldnr44(:,2)+i*ldni44(:,2);ref='JT:  OR   Corn	 http://corndog.chem.wisc.edu/fresnel/audata.txt';
case 2.05;lld=ldnr5(:,1);nn=ldnr5(:,2)+i*ldni5(:,2);ref='JT:  OR   http://www.sopra   sa.com/index2.htm';
end;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if parm==2.04;
if isempty(ld);n=[];else;n=interp1(ldnr4(:,1),ldnr4(:,2),ld,method,NaN)+i*interp1(ldni4(:,1),ldni4(:,2),ld,method,NaN);end
ldr=ldnr4(:,1);ldi=ldni4(:,1);nr=ldnr4(:,2);ni=ldni4(:,2);
else;
if isempty(ld);n=[];else;n=interp1(lld,nn,ld,method,NaN*(1+1i));end
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
end;
bornes=[min(lld),max(lld)];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%





%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [indice,bornes,ref,ldr,nr,ldi,ni]= calcul_indice(milieu,lambda,method);
% function [indice,bornes,ref,ldr,nr,ldi,ni]= calcul_indice(milieu,lambda);
%  calcul l' indice en fonction de lambda pour diffrents matriaux
%
% USAGE: [indice,bornes,ref]= calcul_indice(milieu,lambda)
% milieu: chane de caractres donnant le nom du milieu
% lambda: tableau rel donnant les lambda  EN MICRONS o l'indice doit tre calcul
% indice : tableau rel ou complexe de mme taille que lambda retournant les indices
%
% bornes:domaine de validite (quand interploation retourne nan a l'exterieur)
% ref:references(chaine de characteres)
%  ldr,nr,ldi,ni   couples de points en cas d'interpolation
%
% LISTE DES MILIEUX
% 'air'
% 'bio'
% 'verre' % NOTA: Sans dispersion (Cf. BK7, SF11, etc...)
% 'rsine' % NOTA: Sans dispersion 
% 'eauSD' % Sans Dispersion ('eau' pour le programme IDL initial)
% 'eau' % HS!      Coefficients srie de Laurent d'aprs la Glass Database du catalogue "Misc" de OSLO LT 6.1
% 'polypyrrole'
% 'SF11'
% 'SF10'
% 'BK7'
% 'or','Au'
% 'chrome','Cr'
% 'SiO2'
% 'MoSiO2'
% 'Si'
% 'Saphir_no'
% 'Saphir_ne'
% 'Aluminium'
% 'Titane'
% 'Cuivre'
% 'Tungsten'
% 'MgO'
% Une erreur est dclenche si le milieu est inconnu. 
%
% D'aprs le programme IDL "determ_indice_2.pro" de Yan Pailhas (d'aprs les donnes et la
% biblio de Genoptics et de l'quipe d'Yves Levy...)  - mai-juin 2003
%
% d'apres calcul_epsilon  LCFIO - Herv Sauer - 8 juin 2003


indice=ones(size(lambda));lld=[];nn=[];
switch milieu
  
  case 'air';bornes=[0,inf];ref='AIR sans dispersion';
  case 'bio';indice=1.45*indice;bornes=[0,inf];ref='BIO sans dispersion';
  case 'polypyrrole';indice=1.7*indice;bornes=[0,inf];ref='POLYPYRROLE sans dispersion';
  case 'rsine';indice=1.5*indice;bornes=[0,inf];ref='RESINE sans dispersion';
      
  case 'eauSD';indice=1.333*indice;bornes=[0,inf];ref='EAU sans dispersion ';
  case 'eau' % HS!
    bornes=[0,inf];ref='EAU avec dispersion, bornes a preciser,srie de Laurent d''aprs la Glass Database du catalogue "Misc" de OSLO LT 6.1';
    CL=[1.76755000 -0.01507500 0.00350000 0.00061850 -2.820E-5 0];
    % ^ Coefficients srie de Laurent d'aprs la Glass Database du catalogue "Misc" de OSLO LT 6.1
    % v Formule d'aprs l'aide de CodeV 9.20 (coef pour lambda en m)
    lb_um=lambda;
    indice=sqrt(CL(1) + CL(2)*lb_um.^2 + CL(3)*lb_um.^-2 + CL(4)*lb_um.^-4 +CL(5)*lb_um.^-6 +  CL(6)*lb_um.^-8);
    % NOTA: nd=1.333037  d'aprs OSLO LT 6.1
    % ====: Vd=(nd-1)/(nF-nC)=57.07 d'aprs OSLO LT 6.1
    %       d(He): 587.5618nm; F(H): 486.1327nm; C(H): 656.2725nm   d'aprs aide mmoire Schott 1986 
    %  ==> test de la srie ==> rsultat: OK pour nd (1.3330371346...);  OK  pour Vd (57.07271...)
    
  case 'verre'; indice=1.515*indice;bornes=[0,inf];ref='VERRE sans dispersion '; 
      
  case 'SF11';tmp=lambda.^2;bornes=[0,inf];ref='SF11 bornes a preciser';    
    indice=sqrt(1+ 1.73848403*tmp./(tmp-1.36068604e-2)+3.11168974e-1*tmp./(tmp-6.15960463e-2)+1.17490871*tmp./(tmp-1.21922711e+2));
    
  case 'SF10';tmp=lambda.^2;bornes=[0,inf];ref='SF10 bornes a preciser';
    indice=sqrt(1+1.61625977*tmp./(tmp-1.27534559e-2)+2.59229334e-1*tmp./(tmp-5.81983954e-2)+1.07762317*tmp./(tmp-1.16607680e+2));
      
  case 'BK7';tmp=lambda.^2;bornes=[0,inf];ref='BK7 bornes a preciser';
    indice=sqrt(1+ 1.03961212*tmp./(tmp-0.00600069867)+0.231792344*tmp./(tmp-0.0200179144)+1.01046945*tmp./(tmp-103.560653));

  case {'or','Au'}
    bornes=[0.20664,2.479];ref='OR ???????? ';
    % q=1.6022E-19; h=6.6261E-34;c=299792458; V=(h*c/(1.e-6*q))./lambda) en ev   lambda en microns
    V=1.239829488174885./lambda;  
    X_V=linspace(0.5,6.0,111); % energies hv en eV...
    lld=1.239829488174885./X_V;
    nn=[1.205+    15.5i,...
      1.034     +    14.14438i,... l'alternance de donnes  7 (!!!!) chiffres significatifs
      .896      +    13i,...       et  2 ou 3 chiffres significatifs est surprenante!!!!!
      .78525    +    12.03063i,...
      .696      +    11.2i,...
      .621563   +    10.4575i,...
      .559      +    9.81i,...
      .503063   +    9.203125i,...
      .454      +    8.77i,...
      .410188   +    8.757501i,...
      .372      +    8.77i,...
      .339375   +    8.40375i,...
      .312      +    7.93i,...
      .2905     +    7.485001i,...
      .272      +    7.07i,...
      .253125   +    6.753125i,...
      .236      +    6.47i,...
      .222125   +    6.168125i,...
      .21       +    5.88i,...
      .198125   +    5.63125i,...
      .188      +    5.39i,...
      .181375   +    5.125813i,...
      .176      +    4.86i,...
      .1695     +    4.610188i,...
      .164      +    4.357i,...
      .160875   +    4.087687i,...
      .16       +    3.8i,...
      .161      +    3.445812i,...
      .166      +    3.15i,...
      .17775    +    3.07i ,...
      .194      +    3.06i,...
      .212375   +    3.015i,...
      .236      +    2.97i,...
      .267625   +    2.940625i,...
      .306      +    2.88i,...
      .3455     +    2.730625i,...
      .402      +    2.54i,...
      .49175    +    2.32625i,...
      .608      +    2.12i,...
      .7545     +    1.9565i,...
      .916      +    1.84i,...
      1.08675   +    1.797375i,...
      1.242     +    1.796i,...
      1.345875  +    1.814625i,...
      1.426     +    1.846i,...
      1.502125  +    1.875875i,...
      1.562     +    1.904i,...
      1.59625   +    1.9245i,...
      1.616     +    1.94i,...
      1.628     +    1.951375i,...
      1.636     +    1.958i,...
      1.64725   +    1.959375i,...
      1.658     +    1.956i,...
      1.666     +    1.94775i,...
      1.674     +    1.936i,...
      1.68475   +    1.9225i,...
      1.696     +    1.906i,...
      1.705875  +    1.883i,...
      1.716     +    1.862i,...
      1.727625  +    1.852375i,...
      1.74      +    1.848i,...
      1.7525    +    1.84525i,...
      1.766     +    1.846i,...
      1.782     +    1.85175i,...
      1.798     +    1.86i,...
      1.812     +    1.86825i,...
      1.824     +    1.878i,...
      1.83425   +    1.891375i,...
      1.84      +    1.904i,...
      1.837125  +    1.911375i,...
      1.83      +    1.916i,...
      1.822625  +    1.918875i,...
      1.812     +    1.92i,...
      1.795     +    1.920375i,...
      1.776     +    1.918i,...
      1.76      +    1.91i,...
      1.742     +    1.9i,...
      1.7165    +    1.89175i,...
      1.69      +    1.882i,...
      1.668875  +    1.86775i,...
      1.648     +    1.852i,...
      1.623625  +    1.8375i,...
      1.598     +    1.822i,...
      1.5715    +    1.803375i,...
      1.546     +    1.784i,...
      1.522625  +    1.76675i,...
      1.504     +    1.748i,...
      1.49475   +    1.724625i,...
      1.49      +    1.698i,...
      1.4865    +    1.66675i,...
      1.484     +    1.636i,...
      1.481125  +    1.611625i,...
      1.478     +    1.59i,...
      1.474125  +    1.569625i,...
      1.47      +    1.55i,...
      1.466     +    1.5295i,...
      1.462     +    1.51i,...
      1.457625  +    1.49375i,...
      1.454     +    1.478i,...
      1.453125  +    1.4595i,...
      1.452     +    1.442i,...
      1.447125  +    1.429625i,...
      1.442     +    1.418i,...
      1.43975   +    1.403i,...
      1.438     +    1.388i,...
      1.434875  +    1.376i,...
      1.432     +    1.364i,...
      1.431125  +    1.34925i,...
      1.43      +    1.334i,...
      1.427375  +    1.31925i,...
      1.422     +    1.306i];


if isempty(V);indice=[];else;indice=interp1(X_V,nn,V,method,NaN*(1+1i));end; 
% retourne NaN+NaNi en cas d'extrapolation!
case {'SiO2'}   %(extrait de la bibliothque CodeV 9.61)
bornes=[3.7070,.2130];ref='SiO2avec dispersion, (extrait de la bibliothque CodeV 9.61)';
%   lld =[     3.7070   3.3188   2.9306   2.5423   2.1541   1.7659   1.3777    .9894    .6012    .2130];
%   nn=[ 1.39935  1.41111  1.42088  1.42901  1.43574  1.44132  1.44603  1.45055  1.45800  1.53519];
lld =[ 3.7070,3.3188,2.9306,2.5423,2.1541,1.7659,1.3777,.9894,.7,.65,.6,.6012,.55,.5,.45,.4,.35,.3,.25,.2130];
nn=[1.39935,1.41111,1.42088,1.42901,1.43574,1.44132,1.44603,1.45055 ,1.455294,1.456538,1.458041,1.45800,1.459915,1.462331,1.465571,1.470122,1.476896,1.487793,1.507439,1.53519];
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 
% retourne NaN+NaNi en cas d'extrapolation!

case {'Saphir_no'}   %(extrait de la bibliothque CodeV 9.61)
bornes=[1.01398,1.69320];ref='Handbook of Optical Constants of Solids III, Ed. D. Palik (1998) Laurent FREY CEA-G/LETI/DOPT/STCO transmis par P Lalanne';
lld=[1.01398,1.12866,1.36728,1.39506,1.52952,1.69320];
nn=[1.75547,1.75339,1.74936,1.74888,1.74660,1.74368];  
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 
% retourne NaN+NaNi en cas d'extrapolation!

case {'Saphir_ne'}   %(extrait de la bibliothque CodeV 9.61)
bornes=[1.01398,1.69320];ref='Handbook of Optical Constants of Solids III, Ed. D. Palik (1998) Laurent FREY CEA-G/LETI/DOPT/STCO transmis par P Lalanne';
lld=[1.01398,1.12866,1.36728,1.39506,1.52952,1.69320];
nn=[1.74794,1.74549,1.74148,1.74101,1.73874,1.73584];  
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 
% retourne NaN+NaNi en cas d'extrapolation!


case {'chrome','Cr'}
% d'aprs article: ????????
bornes=[0.20664,1.239];ref='CHROME  ???????? ';
% q=1.6022E-19; h=6.6261E-34;c=299792458;V=(h*c/(1.e-6*q))./lambda);% lambda en microns
V=1.239829488174885./lambda;  
X_V=linspace(1,6.0,101); % energies hv en eV...
lld=1.239829488174885./X_V;
    

    nn=[ 3.68 +    3.62i,...
      3.660625  +    3.59125i,... l'alternance de donnes  7 (!!!!) chiffres significatifs
      3.62      +    3.58i,...    et  2 ou 3 chiffres significatifs est surprenante!!!!!
      3.564375  +    3.57875i,...
      3.5       +    3.58i,...
      3.4325    +    3.5725i,...
      3.37      +    3.56i,...
      3.326875  +    3.54125i,...
      3.29      +    3.52i,...
      3.25125   +    3.5i,...
      3.21      +    3.48i,...
      3.158125  +    3.46i,...
      3.11      +    3.44i,...
      3.07875   +    3.419375i,...
      3.06      +    3.4i,...
      3.054375  +    3.385i,...
      3.06      +    3.37i,...
      3.07625   +    3.35i,...
      3.1       +    3.33i,...
      3.129375  +    3.3125i,...
      3.16      +    3.3i,...
      3.188125  +    3.296875i,...
      3.21      +    3.3i,...
      3.225     +    3.309375i,...
      3.22      +    3.32i,...
      3.178125  +    3.32625i,...
      3.11      +    3.33i,...
      3.013125  +    3.3325i,...
      2.91      +    3.33i,...
      2.829375  +    3.3175i,...
      2.75      +    3.3i,...
      2.655     +    3.283125i,...
      2.56      +    3.26i,...
      2.47625   +    3.2225i,...
      2.4       +    3.18i,...
      2.33125   +    3.14i,...
      2.27      +    3.1i,...
      2.2175    +    3.060625i,...
      2.17      +    3.02i,...
      2.123125  +    2.975i,...
      2.08      +    2.93i,...
      2.043125  +    2.889375i,...
      2.01      +    2.85i,...
      1.97875   +    2.808125i,...
      1.95      +    2.77i,...
      1.92375   +    2.7425i,...
      1.9       +    2.72i,...
      1.878125  +    2.699375i,...
      1.86      +    2.68i,...
      1.851875  +    2.660625i,...
      1.84      +    2.64i,...
      1.8075    +    2.615i,...
      1.77      +    2.59i,...
      1.738125  +    2.57i,...
      1.71      +    2.55i,...
      1.68875   +    2.525625i,...
      1.67      +    2.5i,...
      1.650625  +    2.475i,...
      1.63      +    2.45i,...
      1.605     +    2.425i,...
      1.58      +    2.4i,...
      1.559375  +    2.375i,...
      1.54      +    2.35i,...
      1.52      +    2.325625i,...
      1.5       +    2.3i,...
      1.47875   +    2.27i,...
      1.46      +    2.24i,...
      1.448125  +    2.214375i,...
      1.44      +    2.19i,...
      1.43625   +    2.165i,...
      1.43      +    2.14i,...
      1.41      +    2.114375i,...
      1.39      +    2.09i,...
      1.3825    +    2.07i,...
      1.38      +    2.05i,...
      1.38      +    2.025625i,...
      1.38      +    2i,...
      1.375625  +    1.975i,...
      1.37      +    1.95i,...
      1.364375  +    1.925i,...
      1.36      +    1.9i,...
      1.35875   +    1.874375i,...
      1.36      +    1.85i,...
      1.36375   +    1.82875i,...
      1.37      +    1.81i,...
      1.379375  +    1.79375i,...
      1.39      +    1.78i,...
      1.4       +    1.769375i,...
      1.41      +    1.76i,...
      1.42      +    1.749375i,...
      1.43      +    1.74i,...
      1.44      +    1.734375i,...
      1.45      +    1.73i,...
      1.46125   +    1.724375i,...
      1.47      +    1.72i,...
      1.471875  +    1.719375i,...
      1.47      +    1.72i,...
      1.465     +    1.720625i,...
      1.46      +    1.72i,...
      1.4575    +    1.716875i,...
      1.46      +    1.71i];
    if isempty(V);indice=[];else;indice=interp1(X_V,nn,V,method,NaN*(1+1i));end; 
  
    % retourne NaN+NaNi en cas d'extrapolation!
%------------------------------------------------------
	
case {'Aluminium'}
bornes=[0.00415,30.99];ref='ALUMINIUM Savannah.';
% q=1.6022E-19; h=6.6261E-34;c=299792458;V=(h*c/(1.e-6*q))./lambda);% lambda en microns
% % correction d'une erreur (test sur R)  
% 
% 15.200	0.178	0.108	0.4903
% 15.400	0.234	0.084	0.3881
% 15.600	0.280	0.073	0.3182
% 
% au lieu de
% 
% 15.200	0.178	0.108	0.4903
% 15.400	0.234	0.184	0.3881
% 15.600	0.280	0.073	0.3182
% 
% 


AAA=[
0.040	98.595	203.701	0.9923
0.050	74.997	172.199	0.9915
0.060	62.852	150.799	0.9906
0.070	53.790	135.500	0.9899
0.080	45.784	123.734	0.9895
0.090	39.651	114.102	0.9892
0.100	34.464	105.600	0.9889
0.125	24.965	89.250	0.9884
0.150	18.572	76.960	0.9882
0.175	14.274	66.930	0.9879
0.200	11.733	59.370	0.9873
0.250	8.586	48.235	0.9858
0.300	6.759	40.960	0.9844
0.350	5.438	35.599	0.9834
0.400	4.454	31.485	0.9826
0.500	3.072	25.581	0.9817
0.600	2.273	21.403	0.9806
0.700	1.770	18.328	0.9794
0.800	1.444	15.955	0.9778
0.900	1.264	14.021	0.9749
1.000	1.212	12.464	0.9697
1.100	1.201	11.181	0.9630
1.200	1.260	10.010	0.9521
1.300	1.468	8.949	0.9318
1.400	2.237	8.212	0.8852
1.500	2.745	8.309	0.8678
1.600	2.625	8.597	0.8794
1.700	2.143	8.573	0.8972
1.800	1.741	8.205	0.9069
1.900	1.488	7.821	0.9116
2.000	1.304	7.479	0.9148
2.200	1.018	6.846	0.9200
2.400	0.826	6.283	0.9228
2.600	0.695	5.800	0.9238
2.800	0.598	5.385	0.9242
3.000	0.523	5.024	0.9241
3.200	0.460	4.708	0.9243
3.400	0.407	4.426	0.9245
3.600	0.363	4.174	0.9246
3.800	0.326	3.946	0.9247
4.000	0.294	3.740	0.9248
4.200	0.267	3.552	0.9248
4.400	0.244	3.380	0.9249
4.600	0.223	3.222	0.9249
4.800	0.205	3.076	0.9249
5.000	0.190	2.942	0.9244
6.000	0.130	2.391	0.9257
6.500	0.110	2.173	0.9260
7.000	0.095	1.983	0.9262
7.500	0.082	1.814	0.9265
8.000	0.072	1.663	0.9269
8.500	0.063	1.527	0.9272
9.000	0.056	1.402	0.9277
9.500	0.049	1.286	0.9282
10.000	0.044	1.178	0.9286
10.500	0.040	1.076	0.9293
11.000	0.036	0.979	0.9298
11.500	0.033	0.883	0.9283
12.000	0.033	0.791	0.9224
12.500	0.034	0.700	0.9118
13.000	0.038	0.609	0.8960
13.500	0.041	0.517	0.8789
14.000	0.048	0.417	0.8486
14.200	0.053	0.373	0.8312
14.400	0.058	0.327	0.8102
14.600	0.067	0.273	0.7802
14.800	0.086	0.211	0.7202
15.000	0.125	0.153	0.6119
15.200	0.178	0.108	0.4903
15.400	0.234	0.084	0.3881
15.600	0.280	0.073	0.3182
15.800	0.318	0.065	0.2694
16.000	0.351	0.060	0.2326
16.200	0.380	0.055	0.2031
16.400	0.407	0.050	0.1789
16.750	0.448	0.045	0.1460
17.000	0.474	0.042	0.1278
17.250	0.498	0.040	0.1129
17.500	0.520	0.038	0.1005
17.750	0.540	0.036	0.0899
18.000	0.558	0.035	0.0809
18.500	0.591	0.032	0.0664
19.000	0.620	0.030	0.0554
19.500	0.646	0.028	0.0467
20.000	0.668	0.027	0.0398
20.500	0.689	0.025	0.0342
21.000	0.707	0.024	0.0296
21.500	0.724	0.023	0.0258
22.000	0.739	0.022	0.0226
22.500	0.753	0.021	0.0199
23.000	0.766	0.021	0.0177
23.500	0.778	0.020	0.0157
24.000	0.789	0.019	0.0140
24.500	0.799	0.018	0.0126
25.000	0.809	0.018	0.0113
25.500	0.817	0.017	0.0102
26.000	0.826	0.016	0.0092
27.000	0.840	0.015	0.0076
28.000	0.854	0.014	0.0063
29.000	0.865	0.014	0.0053
30.000	0.876	0.013	0.0044
35.000	0.915	0.010	0.0020
40.000	0.940	0.008	0.0010
45.000	0.957	0.007	0.0005
50.000	0.969	0.006	0.0003
55.000	0.979	0.005	0.0001
60.000	0.987	0.004	0.0000
65.000	0.995	0.004	0.0000
70.000	1.006	0.004	0.0000
72.500	1.025	0.004	0.0002
75.000	1.011	0.024	0.0002
77.500	1.008	0.025	0.0002
80.000	1.007	0.024	0.0002
85.000	1.007	0.028	0.0002
90.000	1.005	0.031	0.0002
95.000	0.999	0.036	0.0003
100.000	0.991	0.030	0.0002
110.000	0.994	0.025	0.0002
120.000	0.991	0.024	0.0002
130.000	0.987	0.021	0.0001
140.000	0.989	0.016	0.0001
150.000	0.990	0.015	0.0001
160.000	0.989	0.014	0.0001
170.000	0.989	0.011	0.0001
180.000	0.990	0.010	0.0000
190.000	0.990	0.009	0.0000
200.000	0.991	0.007	0.0000
220.000	0.992	0.006	0.0000
240.000	0.993	0.005	0.0000
260.000	0.993	0.004	0.0000
280.000	0.994	0.003	0.0000
300.000	0.995	0.002	0.0000] ;

nn=AAA(:,2)+i*AAA(:,3);
% R=abs((nn-1)./(nn+1)).^2;
% retcompare(R,AAA(:,4)) % 4.1814e-004  erreur sur R < 1.6e-3  et < 1.e-4 de ld= .3  29.28

X_V=AAA(:,1); % energies hv en eV...
V=1.239829488174885./lambda;  
lld=1.239829488174885./X_V;

if isempty(V);indice=[];else;indice=interp1(X_V,nn,V,method,NaN*(1+1i));end; 

%------------------------------------------------------
	
case {'Titane'}
bornes=[ 0.000123,200];ref=' Electronic HandBook of Optical Constants of Solids (HOC)';
% les valeurs absentes de real(n) ld<0.04  ont ete remplaces par des 1
% point double en ld=0.266

AAA=[
0.000123	0.9999912 	4.89E-007	 
0.000145	0.999988	9.03E-007	
0.000175	0.9999828	1.8E-006	
0.000234	0.999972	5.17E-006	
0.000241	0.9999716	5.67E-006	
0.000247	0.9999729	6.22E-006	
0.000254	0.9999705	7.97E-007	
0.000261	0.999967	8.81E-007	
0.00035	    0.9999351  	2.63E-006	
0.000481	0.999874	8.63E-006	
0.000738	0.999697	4.13E-005	
0.000843	0.999603	6.69E-005	
0.000964	0.999483	0.000108	
0.0011	    0.99933	    0.000172	
0.00123	    0.999181	0.000249	
0.0014	    0.99896 	0.000394	
0.00169		0.9986		0.00074	
0.00209		0.99831		0.00148	
0.00215		0.99834		0.00161	
0.0022		0.99847		0.00152	
0.00226		0.99834		0.00165	
0.00232		0.99832		0.0018	
0.00252		0.99857		0.00231	
0.00259		0.99886		0.00252	
0.00266		0.99945		0.00274	
0.00273		1.000853	0.000324	
0.0028		0.99946		0.00035	
0.00288		0.99887		0.000379	
0.00386		0.99588		0.000919	
0.00576		0.99028		0.00279	
0.0062		1			0.0044	 
0.00653		1			0.0051	
0.00689		1			0.006	
0.00729		1			0.0069	
0.00775		1			0.0079	
0.00815		0.9817		0.00623	 
0.00827		1			0.0091	 
0.00886		1			0.0105	
0.00954		1			0.012	
0.0103		1			0.0139	
0.0113		1			0.016	
0.0122		0.9611		0.0145	 
0.0124		1			0.0186	 
0.0131		1			0.0201	
0.0138		1			0.0217	
0.0146		1			0.0242	
0.0155		1			0.0263	
0.0165		1			0.0287	
0.0168		1			0.0304	
0.0172		1			0.0326	
0.0177		1			0.0349	
0.0182		1			0.0388	
0.0188		1			0.0445	
0.0194		1			0.052	
0.02		1			0.0615	
0.0207		1			0.0701	
0.0225		1			0.112	
0.0238		1			0.151	
0.0248		1			0.176	
0.0258		1			0.194	
0.027		1			0.208	
0.0282		1			0.209	
0.0295		1			0.191	
0.031		1			0.164	
0.0326		1			0.154	
0.0496		0.991	 	0.151	 
0.0506		0.987		0.152	
0.0517		0.982		0.153	
0.0528		0.977		0.154	
0.0539		0.972		0.154	
0.0551		0.961		0.156	
0.0564		0.951		0.158	
0.0577		0.941		0.159	
0.059		0.936		0.16	
0.0605		0.932		0.172	
0.062		0.93		0.188	
0.0636		0.922		0.201	
0.0653		0.92		0.217	
0.067		0.902		0.233	
0.0689		0.891		0.253	
0.0708		0.883		0.283	
0.0729		0.882		0.312	
0.0751		0.871		0.344	
0.0775		0.852		0.382	
0.08		0.853		0.41	
0.0827		0.86		0.436	
0.0855		0.872		0.459	
0.0886		0.892		0.505	
0.0918		0.903		0.515	
0.0954		0.912		0.521	
0.0992		0.923		0.531	
0.103		0.932		0.537	
0.108		0.94		0.542	
0.113		0.951		0.552	
0.118		0.966		0.569	
0.124		0.993		0.604	
0.131		1	 	    0.625	 
0.138		1.01		0.645	
0.146		1.01		0.691	
0.155		1.02		0.763	
0.165		1.02		0.81	
0.177		1.03		0.873	
0.191		1.06		0.947	
0.192		1.07		1.11	
0.195		1.08		1.14	
0.198		1.09		1.16	
0.2		    1.11		1.18	
0.202		1.12		1.19	
0.204		1.13		1.2	
0.206		1.15		1.21	
0.21		1.18		1.22	
0.212		1.19		1.22	
0.214		1.21		1.22	
0.216		1.22		1.22	
0.218		1.23		1.21	
0.22		1.23		1.21	
0.222		1.24		1.21	
0.224		1.24		1.2	
0.226		1.24		1.19	
0.228		1.23		1.19	
0.23		1.17		1.16	
0.232		1.18		1.41	
0.234		1.35		1.21	
0.236		1.28		1.2	
0.238		1.27		1.2	
0.24		1.25		1.2	
0.242		1.25		1.2	
0.244		1.24		1.21	
0.248		1.24		1.21	
0.25		1.23		1.21	
0.252		1.22		1.22	
0.254		1.22		1.22	
0.256		1.22		1.23	
0.258		1.21		1.23	
0.26		1.2		    1.23	
0.262		1.19		1.24	
0.266		1.16		1.27	
0.26601		1.17		1.26	
0.27		1.17		1.29	
0.272		1.17		1.29	
0.274		1.16		1.3	
0.276		1.15		1.31	
0.278		1.14		1.32	
0.28		1.14		1.33	
0.282		1.13		1.33	
0.284		1.12		1.34	
0.286		1.11		1.36	
0.288		1.1		    1.37	
0.29		1.09		1.38	
0.292		1.07		1.4	
0.294		1.06		1.42	
0.296		1.05		1.45	
0.298		1.05		1.47	
0.3		    1.04		1.49	
0.302		1.04		1.52	
0.304		1.04		1.54	
0.306		1.03		1.57	
0.308		1.03		1.59	
0.311		1.04		1.62	
0.312		1.04		1.64	
0.314		1.04		1.67	
0.316		1.05		1.71	
0.318		1.06		1.73	
0.32		1.07		1.75	
0.322		1.08		1.78	
0.326		1.11		1.83	
0.335		1.17		1.9	
0.344		1.24		1.96	
0.354		1.3		    2.01	
0.365		1.37		2.06	
0.376		1.44		2.09	
0.387		1.5		    2.12	
0.4		    1.55		2.15	
0.413		1.59		2.17	
0.428		1.63		2.21	
0.443		1.68		2.25	
0.459		1.71		2.29	
0.477		1.75		2.34	
0.496		1.78		2.39	
0.517		1.81		2.47	
0.539		1.86		2.56	
0.564		1.92		2.67	
0.59		2.01		2.77	
0.62		2.11		2.88	
0.636		2.16		2.93	
0.653		2.22		2.99	
0.67		2.29		3.05	
0.689		2.36		3.11	
0.708		2.44		3.17	
0.729		2.54		3.23	
0.751		2.65		3.24	
0.775		2.74		3.3	
0.8		    2.87		3.33	
0.827		2.98		3.32	
0.855		3.08		3.31	
0.886		3.17		3.28	
0.918		3.23		3.26	
0.954		3.28		3.25	
0.992		3.31		3.26	
1.03		3.35		3.3	
1.08		3.4		    3.35	
1.13		3.47		3.4	
1.18		3.54		3.46	
1.24		3.62		3.52	
1.27		3.65		3.55	
1.29		3.69		3.57	
1.32		3.73		3.6	
1.35		3.77		3.62	
1.38		3.8		    3.65	
1.41		3.85		3.69	
1.44		3.93		3.73	
1.48		3.94		3.65	
1.51		3.94		3.8	
1.55		4.04		3.82	
1.59		4.11		3.85	
1.63		4.18		3.87	
1.68		4.25		3.88	
1.72		4.32		3.89	
1.77		4.38		3.89	
1.82		4.44		3.9	
1.88		4.52		3.9	
1.94		4.6		    3.88	
2		    4.66		3.83	
2.07		4.71		3.77	
2.14		4.74		3.71	
2.21		4.75		3.62	
2.3		    4.73		3.5	
2.38		4.62		3.35	
2.48		4.43		3.22	
2.53		4.29		3.15	
2.58		4.11		3.1	
2.64		3.89		3.08	
2.7		    3.62		3.12	
2.76		3.35		3.25	
2.82		3.1		    3.44	
2.88		2.88		3.68	
2.95		2.69		3.98	
3.02		2.56		4.32	
3.1		    2.49		4.68	
3.18		2.48		5.02	
3.26		2.52		5.35	
3.35		2.59		5.65	
3.44		2.67		5.93	
3.54		2.74		6.21	
3.65		2.82		6.52	
3.76		2.95		6.88	
3.87		3.17		7.29	
4		     3.6		7.87	
4.13		6.39		9.94	
4.28		4.15		3.06	
4.43		1.43		5.65	
4.59		2		    6.98	
4.77		2.08		7.56	
4.96		2.05		8.1	
5.17		2.01		8.66	
5.39		1.98		9.27	
5.64		2		   9.92	
5.9		    2.04		10.61	
6.2		    2.12		11.34	
6.53		2.23		12.12	
6.89		2.41		12.94	
7.29		2.6		    13.79	
7.75		2.81		14.7	
8.27		3		    15.72	
8.86		3.22		16.87	
9.54		3.49		18.23	
10.3		3.9		    19.79	
11.3		4.48		21.45	
12.5		5.2	 	    24.4	 
14.3		6.24		27.6	
16.7		7.58		31.8	
20		    9.67		37.6	
22.2		11.2		41.3	
25		    13.3		45.8	
28.6		16.3		51.5	
33.3		20.9		58	
40		    27.2		65.9	
44.4		31.3		70.6	
50		    36.2		75.8	
57.1		42.3		81.8	
66.7		49.9		88.9	
80		   59.64		97.5	
100		    72.5		108.4	
125		    86.4		120	
154		   100.5		131.9	
200		    120		   148.4];	



lld=AAA(:,1);nn=AAA(:,2)+i*AAA(:,3);
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 
%------------------------------------------------------
	
case {'Cuivre'}
bornes=[ .000137756 ,9.53692];ref='Handbook of Optical Constants of Solids, Ed. by Edward D. Palik,Academic Press, Inc., 1985.';
% le point  1.2398 a ete retire (partie imaginaire aberrente  ?)
AAA=[1.37756       1.00100   3.65000e-07
       1.39303       1.00100   3.56000e-07
       1.54975       1.00100   5.38000e-07
       3.09950       1.00000   6.62000e-06
       4.13267       1.00000   9.49000e-05
       8.26533       1.00000   0.000273000
       10.3317      0.999000   0.000624000
       11.2709      0.999000   0.000823000
       12.3980      0.999000   0.000851000
       13.7756       1.00000   0.000174000
       15.4975      0.999000   0.000263000
       17.7114      0.998000   0.000395000
       20.6633      0.997000   0.000651000
       24.7960      0.997000    0.00111000
       30.9950      0.993000    0.00245000
       41.3267      0.988000    0.00513000
       61.9900      0.978000     0.0159000
       82.6533      0.969000     0.0310000
       103.317      0.965000     0.0540000
       112.709      0.968000     0.0599000
       118.076      0.968000     0.0625000
       123.980      0.967000     0.0653000
       130.505      0.965000     0.0693000
       134.761      0.963000     0.0744000
       137.756      0.963000     0.0788000
       140.886      0.963000     0.0834000
       144.163      0.966000     0.0877000
       147.595      0.969000     0.0911000
       149.373      0.971000     0.0923000
       151.195      0.973000     0.0934000
       153.062      0.974000     0.0943000
       154.975      0.977000     0.0946000
       156.937      0.979000     0.0941000
       158.949      0.980000     0.0936000
       161.013      0.982000     0.0921000
       163.132      0.984000     0.0910000
       165.307      0.983000     0.0872000
       167.541      0.979000     0.0850000
       169.836      0.977000     0.0857000
       172.194      0.975000     0.0868000
       174.620      0.974000     0.0881000
       177.114      0.972000     0.0893000
       182.324      0.968000     0.0927000
       187.848      0.967000     0.0963000
       193.719      0.965000      0.102000
       199.968      0.965000      0.107000
       206.633      0.966000      0.109000
       213.759      0.964000      0.110000
       221.393      0.960000      0.114000
       229.593      0.957000      0.120000
       238.423      0.954000      0.126000
       247.960      0.953000      0.134000
       258.292      0.952000      0.140000
       269.522      0.951000      0.146000
       281.773      0.948000      0.151000
       295.190      0.942000      0.157000
       309.950      0.934000      0.167000
       326.263      0.927000      0.180000
       344.389      0.922000      0.193000
       364.647      0.910000      0.205000
       387.438      0.891000      0.223000
       413.267      0.856000      0.262000
       427.517      0.849000      0.302000
       442.786      0.862000      0.346000
       459.185      0.884000      0.382000
       476.846      0.921000      0.403000
       495.920      0.960000      0.396000
       516.583      0.961000      0.373000
       539.043      0.943000      0.366000
       563.545      0.917000      0.381000
       590.381      0.897000      0.411000
       619.900      0.882000      0.455000
       652.526      0.883000      0.507000
       688.778      0.890000      0.561000
       729.294      0.913000      0.620000
       774.875      0.954000      0.673000
       799.871      0.981000      0.695000
       826.533       1.01000      0.707000
       855.034       1.03000      0.717000
       885.571       1.06000      0.724000
       953.692       1.08000      0.724000
       1033.17       1.09000      0.731000
       1127.09       1.07000      0.754000
       1239.80       1.04000      0.818000
       1305.05       1.03000      0.867000
       1377.56       1.03000      0.921000
       1458.59       1.03000      0.979000
       1549.75       1.03000       1.03000
       1653.07       1.01000       1.09000
       1771.14      0.972000       1.20000
       1907.38      0.958000       1.37000
       2066.33       1.04000       1.59000
       2137.59       1.10000       1.67000
       2213.93       1.18000       1.74000
       2295.93       1.28000       1.78000
       2384.23       1.38000       1.80000
       2479.60       1.47000       1.78000
       2582.92       1.53000       1.71000
       2695.22       1.52000       1.67000
       2817.73       1.49000       1.64000
       2951.90       1.42000       1.64000
       3099.50       1.34000       1.72000
       3262.63       1.34000       1.81000
       3443.89       1.31000       1.87000
       3646.47       1.27000       1.95000
       3874.38       1.18000       2.21000
       4132.67       1.18000       2.21000
       4427.86       1.17000       2.36000
       4768.46       1.15000       2.50000
       5165.83       1.12000       2.60000
       5390.44       1.04000       2.59000
       5635.45      0.826000       2.60000
       5903.81      0.468000       2.81000
       6199.00      0.272000       3.24000
       6525.26      0.214000       3.67000
       6701.62      0.215000       3.86000
       6887.78      0.213000       4.05000
       7084.57      0.214000       4.24000
       7292.94      0.223000       4.43000
       8265.33      0.260000       5.26000
 %      12398.0      0.433000       8.46000
       12651.0      0.496000       6.78000
       12914.6      0.505000       6.92000
       13189.4      0.515000       7.06000
       13476.1      0.525000       7.21000
       13775.6      0.536000       7.36000
       14088.6      0.547000       7.53000
       14416.3      0.559000       7.70000
       14759.5      0.572000       7.88000
       15119.5      0.586000       8.06000
       15497.5      0.606000       8.26000
       15894.9      0.627000       8.47000
       16313.2      0.649000       8.69000
       16754.1      0.672000       8.92000
       17219.4      0.697000       9.16000
       17711.4      0.723000       9.41000
       18232.4      0.752000       9.68000
       18784.8      0.782000       9.97000
       19371.9      0.815000       10.3000
       19996.8      0.850000       10.6000
       20663.3      0.890000       11.0000
       21375.9      0.933000       11.3000
       22139.3      0.980000       11.8000
       22959.3       1.03000       12.2000
       23842.3       1.09000       12.7000
       24796.0       1.15000       13.2000
       25829.2       1.22000       13.7000
       26952.2       1.29000       14.4000
       28177.3       1.37000       15.0000
       29519.0       1.47000       15.7000
       30995.0       1.59000       16.5000
       32626.3       1.73000       17.4000
       34438.9       1.90000       18.4000
       36464.7       2.09000       19.6000
       38743.8       2.32000       20.8000
       41326.7       2.59000       22.2000
       44278.6       2.92000       23.7000
       47684.6       3.32000       25.4000
       51658.3       3.81000       27.5000
       56354.5       4.44000       30.0000
       61990.0       5.23000       33.0000
       68877.8       6.23000       36.3000
       77487.5       7.66000       40.3000
       82653.3       8.57000       42.6000
       88557.1       9.64000       45.1000
       95369.2       10.8000       47.5000];




lld=AAA(:,1)*1.e-4;nn=AAA(:,2)+i*AAA(:,3);
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 
%------------------------------------------------------
case {'Tungsten'}
bornes=[0.000012398,24.7960];ref='W_llnl_cxro + W_palik';
% 4 valeurs supprimes au voisinage de .0413


AAA=[ 0.12398000       0.99999968    7.9216349e-09
      0.12499080       0.99999967    8.1607060e-09
      0.12600983       0.99999967    8.4068448e-09
      0.12703718       0.99999966    8.6600962e-09
      0.12807290       0.99999966    8.9198151e-09
      0.12911707       0.99999965    9.1878019e-09
      0.13016975       0.99999965    9.4631299e-09
      0.13123101       0.99999964    9.7458867e-09
      0.13230092       0.99999964    1.0036597e-08
      0.13337956       0.99999963    1.0335718e-08
      0.13446699       0.99999962    1.0645433e-08
      0.13556329       0.99999962    1.0961723e-08
      0.13666852       0.99999961    1.1290270e-08
      0.13778277       0.99999961    1.1625502e-08
      0.13890610       0.99999960    1.1971925e-08
      0.14003858       0.99999959    1.2328260e-08
      0.14118030       0.99999959    1.2694739e-08
      0.14233133       0.99999958    1.3072274e-08
      0.14349175       0.99999957    1.3458464e-08
      0.14466162       0.99999957    1.3859241e-08
      0.14584103       0.99999956    1.4268044e-08
      0.14703006       0.99999955    1.4695434e-08
      0.14822878       0.99999954    1.5134870e-08
      0.14943728       0.99999954    1.5586784e-08
      0.15065563       0.99999953    1.6059367e-08
      0.15188391       0.99999952    1.6541458e-08
      0.15312220       0.99999952    1.7042134e-08
      0.15437059       0.99999951    1.7555093e-08
      0.15562916       0.99999950    1.8082721e-08
      0.15689799       0.99999949    1.8627699e-08
      0.15817716       0.99999948    1.9187948e-08
      0.15946677       0.99999948    1.9761401e-08
      0.16076688       0.99999947    2.0346328e-08
      0.16207760       0.99999946    2.0940760e-08
      0.16339900       0.99999945    2.1547048e-08
      0.16473118       0.99999944    2.2168631e-08
      0.16607422       0.99999944    2.2809082e-08
      0.16742821       0.99999943    2.3472531e-08
      0.16879323       0.99999942    2.4176492e-08
      0.17016939       0.99999941    2.4927021e-08
      0.17155676       0.99999940    2.5668794e-08
      0.17295545       0.99999939    2.6412719e-08
      0.17436554       0.99999939    2.7162877e-08
      0.17578712       0.99999938    2.7947132e-08
      0.17722030       0.99999938    2.8753252e-08
      0.17866516       0.99999937    5.8172336e-09
      0.18012180       0.99999935    5.9988347e-09
      0.18159031       0.99999934    6.1859752e-09
      0.18307080       0.99999932    6.3793844e-09
      0.18456336       0.99999931    6.5789265e-09
      0.18606809       0.99999930    6.7844338e-09
      0.18758508       0.99999929    6.9963407e-09
      0.18911444       0.99999927    7.2151035e-09
      0.19065628       0.99999926    7.4397552e-09
      0.19221068       0.99999925    7.6731753e-09
      0.19377775       0.99999923    7.9117667e-09
      0.19535760       0.99999922    8.1600570e-09
      0.19695034       0.99999921    8.4138233e-09
      0.19855605       0.99999919    8.6768883e-09
      0.20017486       0.99999918    8.9484075e-09
      0.20180687       0.99999917    9.2272095e-09
      0.20345218       0.99999915    9.5156556e-09
      0.20511091       0.99999914    9.8134432e-09
      0.20678315       0.99999912    1.0118886e-08
      0.20846904       0.99999911    1.0434879e-08
      0.21016867       0.99999909    1.0761838e-08
      0.21188215       0.99999908    1.1095409e-08
      0.21360961       0.99999906    1.1441462e-08
      0.21535114       0.99999905    1.1798249e-08
      0.21710688       0.99999903    1.2167204e-08
      0.21887693       0.99999901    1.2546385e-08
      0.22066141       0.99999900    1.2937766e-08
      0.22246045       0.99999898    1.3342147e-08
      0.22427414       0.99999896    1.3758265e-08
      0.22610263       0.99999895    1.4186375e-08
      0.22794602       0.99999893    1.4629222e-08
      0.22980444       0.99999891    1.5082301e-08
      0.23167802       0.99999889    1.5554475e-08
      0.23356687       0.99999888    1.6036340e-08
      0.23547111       0.99999886    1.6537859e-08
      0.23739089       0.99999884    1.7051918e-08
      0.23932631       0.99999882    1.7582582e-08
      0.24127752       0.99999880    1.8131294e-08
      0.24324463       0.99999878    1.8692221e-08
      0.24522778       0.99999876    1.9278148e-08
      0.24722710       0.99999874    1.9875975e-08
      0.24924272       0.99999872    2.0492716e-08
      0.25127477       0.99999870    2.1129302e-08
      0.25332339       0.99999867    2.1785264e-08
      0.25538871       0.99999865    2.2462123e-08
      0.25747087       0.99999863    2.3160934e-08
      0.25957000       0.99999861    2.3880837e-08
      0.26168625       0.99999858    2.4622444e-08
      0.26381975       0.99999856    2.5386362e-08
      0.26597065       0.99999854    2.6176402e-08
      0.26813908       0.99999851    2.6990367e-08
      0.27032520       0.99999849    2.7829601e-08
      0.27252913       0.99999846    2.8695049e-08
      0.27475103       0.99999844    2.9586677e-08
      0.27699105       0.99999841    3.0505990e-08
      0.27924933       0.99999838    3.1449580e-08
      0.28152603       0.99999836    3.2426731e-08
      0.28382128       0.99999833    3.3431153e-08
      0.28613525       0.99999830    3.4471121e-08
      0.28846808       0.99999827    3.5533241e-08
      0.29081993       0.99999825    3.6634674e-08
      0.29319096       0.99999822    3.7770142e-08
      0.29558132       0.99999819    3.8935816e-08
      0.29799117       0.99999816    4.0142658e-08
      0.30042066       0.99999813    4.1384265e-08
      0.30286996       0.99999810    4.2662996e-08
      0.30533923       0.99999806    4.3982100e-08
      0.30782863       0.99999803    4.5343457e-08
      0.31033833       0.99999800    4.6747149e-08
      0.31286848       0.99999797    4.8192241e-08
      0.31541927       0.99999793    4.9681004e-08
      0.31799085       0.99999790    5.1210456e-08
      0.32058340       0.99999786    5.2795842e-08
      0.32319709       0.99999783    5.4425416e-08
      0.32583208       0.99999779    5.6103222e-08
      0.32848856       0.99999776    5.7835537e-08
      0.33116670       0.99999772    5.9613170e-08
      0.33386667       0.99999768    6.1451299e-08
      0.33658865       0.99999764    6.3344626e-08
      0.33933282       0.99999760    6.5291093e-08
      0.34209937       0.99999756    6.7306521e-08
      0.34488847       0.99999752    6.9375055e-08
      0.34770032       0.99999748    7.1510599e-08
      0.35053508       0.99999744    7.3713944e-08
      0.35339296       0.99999740    7.5986325e-08
      0.35627414       0.99999736    7.8324286e-08
      0.35917881       0.99999731    8.0730878e-08
      0.36210716       0.99999727    8.3220172e-08
      0.36505938       0.99999722    8.5778718e-08
      0.36803568       0.99999718    8.8420193e-08
      0.37103624       0.99999713    9.1136098e-08
      0.37406126       0.99999708    9.3947140e-08
      0.37711095       0.99999703    9.6834965e-08
      0.38018549       0.99999698    9.9801775e-08
      0.38328511       0.99999694    1.0286857e-07
      0.38641000       0.99999688    1.0603431e-07
      0.38956036       0.99999683    1.0928663e-07
      0.39273641       0.99999678    1.1264197e-07
      0.39593835       0.99999673    1.1610077e-07
      0.39916640       0.99999667    1.1967075e-07
      0.40242076       0.99999662    1.2333562e-07
      0.40570166       0.99999656    1.2713400e-07
      0.40900931       0.99999651    1.3008059e-07
      0.41234392       0.99999645    1.3137783e-07
      0.41570572       0.99999639    1.3466780e-07
      0.41909493       0.99999633    1.3878163e-07
      0.42251177       0.99999627    1.4304130e-07
      0.42595647       0.99999621    1.4740827e-07
      0.42942925       0.99999614    1.5193253e-07
      0.43293034       0.99999608    1.5656857e-07
      0.43645998       0.99999602    1.6137266e-07
      0.44001840       0.99999595    1.6629296e-07
      0.44360583       0.99999588    1.7139520e-07
      0.44722250       0.99999581    1.7661864e-07
      0.45086866       0.99999575    1.8203648e-07
      0.45454455       0.99999568    1.8758031e-07
      0.45825041       0.99999560    1.9333326e-07
      0.46198648       0.99999553    1.9921733e-07
      0.46575301       0.99999546    2.0532605e-07
      0.46955025       0.99999538    2.1157077e-07
      0.47337845       0.99999531    2.1805755e-07
      0.47723786       0.99999523    2.2468593e-07
      0.48112873       0.99999515    2.3157292e-07
      0.48505133       0.99999507    2.3860683e-07
      0.48900590       0.99999499    2.4591850e-07
      0.49299272       0.99999491    2.5338251e-07
      0.49701204       0.99999483    2.6114479e-07
      0.50106413       0.99999474    2.6906734e-07
      0.50514926       0.99999465    2.7730537e-07
      0.50926770       0.99999457    2.8571559e-07
      0.51341971       0.99999448    2.9445765e-07
      0.51760557       0.99999439    3.0338670e-07
      0.52182555       0.99999429    3.1266180e-07
      0.52607995       0.99999420    3.2213970e-07
      0.53036903       0.99999411    3.3197988e-07
      0.53469307       0.99999401    3.4203986e-07
      0.53905238       0.99999391    3.5247957e-07
      0.54344722       0.99999381    3.6315735e-07
      0.54787789       0.99999371    3.7423287e-07
      0.55234469       0.99999361    3.8556581e-07
      0.55684790       0.99999350    3.9731421e-07
      0.56138783       0.99999340    4.0934123e-07
      0.56596477       0.99999329    4.2180376e-07
      0.57057902       0.99999318    4.3456735e-07
      0.57523090       0.99999307    4.4778426e-07
      0.57992070       0.99999295    4.6132647e-07
      0.58464874       0.99999284    4.7534542e-07
      0.58941532       0.99999272    4.8971596e-07
      0.59422077       0.99999260    5.0458365e-07
      0.59906539       0.99999248    5.1983122e-07
      0.60394951       0.99999236    5.3559892e-07
      0.60887346       0.99999224    5.5177557e-07
      0.61383754       0.99999211    5.6849419e-07
      0.61884210       0.99999198    5.8565463e-07
      0.62388746       0.99999185    6.0338249e-07
      0.62897395       0.99999172    6.2158579e-07
      0.63410192       0.99999159    6.4037903e-07
      0.63927169       0.99999145    6.5968628e-07
      0.64448361       0.99999131    6.7961255e-07
      0.64973802       0.99999117    7.0009144e-07
      0.65503527       0.99999103    7.2121416e-07
      0.66037571       0.99999088    7.4293169e-07
      0.66575969       0.99999074    7.6532418e-07
      0.67118757       0.99999059    7.8835466e-07
      0.67665969       0.99999044    8.1208493e-07
      0.68217644       0.99999028    8.3650247e-07
      0.68773815       0.99999013    8.6165323e-07
      0.69334522       0.99998997    8.8754349e-07
      0.69899799       0.99998981    9.1419836e-07
      0.70469686       0.99998965    9.4164411e-07
      0.71044218       0.99998948    9.6988286e-07
      0.71623435       0.99998931    9.9897578e-07
      0.72207374       0.99998914    1.0288997e-06
      0.72796073       0.99998897    1.0597391e-06
      0.73389573       0.99998879    1.0914416e-06
      0.73987911       0.99998862    1.1241241e-06
      0.74591127       0.99998844    1.1577059e-06
      0.75199261       0.99998825    1.1923395e-06
      0.75812354       0.99998807    1.2279100e-06
      0.76430445       0.99998788    1.2646031e-06
      0.77053575       0.99998769    1.3022706e-06
      0.77681785       0.99998750    1.3411458e-06
      0.78315117       0.99998730    1.3810381e-06
      0.78953613       0.99998710    1.4222241e-06
      0.79597314       0.99998690    1.4644659e-06
      0.80246263       0.99998670    1.5080903e-06
      0.80900503       0.99998649    1.5528113e-06
      0.81560077       0.99998628    1.5990095e-06
      0.82225028       0.99998607    1.6463486e-06
      0.82895401       0.99998586    1.6952697e-06
      0.83571239       0.99998564    1.7453769e-06
      0.84252588       0.99998542    1.7971660e-06
      0.84939491       0.99998520    1.8501835e-06
      0.85631994       0.99998498    1.9050169e-06
      0.86330144       0.99998475    1.9611337e-06
      0.87033985       0.99998452    2.0191668e-06
      0.87743565       0.99998429    2.0785279e-06
      0.88458929       0.99998406    2.1399500e-06
      0.89180126       0.99998383    2.2027540e-06
      0.89907203       0.99998359    2.2677429e-06
      0.90640208       0.99998335    2.3341633e-06
      0.91379189       0.99998311    2.4029242e-06
      0.92124195       0.99998287    2.4731747e-06
      0.92875274       0.99998262    2.5458934e-06
      0.93632477       0.99998238    2.6201535e-06
      0.94395854       0.99998214    2.6970727e-06
      0.95165454       0.99998189    2.7755946e-06
      0.95941329       0.99998165    2.8569106e-06
      0.96723529       0.99998141    2.9398865e-06
      0.97512107       0.99998117    3.0258758e-06
      0.98307114       0.99998093    3.1136133e-06
      0.99108602       0.99998071    3.2044630e-06
      0.99916625       0.99998049    3.2971903e-06
       1.0073124       0.99998031    3.3931680e-06
       1.0155249       0.99998018    3.4911532e-06
       1.0238044       0.99998083    3.5923791e-06
       1.0321513       0.99997970    3.2262700e-06
       1.0405664       0.99997940    3.3246282e-06
       1.0490500       0.99997911    3.4251998e-06
       1.0576028       0.99997895    3.5295732e-06
       1.0662253       0.99997908    3.6362758e-06
       1.0749182       0.99998050    2.6759382e-06
       1.0836819       0.99997814    2.7602617e-06
       1.0925170       0.99997764    2.8474661e-06
       1.1014242       0.99997714    2.9368126e-06
       1.1104040       0.99997674    3.0295679e-06
       1.1194570       0.99997633    3.1246477e-06
       1.1285839       0.99997596    3.2232978e-06
       1.1377851       0.99997560    3.3244719e-06
       1.1470613       0.99997526    3.4293990e-06
       1.1564132       0.99997493    3.5370706e-06
       1.1658413       0.99997464    3.6486870e-06
       1.1753463       0.99997438    3.7632789e-06
       1.1849288       0.99997419    3.8819942e-06
       1.1945894       0.99997416    4.0039409e-06
       1.2043288       0.99997449    4.1302219e-06
       1.2141476       0.99997870    4.2596202e-06
       1.2240464       0.99997544    1.7009730e-06
       1.2340259       0.99997245    1.7507811e-06
       1.2440868       0.99997166    1.8021202e-06
       1.2542298       0.99997091    1.8552688e-06
       1.2644554       0.99997024    1.9107726e-06
       1.2747644       0.99996958    1.9677682e-06
       1.2851574       0.99996894    2.0264697e-06
       1.2956351       0.99996830    2.0868209e-06
       1.3061983       0.99996767    2.1490186e-06
       1.3168476       0.99996704    2.2129865e-06
       1.3275838       0.99996640    2.2788757e-06
       1.3384074       0.99996577    2.3466860e-06
       1.3493193       0.99996512    2.4165226e-06
       1.3603202       0.99996448    2.4883872e-06
       1.3714108       0.99996382    2.5623278e-06
       1.3825918       0.99996316    2.6384695e-06
       1.3938639       0.99996249    2.7167958e-06
       1.4052279       0.99996182    2.7974403e-06
       1.4166846       0.99996113    2.8803233e-06
       1.4282347       0.99996044    2.9657408e-06
       1.4398790       0.99995973    3.0535335e-06
       1.4516182       0.99995902    3.1440718e-06
       1.4634531       0.99995829    3.2370971e-06
       1.4753845       0.99995756    3.3330495e-06
       1.4874131       0.99995681    3.4315817e-06
       1.4995399       0.99995606    3.5332202e-06
       1.5117655       0.99995529    3.6375234e-06
       1.5240907       0.99995451    3.7452221e-06
       1.5365165       0.99995372    3.8557258e-06
       1.5490436       0.99995292    3.9698246e-06
       1.5616728       0.99995211    4.0868309e-06
       1.5744049       0.99995128    4.2076308e-06
       1.5872409       0.99995044    4.3314282e-06
       1.6001815       0.99994959    4.4594352e-06
       1.6132277       0.99994872    4.5906185e-06
       1.6263801       0.99994784    4.7261958e-06
       1.6396399       0.99994695    4.8650371e-06
       1.6530077       0.99994604    5.0087032e-06
       1.6664845       0.99994512    5.1557973e-06
       1.6800712       0.99994418    5.3079158e-06
       1.6937687       0.99994323    5.4635598e-06
       1.7075778       0.99994227    5.6247490e-06
       1.7214995       0.99994128    5.7896390e-06
       1.7355347       0.99994029    5.9603066e-06
       1.7496844       0.99993927    6.1347923e-06
       1.7639494       0.99993825    6.3155170e-06
       1.7783307       0.99993720    6.5002243e-06
       1.7928292       0.99993614    6.6916986e-06
       1.8074460       0.99993506    6.8873121e-06
       1.8221819       0.99993396    7.0901014e-06
       1.8370380       0.99993285    7.2971875e-06
       1.8520152       0.99993172    7.5119370e-06
       1.8671145       0.99993057    7.7311424e-06
       1.8823369       0.99992940    7.9584778e-06
       1.8976834       0.99992821    8.1904470e-06
       1.9131550       0.99992701    8.4313103e-06
       1.9287528       0.99992579    8.6771072e-06
       1.9444777       0.99992454    8.9319347e-06
       1.9603309       0.99992328    9.1920947e-06
       1.9763133       0.99992199    9.4618714e-06
       1.9924260       0.99992069    9.7374066e-06
       2.0086700       0.99991937    1.0022921e-05
       2.0250465       0.99991802    1.0314640e-05
       2.0415565       0.99991665    1.0616746e-05
       2.0582011       0.99991526    1.0925566e-05
       2.0749815       0.99991385    1.1245336e-05
       2.0918986       0.99991242    1.1572330e-05
       2.1089536       0.99991096    1.1910732e-05
       2.1261477       0.99990948    1.2256927e-05
       2.1434820       0.99990798    1.2615056e-05
       2.1609576       0.99990646    1.2981570e-05
       2.1785757       0.99990491    1.3360473e-05
       2.1963374       0.99990333    1.3748451e-05
       2.2142439       0.99990174    1.4149573e-05
       2.2322964       0.99990011    1.4560350e-05
       2.2504961       0.99989846    1.4984495e-05
       2.2688442       0.99989679    1.5419132e-05
       2.2873419       0.99989509    1.5868035e-05
       2.3059904       0.99989336    1.6328147e-05
       2.3247909       0.99989161    1.6802974e-05
       2.3437447       0.99988983    1.7289924e-05
       2.3628530       0.99988802    1.7792389e-05
       2.3821171       0.99988618    1.8307873e-05
       2.4015383       0.99988432    1.8839493e-05
       2.4211178       0.99988243    1.9384976e-05
       2.4408569       0.99988051    1.9947090e-05
       2.4607570       0.99987856    2.0524183e-05
       2.4808193       0.99987658    2.1118845e-05
       2.5010452       0.99987457    2.1729607e-05
       2.5214360       0.99987253    2.2358668e-05
       2.5419930       0.99987045    2.3005036e-05
       2.5627176       0.99986835    2.3670470e-05
       2.5836112       0.99986622    2.4354358e-05
       2.6046751       0.99986405    2.5058019e-05
       2.6259108       0.99986186    2.5781485e-05
       2.6473196       0.99985963    2.6525539e-05
       2.6689029       0.99985736    2.7290859e-05
       2.6906622       0.99985507    2.8077688e-05
       2.7125989       0.99985274    2.8887386e-05
       2.7347145       0.99985037    2.9719601e-05
       2.7570104       0.99984798    3.0576184e-05
       2.7794880       0.99984554    3.1456037e-05
       2.8021489       0.99984308    3.2362095e-05
       2.8249946       0.99984057    3.3292518e-05
       2.8480265       0.99983804    3.4250855e-05
       2.8712461       0.99983546    3.5234464e-05
       2.8946551       0.99983285    3.6248051e-05
       2.9182550       0.99983021    3.7287984e-05
       2.9420472       0.99982752    3.8359773e-05
       2.9660335       0.99982480    3.9458919e-05
       2.9902152       0.99982205    4.0592480e-05
       3.0145942       0.99981925    4.1754638e-05
       3.0391719       0.99981642    4.2953231e-05
       3.0639499       0.99981355    4.4181364e-05
       3.0889300       0.99981064    4.5448864e-05
       3.1141138       0.99980769    4.6747270e-05
       3.1395029       0.99980471    4.8087322e-05
       3.1650989       0.99980169    4.9459376e-05
       3.1909037       0.99979863    5.0875528e-05
       3.2169188       0.99979553    5.2324789e-05
       3.2431460       0.99979239    5.3821963e-05
       3.2695871       0.99978921    5.5353767e-05
       3.2962437       0.99978600    5.6936902e-05
       3.3231177       0.99978274    5.8556110e-05
       3.3502107       0.99977945    6.0228477e-05
       3.3775247       0.99977611    6.1938034e-05
       3.4050613       0.99977275    6.3705708e-05
       3.4328224       0.99976933    6.5512211e-05
       3.4608099       0.99976589    6.7380158e-05
       3.4890255       0.99976240    6.9288318e-05
       3.5174712       0.99975889    7.1263265e-05
       3.5461488       0.99975533    7.3280091e-05
       3.5750602       0.99975174    7.5363932e-05
       3.6042074       0.99974811    7.7490879e-05
       3.6335921       0.99974445    7.9693839e-05
       3.6632164       0.99974075    8.1941815e-05
       3.6930823       0.99973703    8.4267208e-05
       3.7231916       0.99973326    8.6639500e-05
       3.7535465       0.99972948    8.9095849e-05
       3.7841488       0.99972565    9.1602183e-05
       3.8150006       0.99972182    9.4193312e-05
       3.8461039       0.99971793    9.6838274e-05
       3.8774608       0.99971405    9.9574326e-05
       3.9090734       0.99971012    0.00010236800
       3.9409437       0.99970621    0.00010525350
       3.9730738       0.99970224    0.00010820095
       4.0054659       0.99969830    0.00011124727
       4.0381221       0.99969432    0.00011435948
       4.0710445       0.99969037    0.00011757015
       4.1042353       0.99968639    0.00012085129
       4.1376967       0.99968247    0.00012423514
       4.1714310       0.99967852    0.00012769484
       4.2054403       0.99967468    0.00013126424
       4.2397268       0.99967084    0.00013491467
       4.2742929       0.99966720    0.00013867839
       4.3091408       0.99966371    0.00014252830
       4.3442728       0.99966087    0.00014649482
       4.3796912       0.99966133    0.00015055036
       4.4153984       0.99965941    0.00014817432
       4.4513967       0.99964804    0.00015245002
       4.4876885       0.99964295    0.00015685509
       4.5242762       0.99963803    0.00016137163
       4.5611622       0.99963360    0.00016603422
       4.5983489       0.99962929    0.00017081571
       4.6358388       0.99962532    0.00017574736
       4.6736344       0.99962159    0.00018080984
       4.7117381       0.99961838    0.00018603493
       4.7501524       0.99961732    0.00019140005
       4.7888799       0.99962001    0.00019693334
       4.8279232       0.99962632    0.00018968323
       4.8672848       0.99960968    0.00019528547
       4.9069673       0.99959761    0.00020104355
       4.9469733       0.99959217    0.00020697749
       4.9873055       0.99958712    0.00021307854
       5.0279666       0.99958262    0.00021936295
       5.0689591       0.99957844    0.00022583178
       5.1102858       0.99957467    0.00023249736
       5.1519495       0.99957134    0.00023935739
       5.1939528       0.99956849    0.00024641867
       5.2362986       0.99956639    0.00025369109
       5.2789897       0.99956504    0.00026117557
       5.3220288       0.99956579    0.00026888611
       5.3654187       0.99956840    0.00027681666
       5.4091625       0.99960038    0.00028497246
       5.4532629       0.99960250    0.00025157966
       5.4977228       0.99956368    0.00025871035
       5.5425452       0.99955146    0.00026601640
       5.5877330       0.99954279    0.00027354114
       5.6332893       0.99953638    0.00028126232
       5.6792169       0.99953102    0.00028921796
       5.7255190       0.99952625    0.00029737860
       5.7721986       0.99952230    0.00030579313
       5.8192588       0.99951875    0.00031442243
       5.8667027       0.99951603    0.00032331944
       5.9145333       0.99951370    0.00033243796
       5.9627539       0.99951234    0.00034184607
       6.0113677       0.99951140    0.00035148619
       6.0603778       0.99951173    0.00036143765
       6.1097875       0.99951258    0.00037163059
       6.1596000       0.99951515    0.00038211569
       6.2098186       0.99951837    0.00039283793
       6.2604467       0.99952417    0.00040394907
       6.3114875       0.99953087    0.00041533055
       6.3629444       0.99954197    0.00042708871
       6.4148209       0.99955442    0.00043911712
       6.4671203       0.99957652    0.00045155064
       6.5198461       0.99960116    0.00046426589
       6.5730018       0.99974818    0.00047740677
       6.6265908       0.99987610    0.00033109399
       6.6806168       0.99971807    0.00033920120
       6.7350832       0.99967638    0.00034746871
       6.7899937       0.99978075    0.00035597380
       6.8453519       0.99990421    0.00036465004
       6.9011614       0.99987409    0.00012344993
       6.9574259       0.99962959    0.00012669162
       7.0141491       0.99957923    0.00013003752
       7.0713348       0.99953861    0.00013345088
       7.1289867       0.99951031    0.00013696923
       7.1871086       0.99948176    0.00014055702
       7.2457044       0.99945846    0.00014423043
       7.3047779       0.99943475    0.00014797629
       7.3643331       0.99941383    0.00015184472
       7.4243738       0.99939258    0.00015579044
       7.4849040       0.99937302    0.00015986225
       7.5459277       0.99935317    0.00016401646
       7.6074489       0.99933440    0.00016829831
       7.6694717       0.99931538    0.00017266846
       7.7320001       0.99929708    0.00017717417
       7.7950384       0.99927853    0.00018177417
       7.8585906       0.99926048    0.00018651614
       7.9226609       0.99924218    0.00019135832
       7.9872536       0.99922421    0.00019634521
       8.0523729       0.99920601    0.00020143962
       8.1180231       0.99918802    0.00020668781
       8.1842086       0.99916980    0.00021205007
       8.2509337       0.99915171    0.00021756924
       8.3182027       0.99913339    0.00022320258
       8.3860202       0.99911512    0.00022897105
       8.4543906       0.99909662    0.00023486503
       8.5233185       0.99907811    0.00024091818
       8.5928082       0.99905936    0.00024710630
       8.6628646       0.99904055    0.00025346561
       8.7334921       0.99902150    0.00025997056
       8.8046954       0.99900236    0.00026665928
       8.8764792       0.99898298    0.00027350283
       8.9488483       0.99896349    0.00028053662
       9.0218074       0.99894375    0.00028773566
       9.0953613       0.99892387    0.00029513333
       9.1695149       0.99890375    0.00030270722
       9.2442731       0.99888347    0.00031048759
       9.3196408       0.99886294    0.00031845343
       9.3956229       0.99884224    0.00032663081
       9.4722245       0.99882130    0.00033500639
       9.5494506       0.99880017    0.00034360363
       9.6273064       0.99877881    0.00035241486
       9.7057969       0.99875726    0.00036146106
       9.7849273       0.99873551    0.00037067253
       9.8647029       0.99871360    0.00038004959
       9.9451288       0.99869140    0.00038960832
       10.026211       0.99866892    0.00039935195
       10.107953       0.99864614    0.00040928578
       10.190362       0.99862307    0.00041941316
       10.273443       0.99859970    0.00042979231
       10.357202       0.99857602    0.00044042300
       10.441643       0.99855204    0.00045132168
       10.526773       0.99852775    0.00046248223
       10.612597       0.99850317    0.00047392500
       10.699120       0.99847827    0.00048563818
       10.786349       0.99845309    0.00049765212
       10.874289       0.99842758    0.00050994834
       10.962946       0.99840179    0.00052256557
       11.052326       0.99837566    0.00053547761
       11.142434       0.99834926    0.00054872330
       11.233278       0.99832250    0.00056227061
       11.324861       0.99829548    0.00057617427
       11.417192       0.99826810    0.00059039269
       11.510275       0.99824046    0.00060499059
       11.604117       0.99821246    0.00061991551
       11.698725       0.99818423    0.00063523861
       11.794103       0.99815563    0.00065090015
       11.890259       0.99812687    0.00066692310
       11.987199       0.99809775    0.00068327371
       12.084930       0.99806840    0.00070000819
       12.183457       0.99803866    0.00071708472
       12.282788       0.99800867    0.00073436372
       12.382928       0.99797826    0.00075193691
       12.483885       0.99794750    0.00076995403
       12.585665       0.99791629    0.00078833877
       12.688275       0.99788476    0.00080718894
       12.791721       0.99785277    0.00082641921
       12.896010       0.99782052    0.00084617480
       13.001150       0.99778780    0.00086633084
       13.107148       0.99775489    0.00088699214
       13.214009       0.99772152    0.00090806088
       13.321741       0.99768793    0.00092965005
       13.430352       0.99765386    0.00095166013
       13.539849       0.99761954    0.00097405390
       13.650238       0.99758471    0.00099686697
       13.761527       0.99754959     0.0010203025
       13.873723       0.99751393     0.0010441892
       13.986834       0.99747805     0.0010687311
       14.100868       0.99744163     0.0010937413
       14.215831       0.99740507     0.0011194458
       14.331731       0.99736796     0.0011456382
       14.448576       0.99733081     0.0011724483
       14.566374       0.99729311     0.0011997690
       14.685132       0.99725521     0.0012277079
       14.804859       0.99721677     0.0012561805
       14.925561       0.99717815     0.0012852987
       15.047248       0.99713900     0.0013149884
       15.169927       0.99709969     0.0013454631
       15.293606       0.99705987     0.0013765427
       15.418293       0.99701998     0.0014084388
       15.543997       0.99697962     0.0014409737
       15.670725       0.99693933     0.0014743482
       15.798487       0.99689865     0.0015083080
       15.927291       0.99685835     0.0015427036
       16.057145       0.99681752     0.0015776674
       16.188057       0.99677628     0.0016129736
       16.320037       0.99673454     0.0016489479
       16.453092       0.99669253     0.0016856373
       16.587233       0.99665006     0.0017230667
       16.722467       0.99660741     0.0017613777
       16.858803       0.99656436     0.0018004704
       16.996252       0.99652129     0.0018404872
       17.134820       0.99647798     0.0018813329
       17.274519       0.99643491     0.0019231485
       17.415356       0.99639190     0.0019654380
       17.557342       0.99634973     0.0020078504
       17.700486       0.99630681     0.0020504917
       17.844796       0.99626295     0.0020928361
       17.990283       0.99621830     0.0021360904
       18.136956       0.99617270     0.0021803836
       18.284825       0.99612629     0.0022261089
       18.433899       0.99607892     0.0022736696
       18.584189       0.99603220     0.0023229989
       18.735704       0.99598674     0.0023745313
       18.888454       0.99594237     0.0024271781
       19.042450       0.99589963     0.0024810093
       19.197701       0.99585948     0.0025340758
       19.354218       0.99582255     0.0025860242
       19.512011       0.99578443     0.0026358389
       19.671091       0.99574504     0.0026832238
       19.831467       0.99570417     0.0027291330
       19.993151       0.99566184     0.0027735737
       20.156154       0.99561721     0.0028185313
       20.320485       0.99557045     0.0028640359
       20.486156       0.99552203     0.0029102677
       20.653177       0.99547213     0.0029572408
       20.821561       0.99542050     0.0030049712
       20.991317       0.99536742     0.0030534728
       21.162457       0.99531179     0.0031034961
       21.334993       0.99525418     0.0031548241
       21.508935       0.99519483     0.0032082383
       21.684295       0.99513408     0.0032632842
       21.861085       0.99507274     0.0033242935
       22.039316       0.99501067     0.0033890817
       22.219001       0.99495152     0.0034556209
       22.400150       0.99489341     0.0035236324
       22.582776       0.99483629     0.0035923262
       22.766892       0.99477920     0.0036620075
       22.952508       0.99472321     0.0037331896
       23.139637       0.99466725     0.0038057333
       23.328293       0.99461386     0.0038784835
       23.518486       0.99456096     0.0039521445
       23.710230       0.99450784     0.0040245047
       23.903537       0.99445426     0.0040973402
       24.098420       0.99440046     0.0041700171
       24.294892       0.99434618     0.0042435657
       24.492966       0.99429102     0.0043179552
       24.692655       0.99423519     0.0043935127
       24.893972       0.99417924     0.0044703615
       25.096930       0.99412278     0.0045485113
       25.301543       0.99406667     0.0046281388
       25.507824       0.99401012     0.0047091364
       25.715787       0.99395453     0.0047916134
       25.925445       0.99389856     0.0048754980
       26.136813       0.99384420     0.0049608810
       26.349904       0.99378952     0.0050477145
       26.564732       0.99373829     0.0051360255
       26.781312       0.99368689     0.0052258291
       26.999657       0.99364348     0.0053132780
       27.219783       0.99360016     0.0054020304
       27.441703       0.99355711     0.0054837973
       27.665432       0.99351382     0.0055666019
       27.890986       0.99346618     0.0056389600
       28.118378       0.99341802     0.0057123427
       28.347625       0.99336368     0.0057865858
       28.578740       0.99330867     0.0058618762
       28.811740       0.99324976     0.0059380658
       29.046639       0.99319010     0.0060153256
       29.283454       0.99312763     0.0060935148
       29.522199       0.99306424     0.0061728281
       29.762891       0.99299782     0.0062533546
       30.005545       0.99293058     0.0063352942
       30.250177       0.99286187     0.0064201394
       30.496804       0.99279239     0.0065061882
       30.745441       0.99272199     0.0065933237
       30.996106       0.99265072     0.0066816903
       31.248814       0.99257815     0.0067711754
       31.503583       0.99250463     0.0068619241
       31.760428       0.99242965     0.0069538391
       32.019368       0.99235360     0.0070470436
       32.280419       0.99227589     0.0071414352
       32.543598       0.99219688     0.0072371431
       32.808923       0.99211571     0.0073340687
       33.076411       0.99203246     0.0074323482
       33.346079       0.99194524     0.0075319089
       33.617947       0.99185685     0.0076361204
       33.892031       0.99176719     0.0077494251
       34.168349       0.99167805     0.0078646911
       34.446920       0.99159129     0.0079821756
       34.727763       0.99150470     0.0081014447
       35.010895       0.99141934     0.0082224863
       35.296335       0.99133387     0.0083453520
       35.584103       0.99124888     0.0084700262
       35.874217       0.99116390     0.0085965819
       36.166696       0.99107944     0.0087250148
       36.461559       0.99099539     0.0088549359
       36.758827       0.99091230     0.0089861619
       37.058518       0.99082864     0.0091187148
       37.360653       0.99074449     0.0092524231
       37.665250       0.99066009     0.0093880951
       37.972331       0.99057556     0.0095257470
       38.281916       0.99049108     0.0096654121
       38.594025       0.99040672     0.0098071167
       38.908678       0.99032266     0.0099509015
       39.225897       0.99023886      0.010096805
       39.545702       0.99015573      0.010244845
       39.868114       0.99007310      0.010395072
       40.193155       0.98999200      0.010547482
       40.520846       0.98991199      0.010702139
       40.851209       0.98983434      0.010856448
       41.184265       0.98975823      0.011011178
       41.520036       0.98968133      0.011166408
       41.858545       0.98960372      0.011322795
       42.199813       0.98952629      0.011481332
       42.543864       0.98944877      0.011642132
       42.890720       0.98937278      0.011805124
       43.240404       0.98929740      0.011970446
       43.592939       0.98922459      0.012138029
       43.948348       0.98915282      0.012308019
       44.306655       0.98908533      0.012480331
       44.667883       0.98901955      0.012655130
       45.032056       0.98896175      0.012832303
       45.399198       0.98890702      0.013012038
       45.769333       0.98887390      0.013191989
       46.142486       0.98884844      0.013373808
       46.518681       0.98881822      0.013507381
       46.897943       0.98878715      0.013627942
       47.280298       0.98873811      0.013749534
       47.665769       0.98868476      0.013872261
       48.054384       0.98861920      0.013996020
       48.446167       0.98855094      0.014120933
       48.841143       0.98847575      0.014246933
       49.239341       0.98839905      0.014374116
       49.640784       0.98831662      0.014502350
       50.045501       0.98823294      0.014631784
       50.453517       0.98814417      0.014762312
       50.864859       0.98805428      0.014894075
       51.279556       0.98795978      0.015026969
       51.697633       0.98786422      0.015161125
       52.119119       0.98776436      0.015296361
       52.544041       0.98766347      0.015432882
       52.972427       0.98755848      0.015570576
       53.404306       0.98745248      0.015709588
       53.839707       0.98734250      0.015849746
       54.278657       0.98723144      0.015991239
       54.721185       0.98711673      0.016133892
       55.167322       0.98700080      0.016277906
       55.617096       0.98688137      0.016423155
       56.070536       0.98676061      0.016569782
       56.527674       0.98663664      0.016717599
       56.988539       0.98651124      0.016866809
       57.453161       0.98638281      0.017017294
       57.921571       0.98625285      0.017169196
       58.393800       0.98612014      0.017322373
       58.869879       0.98598591      0.017476988
       59.349839       0.98584957      0.017632923
       59.833713       0.98571230      0.017790050
       60.321531       0.98557633      0.017947156
       60.813327       0.98543798      0.018104074
       61.309132       0.98529397      0.018255184
       61.808979       0.98514701      0.018407555
       62.312902       0.98499323      0.018561146
       62.820933       0.98483669      0.018716046
       63.333106       0.98467516      0.018872261
       63.849455       0.98451071      0.019029789
       64.370013       0.98434146      0.019188598
       64.894816       0.98416895      0.019348740
       65.423897       0.98399131      0.019510195
       65.957292       0.98380818      0.019674340
       66.495035       0.98361536      0.019842860
       67.037163       0.98342178      0.020016386
       67.583710       0.98322931      0.020198630
       68.134713       0.98303660      0.020382557
       68.690209       0.98284555      0.020568110
       69.250234       0.98265378      0.020755424
       69.814824       0.98246239      0.020944476
       70.384018       0.98227571      0.021133851
       70.957852       0.98209738      0.021322761
       71.536364       0.98191341      0.021500465
       72.119593       0.98172242      0.021662119
       72.707578       0.98152262      0.021824959
       73.300355       0.98131233      0.021989018
       73.897966       0.98109665      0.022154284
       74.500449       0.98087517      0.022320777
       75.107844       0.98065461      0.022486943
       75.720191       0.98043507      0.022652735
       76.337530       0.98020460      0.022806154
       76.959903       0.97996363      0.022947969
       77.587350       0.97970754      0.023090701
       78.219912       0.97943795      0.023234260
       78.857631       0.97915488      0.023378844
       79.500550       0.97886039      0.023524311
       80.148710       0.97854394      0.023670748
       80.802155       0.97821057      0.023817997
       81.460927       0.97785344      0.023993765
       82.125070       0.97747902      0.024188081
       82.794628       0.97712171      0.024410610
       83.469644       0.97677166      0.024650234
       84.150164       0.97643306      0.024892107
       84.836232       0.97609775      0.025136443
       85.527894       0.97577297      0.025382973
       86.225195       0.97545042      0.025631981
       86.928180       0.97515974      0.025870539
       87.636897       0.97487981      0.026106261
       88.351392       0.97457267      0.026285631
       89.071713       0.97425294      0.026445060
       89.797906       0.97390431      0.026605766
       90.530020       0.97354294      0.026767164
       91.268102       0.97316013      0.026929827
       92.012202       0.97276711      0.027093152
       92.762369       0.97235602      0.027257814
       93.518651       0.97193567      0.027423117
       94.281100       0.97149807      0.027589866
       95.049764       0.97105138      0.027757232
       95.824696       0.97058686      0.027926009
       96.605945       0.97011306      0.028095379
       97.393564       0.96962078      0.028266110
       98.187605       0.96911897      0.028437403
       98.988119       0.96859831      0.028611823
       99.795159       0.96806793      0.028786999
       100.60878       0.96751879      0.028963624
       101.42903       0.96695972      0.029140808
       102.25597       0.96638045      0.029319689
       103.08966       0.96579089      0.029499119
       103.93014       0.96518660      0.029680198
       104.77747       0.96457192      0.029861716
       105.63171       0.96393676      0.030023840
       106.49292       0.96328967      0.030185550
       107.36114       0.96259398      0.030332909
       108.23645       0.96188419      0.030479954
       109.11889       0.96112643      0.030628624
       110.00852       0.96035045      0.030777047
       110.90541       0.95950250      0.030927143
       111.80961       0.95863586      0.031083593
       112.72119       0.95772447      0.031301639
       113.64019       0.95679670      0.031523190
       114.56669       0.95585511      0.031762443
       115.50074       0.95489296      0.032003419
       116.44241       0.95389442      0.032246435
       117.39175       0.95286610      0.032491160
       118.34883       0.95176370      0.032737716
       119.31372       0.95062197      0.033003196
       120.28647       0.94938183      0.033341966
       121.26716       0.94813079      0.033712509
       122.25584       0.94690212      0.034187623
       123.25258       0.94565804      0.034669662
       124.25744       0.94441497      0.035158265
       125.27050       0.94312636      0.035653973
       126.29182       0.94175212      0.036156468
       127.32147       0.94036104      0.036721890
       128.35951       0.93896639      0.037431916
       129.40601       0.93757273      0.038155914
       130.46104       0.93621260      0.038895855
       131.52468       0.93483793      0.039649720
       132.59699       0.93345864      0.040418589
       133.67804       0.93205811      0.041202003
       134.76790       0.93063901      0.042000967
       135.86665       0.92918611      0.042815085
       136.97436       0.92769398      0.043645247
       138.09110       0.92617805      0.044523553
       139.21694       0.92463943      0.045465868
       140.35196       0.92308439      0.046431959
       141.49624       0.92151626      0.047424307
       142.64984       0.91992592      0.048437595
       143.81285       0.91831411      0.049472736
       144.98535       0.91666799      0.050529860
       146.16740       0.91498650      0.051609616
       147.35909       0.91325518      0.052712501
       148.56049       0.91147346      0.053838865
       149.77169       0.90957838      0.054998140
       150.99277       0.90757643      0.056189611
       152.22379       0.90557040      0.057589083
       153.46486       0.90355410      0.059165552
       154.71604       0.90156364      0.060787102
       155.97743       0.89958311      0.062450825
       157.24910       0.89760653      0.064162715
       158.53114       0.89562098      0.065918481
       159.82362       0.89362137      0.067725777
       161.12665       0.89159921      0.069578792
       162.44030       0.88950059      0.071501958
       163.76466       0.88734283      0.073481206
       165.09982       0.88518972      0.075609367
       166.44586       0.88301958      0.077833115
       167.80288       0.88087694      0.080130639
       169.17096       0.87872568      0.082486815
       170.55019       0.87655956      0.084922425
       171.94067       0.87436498      0.087419241
       173.34249       0.87213613      0.090000939
       174.75573       0.86987043      0.092646205
       176.18050       0.86752663      0.095382970
       177.61688       0.86513093      0.098185980
       179.06498       0.86246604       0.10127595
       180.52487       0.85969482       0.10448827
       181.99668       0.85707012       0.10823467
       183.48048       0.85444320       0.11216280
       184.97638       0.85210513       0.11627662
       186.48447       0.84979149       0.12049571
       188.00486       0.84761800       0.12491605
       189.53765       0.84543842       0.12944616
       191.08293       0.84332964       0.13419728
       192.64081       0.84120247       0.13906318
       194.21139       0.83919138       0.14435783
       195.79478       0.83716575       0.14979398
       197.39107       0.83554780       0.15574428
       199.00038       0.83393987       0.16184624
       200.62281       0.83281260       0.16829231
       202.25847       0.83169847       0.17488295
       203.90747       0.83127665       0.18185113
       205.56990       0.83088271       0.18896899
       207.24590       0.83110728       0.19618304
       208.93555       0.83138365       0.20355695
       210.63898       0.83232482       0.21130376
       212.35630       0.83338664       0.21922926
       214.08762       0.83567540       0.22756920
       215.83306       0.83813337       0.23600257
       217.59272       0.84166888       0.24371558
       219.36674       0.84536935       0.25156095
       221.15521       0.84965350       0.25935565
       222.95827       0.85455279       0.26733003
       224.77603       0.86283400       0.27561154
       226.60860       0.87131653       0.28310577
       228.45612       0.87990923       0.28553454
       230.31870       0.88822326       0.28793007
       232.19647       0.89442779       0.29009352
       234.08954       0.90064464       0.29227235
       235.99805       0.90641822       0.29446885
       237.92212       0.91239184       0.29668087
       239.86188       0.91867917       0.29891002
       241.81745       0.92504158       0.30085885
       243.78896       0.93129894       0.30193022
       245.77655       0.93752703       0.30297857
       247.78034       0.94340711       0.30398790
       249.80047       0.94940900       0.30499281
       251.83707       0.95549312       0.30600836
       253.89028       0.96214918       0.30702003
       255.96022       0.96985113       0.30804104
       258.04704       0.97820595       0.30823389
       260.15087       0.98763911       0.30685215
       262.27186       0.99634203       0.30401901
       264.41013        1.0036143       0.29872706
       266.56584        1.0101546       0.29346704
       268.73913        1.0154502       0.28835014
       270.93013        1.0204257       0.28309305
       273.13900        1.0248956       0.27770241
       275.36588        1.0289428       0.27217894
       277.61091        1.0324163       0.26652940
       279.87424        1.0361334       0.26097169
       282.15603        1.0401237       0.25554885
       284.45642        1.0438975       0.24884865
       286.77557        1.0474357       0.24079225
       289.11362        1.0494049       0.23205951
       291.47073        1.0498525       0.22265816
       293.84707        1.0491180       0.21366343
       296.24277        1.0473378       0.20499311
       298.65801        1.0447661       0.19672534
       301.09294        1.0415465       0.18873002
       303.54772        1.0375654       0.18113005
       306.02252        1.0330025       0.17375613
       308.51749        1.0275681       0.16677119
       311.03281        1.0215128       0.15997217
       313.56863        1.0140270       0.15393728
       316.12512        1.0056146       0.14828433
       318.70246       0.99760008       0.14525392
       321.30081       0.98970341       0.14357638
       323.92034       0.98265034       0.14194514
       326.56123       0.97590936       0.14030461
       329.22366       0.96893952       0.13871203
       331.90778       0.96177444       0.13710627
       334.61380       0.95456568       0.13636306
       337.34187       0.94723245       0.13592356
       340.09219       0.94039869       0.13587181
       342.86492       0.93362373       0.13593647
       345.66027       0.92699051       0.13600997
       348.47840       0.92028882       0.13607433
       351.31951       0.91303682       0.13614830
       354.18379       0.90552025       0.13621236
       357.07142       0.89758022       0.13725229
       359.98258       0.88940578       0.13850763
       362.91749       0.88188161       0.14093283
       365.87632       0.87433441       0.14360193
       368.85927       0.86738167       0.14632454
       371.86655       0.86038399       0.14909576
       374.89834       0.85334140       0.15192233
       377.95485       0.84615699       0.15479909
       381.03628       0.83907531       0.15833975
       384.14283       0.83186209       0.16199745
       387.27471       0.82515043       0.16602267
       390.43212       0.81833281       0.17014152
       393.61528       0.81170385       0.17438401
       396.82438       0.80494602       0.17870911
       400.05966       0.79838119       0.18361741
       403.32130       0.79169538       0.18862992
       406.60954       0.78548564       0.19402036
%        409.92459       0.77918091       0.19952200
%        413.26667       0.77339345       0.20522283
%        413.267      0.750000      0.295000
%        416.740      0.742000      0.296000
       420.271      0.733000      0.297000
       423.863      0.724000      0.298000
       427.517      0.712000      0.301000
       431.235      0.700000      0.306000
       435.018      0.690000      0.310000
       438.867      0.679000      0.318000
       442.786      0.671000      0.322000
       446.775      0.656000      0.328000
       450.836      0.643000      0.336000
       454.972      0.631000      0.346000
       459.185      0.619000      0.357000
       462.612      0.610000      0.365000
       466.090      0.600000      0.375000
       469.621      0.591000      0.383000
       473.206      0.580000      0.394000
       476.846      0.571000      0.404000
       480.543      0.560000      0.415000
       484.297      0.548000      0.429000
       488.110      0.537000      0.444000
       491.984      0.528000      0.461000
       495.920      0.521000      0.476000
       499.919      0.511000      0.494000
       503.984      0.503000      0.513000
       508.115      0.497000      0.534000
       512.314      0.491000      0.554000
       516.583      0.487000      0.575000
       520.924      0.483000      0.598000
       525.339      0.484000      0.621000
       529.829      0.486000      0.643000
       534.397      0.490000      0.663000
       539.043      0.494000      0.679000
       543.772      0.494000      0.695000
       548.584      0.493000      0.712000
       553.482      0.492000      0.730000
       558.468      0.491000      0.751000
       563.545      0.491000      0.773000
       568.716      0.492000      0.796000
       573.981      0.496000      0.819000
       579.346      0.499000      0.842000
       584.811      0.503000      0.866000
       590.381      0.508000      0.892000
       596.058      0.517000      0.918000
       601.845      0.525000      0.943000
       607.745      0.537000      0.969000
       613.762      0.549000      0.994000
       619.900      0.563000       1.02000
       626.162      0.579000       1.05000
       632.551      0.599000       1.07000
       639.072      0.621000       1.09000
       645.729      0.642000       1.11000
       652.526      0.665000       1.13000
       659.468      0.688000       1.15000
       666.559      0.712000       1.16000
       673.804      0.738000       1.18000
       681.209      0.766000       1.19000
       688.778      0.796000       1.20000
       696.517      0.822000       1.20000
       704.432      0.847000       1.21000
       712.529      0.874000       1.21000
       720.814      0.900000       1.20000
       729.294      0.922000       1.20000
       737.976      0.944000       1.19000
       746.867      0.959000       1.18000
       755.976      0.973000       1.17000
       765.309      0.980000       1.15000
       774.875      0.981000       1.14000
       784.684      0.976000       1.13000
       794.744      0.966000       1.12000
       805.065      0.951000       1.12000
       815.658      0.933000       1.13000
       826.533      0.917000       1.14000
       837.703      0.904000       1.17000
       849.178      0.900000       1.20000
       860.972      0.901000       1.23000
       873.099      0.906000       1.25000
       885.571      0.913000       1.28000
       898.406      0.923000       1.32000
       911.618      0.941000       1.35000
       925.224      0.962000       1.37000
       939.242      0.984000       1.40000
       953.692       1.01000       1.42000
       968.594       1.04000       1.44000
       983.968       1.07000       1.46000
       999.839       1.10000       1.47000
       1016.23       1.13000       1.48000
       1033.17       1.16000       1.48000
       1041.85       1.18000       1.48000
       1050.68       1.20000       1.48000
       1059.66       1.21000       1.48000
       1068.79       1.22000       1.48000
       1078.09       1.24000       1.47000
       1087.54       1.25000       1.46000
       1097.17       1.26000       1.45000
       1106.96       1.27000       1.44000
       1116.94       1.28000       1.43000
       1127.09       1.28000       1.42000
       1137.43       1.29000       1.41000
       1147.96       1.29000       1.39000
       1158.69       1.28000       1.38000
       1169.62       1.28000       1.36000
       1180.76       1.26000       1.34000
       1192.12       1.24000       1.33000
       1203.69       1.22000       1.33000
       1215.49       1.19000       1.33000
       1227.52       1.16000       1.33000
       1239.80       1.13000       1.34000
       1252.32       1.11000       1.36000
       1265.10       1.09000       1.38000
       1278.14       1.07000       1.41000
       1291.46       1.05000       1.44000
       1305.05       1.04000       1.47000
       1318.94       1.03000       1.50000
       1333.12       1.03000       1.52000
       1347.61       1.02000       1.55000
       1362.42       1.02000       1.57000
       1377.56       1.01000       1.60000
       1393.03       1.02000       1.63000
       1408.86       1.01000       1.65000
       1425.06      0.999000       1.67000
       1441.63      0.986000       1.70000
       1458.59      0.973000       1.73000
       1475.95      0.956000       1.76000
       1493.73      0.943000       1.81000
       1511.95      0.935000       1.86000
       1530.62      0.935000       1.91000
       1549.75      0.937000       1.95000
       1569.37      0.932000       2.00000
       1589.49      0.930000       2.06000
       1610.13      0.937000       2.12000
       1631.32      0.950000       2.18000
       1653.07      0.964000       2.24000
       1675.41      0.979000       2.29000
       1698.36      0.993000       2.36000
       1721.94       1.01000       2.43000
       1746.20       1.04000       2.49000
       1771.14       1.06000       2.56000
       1796.81       1.09000       2.63000
       1823.24       1.12000       2.70000
       1850.45       1.16000       2.78000
       1878.48       1.21000       2.87000
       1907.38       1.26000       2.95000
       1937.19       1.31000       3.04000
       1967.94       1.38000       3.14000
       1999.68       1.47000       3.24000
       2032.46       1.58000       3.33000
       2066.33       1.70000       3.42000
       2101.36       1.83000       3.52000
       2137.59       2.00000       3.61000
       2175.09       2.20000       3.67000
       2213.93       2.43000       3.70000
       2254.18       2.68000       3.68000
       2295.93       2.92000       3.58000
       2339.25       3.11000       3.43000
       2384.23       3.27000       3.27000
       2430.98       3.38000       3.05000
       2479.60       3.40000       2.85000
       2530.20       3.37000       2.68000
       2582.92       3.33000       2.57000
       2637.87       3.29000       2.48000
       2695.22       3.24000       2.41000
       2755.11       3.19000       2.35000
       2817.73       3.13000       2.32000
       2883.26       3.07000       2.31000
       2951.90       3.01000       2.33000
       3023.90       2.97000       2.37000
       3099.50       2.95000       2.43000
       3138.73       2.95000       2.46000
       3178.97       2.96000       2.50000
       3220.26       2.98000       2.53000
       3262.63       2.99000       2.56000
       3306.13       3.02000       2.60000
       3350.81       3.05000       2.62000
       3396.71       3.09000       2.65000
       3443.89       3.13000       2.67000
       3492.39       3.18000       2.69000
       3542.29       3.24000       2.70000
       3593.62       3.32000       2.70000
       3646.47       3.39000       2.66000
       3700.90       3.43000       2.60000
       3756.97       3.45000       2.55000
       3814.77       3.45000       2.49000
       3874.38       3.43000       2.45000
       3935.87       3.41000       2.43000
       3999.35       3.39000       2.41000
       4064.92       3.37000       2.42000
       4132.67       3.35000       2.42000
       4202.71       3.33000       2.43000
       4275.17       3.32000       2.45000
       4350.18       3.31000       2.47000
       4427.86       3.30000       2.49000
       4508.36       3.31000       2.53000
       4591.85       3.31000       2.55000
       4678.49       3.32000       2.59000
       4768.46       3.34000       2.62000
       4861.96       3.35000       2.64000
       4959.20       3.38000       2.68000
       5060.41       3.42000       2.71000
       5165.83       3.45000       2.72000
       5275.75       3.48000       2.72000
       5390.44       3.50000       2.72000
       5510.22       3.50000       2.73000
       5635.45       3.49000       2.75000
       5766.51       3.51000       2.81000
       5903.81       3.54000       2.84000
       6047.81       3.57000       2.86000
       6199.00       3.60000       2.89000
       6357.95       3.65000       2.92000
       6525.26       3.70000       2.94000
       6701.62       3.76000       2.95000
       6887.78       3.82000       2.91000
       7084.57       3.85000       2.86000
       7292.94       3.84000       2.78000
       7513.94       3.78000       2.72000
       7748.75       3.67000       2.68000
       7998.71       3.56000       2.73000
       8265.33       3.48000       2.79000
       8550.34       3.38000       2.85000
       8855.71       3.29000       2.96000
       9183.70       3.20000       3.07000
       9536.92       3.12000       3.24000
       9918.40       3.05000       3.39000
       10331.7       3.00000       3.64000
       10780.9       3.02000       3.83000
       11270.9       3.05000       4.04000
       11807.6       3.11000       4.19000
       12398.0       3.14000       4.32000
       12651.0       3.15000       4.36000
       12914.6       3.15000       4.41000
       13189.4       3.15000       4.43000
       13476.1       3.14000       4.45000
       13775.6       3.11000       4.44000
       14088.6       3.05000       4.42000
       14250.6       3.00000       4.39000
       14416.3       2.92000       4.37000
       14585.9       2.80000       4.33000
       14759.5       2.62000       4.37000
       14937.3       2.47000       4.47000
       15119.5       2.36000       4.61000
       15497.5       2.22000       4.85000
       15894.9       2.12000       5.05000
       16313.2       1.97000       5.27000
       16754.1       1.83000       5.52000
       17219.4       1.70000       5.81000
       17711.4       1.59000       6.13000
       18232.4       1.52000       6.45000
       18784.8       1.45000       6.78000
       19371.9       1.36000       7.14000
       19996.8       1.28000       7.52000
       20663.3       1.21000       7.96000
       21375.9       1.18000       8.44000
       22139.3       1.20000       8.94000
       22959.3       1.23000       9.45000
       23842.3       1.30000       9.98000
       24796.0       1.40000       10.5000
       25829.2       1.54000       11.1000
       26952.2       1.69000       11.6000
       28177.3       1.82000       12.1000
       29519.0       1.92000       12.6000
       30995.0       1.94000       13.2000
       32626.3       1.86000       13.9000
       34438.9       1.82000       14.7000
       36464.7       1.71000       15.7000
       38743.8       1.72000       16.9000
       41326.7       1.83000       18.3000
       49592.0       2.56000       22.4000
       61990.0       3.87000       28.3000
       68877.8       4.72000       31.5000
       77487.5       5.92000       35.3000
       88557.1       7.58000       40.2000
       103317.       10.1000       46.4000
       123980.       14.1000       54.7000
       137756.       17.0000       60.0000
       154975.       20.9000       66.3000
       177114.       26.5000       73.8000
       206633.       34.5000       82.9000
       247960.       46.5000       93.7000];



lld=AAA(:,1)*1.e-4;nn=AAA(:,2).*(1+i*AAA(:,3));
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 
%------------------------------------------------------


case {'Si'}
bornes=[0.00012398,3332];ref='Si llnl cxro + Si palik';
AAA=[
      0.12398000       0.99999995    5.6886657e-12
      0.12499080       0.99999995    5.8869953e-12
      0.12600983       0.99999995    6.0924737e-12
      0.12703718       0.99999995    6.3052263e-12
      0.12807290       0.99999995    6.5246936e-12
      0.12911707       0.99999995    6.7523669e-12
      0.13016975       0.99999995    6.9870917e-12
      0.13123101       0.99999995    7.2303850e-12
      0.13230092       0.99999995    7.4821575e-12
      0.13337956       0.99999994    7.7423847e-12
      0.13446699       0.99999994    8.0137516e-12
      0.13556329       0.99999994    8.2912375e-12
      0.13666852       0.99999994    8.5819229e-12
      0.13778277       0.99999994    8.8788071e-12
      0.13890610       0.99999994    9.1887500e-12
      0.14003858       0.99999994    9.5090412e-12
      0.14118030       0.99999994    9.8401942e-12
      0.14233133       0.99999994    1.0184318e-11
      0.14349175       0.99999994    1.0537031e-11
      0.14466162       0.99999993    1.0906233e-11
      0.14584103       0.99999993    1.1283103e-11
      0.14703006       0.99999993    1.1677256e-11
      0.14822878       0.99999993    1.2084036e-11
      0.14943728       0.99999993    1.2503752e-11
      0.15065563       0.99999993    1.2942665e-11
      0.15188391       0.99999993    1.3390683e-11
      0.15312220       0.99999993    1.3859699e-11
      0.15437059       0.99999993    1.4341252e-11
      0.15562916       0.99999992    1.4838519e-11
      0.15689799       0.99999992    1.5354858e-11
      0.15817716       0.99999992    1.5890793e-11
      0.15946677       0.99999992    1.6446371e-11
      0.16076688       0.99999992    1.7013427e-11
      0.16207760       0.99999992    1.7608646e-11
      0.16339900       0.99999992    1.8218617e-11
      0.16473118       0.99999991    1.8852957e-11
      0.16607422       0.99999991    1.9509898e-11
      0.16742821       0.99999991    2.0188197e-11
      0.16879323       0.99999991    2.0890060e-11
      0.17016939       0.99999991    2.1617159e-11
      0.17155676       0.99999991    2.2371068e-11
      0.17295545       0.99999991    2.3150833e-11
      0.17436554       0.99999990    2.3954437e-11
      0.17578712       0.99999990    2.4790931e-11
      0.17722030       0.99999990    2.5649810e-11
      0.17866516       0.99999990    2.6544934e-11
      0.18012180       0.99999990    2.7465482e-11
      0.18159031       0.99999990    2.8421010e-11
      0.18307080       0.99999989    2.9406273e-11
      0.18456336       0.99999989    3.0427835e-11
      0.18606809       0.99999989    3.1486912e-11
      0.18758508       0.99999989    3.2580897e-11
      0.18911444       0.99999989    3.3714507e-11
      0.19065628       0.99999989    3.4881199e-11
      0.19221068       0.99999988    3.6100224e-11
      0.19377775       0.99999988    3.7347320e-11
      0.19535760       0.99999988    3.8652798e-11
      0.19695034       0.99999988    3.9988382e-11
      0.19855605       0.99999988    4.1379639e-11
      0.20017486       0.99999987    4.2820770e-11
      0.20180687       0.99999987    4.4303912e-11
      0.20345218       0.99999987    4.5844954e-11
      0.20511091       0.99999987    4.7441796e-11
      0.20678315       0.99999987    4.9082912e-11
      0.20846904       0.99999986    5.0789029e-11
      0.21016867       0.99999986    5.2561860e-11
      0.21188215       0.99999986    5.4371550e-11
      0.21360961       0.99999986    5.6258302e-11
      0.21535114       0.99999985    5.8210360e-11
      0.21710688       0.99999985    6.0237475e-11
      0.21887693       0.99999985    6.2325475e-11
      0.22066141       0.99999985    6.4487582e-11
      0.22246045       0.99999984    6.6719228e-11
      0.22427414       0.99999984    6.9026255e-11
      0.22610263       0.99999984    7.1551114e-11
      0.22794602       0.99999984    7.4034717e-11
      0.22980444       0.99999983    7.6598101e-11
      0.23167802       0.99999983    7.9248754e-11
      0.23356687       0.99999983    8.1995396e-11
      0.23547111       0.99999983    8.4836391e-11
      0.23739089       0.99999982    8.7774812e-11
      0.23932631       0.99999982    9.0813815e-11
      0.24127752       0.99999982    9.3956287e-11
      0.24324463       0.99999981    9.7205700e-11
      0.24522778       0.99999981    1.0057307e-10
      0.24722710       0.99999981    1.0405934e-10
      0.24924272       0.99999981    1.0765883e-10
      0.25127477       0.99999980    1.1137688e-10
      0.25332339       0.99999980    1.1522002e-10
      0.25538871       0.99999980    1.1919943e-10
      0.25747087       0.99999979    1.2332352e-10
      0.25957000       0.99999979    1.2758602e-10
      0.26168625       0.99999979    1.3199088e-10
      0.26381975       0.99999978    1.3654185e-10
      0.26597065       0.99999978    1.4126843e-10
      0.26813908       0.99999977    1.4615357e-10
      0.27032520       0.99999977    1.5120727e-10
      0.27252913       0.99999977    1.5643653e-10
      0.27475103       0.99999976    1.6184158e-10
      0.27699105       0.99999976    1.6743358e-10
      0.27924933       0.99999976    1.7318564e-10
      0.28152603       0.99999975    1.7917004e-10
      0.28382128       0.99999975    1.8533718e-10
      0.28613525       0.99999974    1.9175178e-10
      0.28846808       0.99999974    1.9831037e-10
      0.29081993       0.99999973    2.0518710e-10
      0.29319096       0.99999973    2.1221234e-10
      0.29558132       0.99999973    2.1953216e-10
      0.29799117       0.99999972    2.2710554e-10
      0.30042066       0.99999972    2.3493780e-10
      0.30286996       0.99999971    2.4302752e-10
      0.30533923       0.99999971    2.5141338e-10
      0.30782863       0.99999970    2.6007694e-10
      0.31033833       0.99999970    2.6903247e-10
      0.31286848       0.99999969    2.7829842e-10
      0.31541927       0.99999969    2.8787667e-10
      0.31799085       0.99999968    2.9774117e-10
      0.32058340       0.99999968    3.0801635e-10
      0.32319709       0.99999967    3.1860536e-10
      0.32583208       0.99999967    3.2954092e-10
      0.32848856       0.99999966    3.4088956e-10
      0.33116670       0.99999966    3.5256249e-10
      0.33386667       0.99999965    3.6468455e-10
      0.33658865       0.99999964    3.7721021e-10
      0.33933282       0.99999964    3.9012077e-10
      0.34209937       0.99999963    4.0355198e-10
      0.34488847       0.99999963    4.1736797e-10
      0.34770032       0.99999962    4.3169030e-10
      0.35053508       0.99999961    4.4651793e-10
      0.35339296       0.99999961    4.6186272e-10
      0.35627414       0.99999960    4.7768865e-10
      0.35917881       0.99999960    4.9401781e-10
      0.36210716       0.99999959    5.1098294e-10
      0.36505938       0.99999958    5.2846545e-10
      0.36803568       0.99999957    5.4658499e-10
      0.37103624       0.99999957    5.6526357e-10
      0.37406126       0.99999956    5.8468202e-10
      0.37711095       0.99999955    6.0468956e-10
      0.38018549       0.99999955    6.2530808e-10
      0.38328511       0.99999954    6.4670607e-10
      0.38641000       0.99999953    6.6888093e-10
      0.38956036       0.99999952    6.9171937e-10
      0.39273641       0.99999952    7.1536787e-10
      0.39593835       0.99999951    7.3982851e-10
      0.39916640       0.99999950    7.6516992e-10
      0.40242076       0.99999949    7.9124284e-10
      0.40570166       0.99999948    8.1840029e-10
      0.40900931       0.99999947    8.4429647e-10
      0.41234392       0.99999947    8.6734561e-10
      0.41570572       0.99999946    8.9536391e-10
      0.41909493       0.99999945    9.2568993e-10
      0.42251177       0.99999944    9.5723489e-10
      0.42595647       0.99999943    9.8963189e-10
      0.42942925       0.99999942    1.0233469e-09
      0.43293034       0.99999941    1.0579496e-09
      0.43645998       0.99999940    1.0939818e-09
      0.44001840       0.99999939    1.1309418e-09
      0.44360583       0.99999938    1.1694546e-09
      0.44722250       0.99999937    1.2089357e-09
      0.45086866       0.99999936    1.2500920e-09
      0.45454455       0.99999935    1.2922577e-09
      0.45825041       0.99999934    1.3362394e-09
      0.46198648       0.99999933    1.3812739e-09
      0.46575301       0.99999932    1.4282767e-09
      0.46955025       0.99999931    1.4763736e-09
      0.47337845       0.99999930    1.5266074e-09
      0.47723786       0.99999928    1.5779821e-09
      0.48112873       0.99999927    1.6316568e-09
      0.48505133       0.99999926    1.6865167e-09
      0.48900590       0.99999925    1.7438701e-09
      0.49299272       0.99999924    1.8024541e-09
      0.49701204       0.99999922    1.8637335e-09
      0.50106413       0.99999921    1.9263128e-09
      0.50514926       0.99999920    1.9917709e-09
      0.50926770       0.99999919    2.0586390e-09
      0.51341971       0.99999917    2.1285394e-09
      0.51760557       0.99999916    2.1999903e-09
      0.52182555       0.99999914    2.2746342e-09
      0.52607995       0.99999913    2.3509788e-09
      0.53036903       0.99999912    2.4306705e-09
      0.53469307       0.99999910    2.5122287e-09
      0.53905238       0.99999909    2.5973285e-09
      0.54344722       0.99999907    2.6844722e-09
      0.54787789       0.99999906    2.7753365e-09
      0.55234469       0.99999904    2.8684371e-09
      0.55684790       0.99999903    2.9654525e-09
      0.56138783       0.99999901    3.0649140e-09
      0.56596477       0.99999899    3.1684925e-09
      0.57057902       0.99999898    3.2747468e-09
      0.57523090       0.99999896    3.3853233e-09
      0.57992070       0.99999894    3.4988211e-09
      0.58464874       0.99999893    3.6168797e-09
      0.58941532       0.99999891    3.7381313e-09
      0.59422077       0.99999889    3.8641680e-09
      0.59906539       0.99999887    3.9936945e-09
      0.60394951       0.99999885    4.1282591e-09
      0.60887346       0.99999883    4.2666210e-09
      0.61383754       0.99999881    4.4102598e-09
      0.61884210       0.99999880    4.5580620e-09
      0.62388746       0.99999878    4.7114522e-09
      0.62897395       0.99999876    4.8693004e-09
      0.63410192       0.99999873    5.0328512e-09
      0.63927169       0.99999871    5.2012936e-09
      0.64448361       0.99999869    5.3757948e-09
      0.64973802       0.99999867    5.5556087e-09
      0.65503527       0.99999865    5.7417577e-09
      0.66037571       0.99999863    5.9336799e-09
      0.66575969       0.99999860    6.1322729e-09
      0.67118757       0.99999858    6.3371225e-09
      0.67665969       0.99999856    6.5489254e-09
      0.68217644       0.99999853    6.7675379e-09
      0.68773815       0.99999851    6.9934740e-09
      0.69334522       0.99999849    7.2268114e-09
      0.69899799       0.99999846    7.4678225e-09
      0.70469686       0.99999844    7.7168261e-09
      0.71044218       0.99999841    7.9738223e-09
      0.71623435       0.99999838    8.2395481e-09
      0.72207374       0.99999836    8.5136988e-09
      0.72796073       0.99999833    8.7972938e-09
      0.73389573       0.99999830    9.0896738e-09
      0.73987911       0.99999828    9.3922780e-09
      0.74591127       0.99999825    9.7040825e-09
      0.75199261       0.99999822    1.0026986e-08
      0.75812354       0.99999819    1.0359524e-08
      0.76430445       0.99999816    1.0704027e-08
      0.77053575       0.99999813    1.1058589e-08
      0.77681785       0.99999810    1.1426166e-08
      0.78315117       0.99999807    1.1804292e-08
      0.78953613       0.99999803    1.2196521e-08
      0.79597314       0.99999800    1.2599752e-08
      0.80246263       0.99999797    1.3018236e-08
      0.80900503       0.99999794    1.3448199e-08
      0.81560077       0.99999790    1.3894634e-08
      0.82225028       0.99999787    1.4353056e-08
      0.82895401       0.99999783    1.4829338e-08
      0.83571239       0.99999780    1.5318137e-08
      0.84252588       0.99999776    1.5826136e-08
      0.84939491       0.99999772    1.6347142e-08
      0.85631994       0.99999769    1.6889166e-08
      0.86330144       0.99999765    1.7444846e-08
      0.87033985       0.99999761    1.8022946e-08
      0.87743565       0.99999757    1.8615209e-08
      0.88458929       0.99999753    1.9231991e-08
      0.89180126       0.99999749    1.9863585e-08
      0.89907203       0.99999745    2.0521328e-08
      0.90640208       0.99999740    2.1194423e-08
      0.91379189       0.99999736    2.1896031e-08
      0.92124195       0.99999732    2.2613681e-08
      0.92875274       0.99999727    2.3361923e-08
      0.93632477       0.99999723    2.4126809e-08
      0.94395854       0.99999718    2.4925062e-08
      0.95165454       0.99999714    2.5740662e-08
      0.95941329       0.99999709    2.6591702e-08
      0.96723529       0.99999704    2.7460724e-08
      0.97512107       0.99999699    2.8368475e-08
      0.98307114       0.99999694    2.9295331e-08
      0.99108602       0.99999689    3.0262843e-08
      0.99916625       0.99999684    3.1251229e-08
       1.0073124       0.99999679    3.2282637e-08
       1.0155249       0.99999673    3.3336866e-08
       1.0238044       0.99999668    3.4435744e-08
       1.0321513       0.99999663    3.5559600e-08
       1.0405664       0.99999657    3.6731064e-08
       1.0490500       0.99999651    3.7929778e-08
       1.0576028       0.99999645    3.9178060e-08
       1.0662253       0.99999640    4.0456062e-08
       1.0749182       0.99999634    4.1786256e-08
       1.0836819       0.99999627    4.3148947e-08
       1.0925170       0.99999621    4.4566615e-08
       1.1014242       0.99999615    4.6019792e-08
       1.1104040       0.99999609    4.7530600e-08
       1.1194570       0.99999602    4.9079963e-08
       1.1285839       0.99999595    5.0689416e-08
       1.1377851       0.99999589    5.2340922e-08
       1.1470613       0.99999582    5.4055963e-08
       1.1564132       0.99999575    5.5816775e-08
       1.1658413       0.99999568    5.7644213e-08
       1.1753463       0.99999560    5.9521355e-08
       1.1849288       0.99999553    6.1468107e-08
       1.1945894       0.99999545    6.3469063e-08
       1.2043288       0.99999538    6.5543494e-08
       1.2141476       0.99999530    6.7676657e-08
       1.2240464       0.99999522    6.9886444e-08
       1.2340259       0.99999514    7.2160278e-08
       1.2440868       0.99999505    7.4514822e-08
       1.2542298       0.99999496    7.6938680e-08
       1.2644554       0.99999487    7.9446998e-08
       1.2747644       0.99999478    8.2030438e-08
       1.2851574       0.99999470    8.4702177e-08
       1.2956351       0.99999462    8.7455757e-08
       1.3061983       0.99999454    9.0302302e-08
       1.3168476       0.99999445    9.3237004e-08
       1.3275838       0.99999436    9.6268305e-08
       1.3384074       0.99999427    9.9395911e-08
       1.3493193       0.99999418    1.0262562e-07
       1.3603202       0.99999408    1.0595885e-07
       1.3714108       0.99999399    1.0939817e-07
       1.3825918       0.99999389    1.1295000e-07
       1.3938639       0.99999379    1.1661311e-07
       1.4052279       0.99999368    1.2039795e-07
       1.4166846       0.99999358    1.2429915e-07
       1.4282347       0.99999348    1.2833246e-07
       1.4398790       0.99999337    1.3248796e-07
       1.4516182       0.99999326    1.3678624e-07
       1.4634531       0.99999315    1.4121184e-07
       1.4753845       0.99999304    1.4579049e-07
       1.4874131       0.99999292    1.5050139e-07
       1.4995399       0.99999280    1.5537964e-07
       1.5117655       0.99999268    1.6039702e-07
       1.5240907       0.99999256    1.6559541e-07
       1.5365165       0.99999244    1.7093895e-07
       1.5490436       0.99999232    1.7647640e-07
       1.5616728       0.99999219    1.8216511e-07
       1.5744049       0.99999206    1.8806475e-07
       1.5872409       0.99999193    1.9412268e-07
       1.6001815       0.99999180    2.0040643e-07
       1.6132277       0.99999166    2.0685467e-07
       1.6263801       0.99999152    2.1354789e-07
       1.6396399       0.99999138    2.2041303e-07
       1.6530077       0.99999124    2.2754317e-07
       1.6664845       0.99999110    2.3485257e-07
       1.6800712       0.99999095    2.4244776e-07
       1.6937687       0.99999080    2.5022961e-07
       1.7075778       0.99999065    2.5831791e-07
       1.7214995       0.99999049    2.6660017e-07
       1.7355347       0.99999034    2.7521520e-07
       1.7496844       0.99999018    2.8403271e-07
       1.7639494       0.99999001    2.9320668e-07
       1.7783307       0.99998985    3.0259139e-07
       1.7928292       0.99998968    3.1235968e-07
       1.8074460       0.99998951    3.2234660e-07
       1.8221819       0.99998934    3.3274938e-07
       1.8370380       0.99998916    3.4338000e-07
       1.8520152       0.99998898    3.5445918e-07
       1.8671145       0.99998880    3.6577497e-07
       1.8823369       0.99998861    3.7757219e-07
       1.8976834       0.99998843    3.8961585e-07
       1.9131550       0.99998823    4.0215874e-07
       1.9287528       0.99998804    4.1496617e-07
       1.9444777       0.99998784    4.2834142e-07
       1.9603309       0.99998764    4.4200396e-07
       1.9763133       0.99998744    4.5621344e-07
       1.9924260       0.99998723    4.7073748e-07
       2.0086700       0.99998702    4.8586145e-07
       2.0250465       0.99998680    5.0132735e-07
       2.0415565       0.99998659    5.1741547e-07
       2.0582011       0.99998636    5.3387773e-07
       2.0749815       0.99998614    5.5100263e-07
       2.0918986       0.99998591    5.6853095e-07
       2.1089536       0.99998568    5.8673400e-07
       2.1261477       0.99998544    6.0537806e-07
       2.1434820       0.99998520    6.2474248e-07
       2.1609576       0.99998495    6.4458746e-07
       2.1785757       0.99998471    6.6519108e-07
       2.1963374       0.99998445    6.8631623e-07
       2.2142439       0.99998420    7.0823636e-07
       2.2322964       0.99998394    7.3071749e-07
       2.2504961       0.99998367    7.5401595e-07
       2.2688442       0.99998340    7.7792850e-07
       2.2873419       0.99998313    8.0271353e-07
       2.3059904       0.99998285    8.2816256e-07
       2.3247909       0.99998256    8.5452134e-07
       2.3437447       0.99998228    8.8159418e-07
       2.3628530       0.99998198    9.0960674e-07
       2.3821171       0.99998169    9.3840372e-07
       2.4015383       0.99998138    9.6820594e-07
       2.4211178       0.99998108    9.9885289e-07
       2.4408569       0.99998077    1.0305441e-06
       2.4607570       0.99998045    1.0631366e-06
       2.4808193       0.99998013    1.0968043e-06
       2.5010452       0.99997980    1.1314628e-06
       2.5214360       0.99997947    1.1672680e-06
       2.5419930       0.99997913    1.2041325e-06
       2.5627176       0.99997878    1.2421764e-06
       2.5836112       0.99997844    1.2813711e-06
       2.6046751       0.99997808    1.3218092e-06
       2.6259108       0.99997772    1.3634948e-06
       2.6473196       0.99997735    1.4064824e-06
       2.6689029       0.99997698    1.4508059e-06
       2.6906622       0.99997660    1.4964796e-06
       2.7125989       0.99997622    1.5436053e-06
       2.7347145       0.99997583    1.5921538e-06
       2.7570104       0.99997543    1.6422503e-06
       2.7794880       0.99997503    1.6938126e-06
       2.8021489       0.99997462    1.7470576e-06
       2.8249946       0.99997420    1.8018467e-06
       2.8480265       0.99997378    1.8584461e-06
       2.8712461       0.99997335    1.9166549e-06
       2.8946551       0.99997292    1.9768131e-06
       2.9182550       0.99997247    2.0386482e-06
       2.9420472       0.99997202    2.1025636e-06
       2.9660335       0.99997156    2.1682215e-06
       2.9902152       0.99997110    2.2361417e-06
       3.0145942       0.99997063    2.3058879e-06
       3.0391719       0.99997015    2.3780659e-06
       3.0639499       0.99996966    2.4521433e-06
       3.0889300       0.99996917    2.5288114e-06
       3.1141138       0.99996867    2.6074493e-06
       3.1395029       0.99996816    2.6888832e-06
       3.1650989       0.99996764    2.7723746e-06
       3.1909037       0.99996711    2.8588803e-06
       3.2169188       0.99996658    2.9475295e-06
       3.2431460       0.99996604    3.0394126e-06
       3.2695871       0.99996549    3.1335243e-06
       3.2962437       0.99996493    3.2310862e-06
       3.3231177       0.99996436    3.3309651e-06
       3.3502107       0.99996379    3.4345306e-06
       3.3775247       0.99996320    3.5405070e-06
       3.4050613       0.99996261    3.6504760e-06
       3.4328224       0.99996200    3.7629554e-06
       3.4608099       0.99996139    3.8796926e-06
       3.4890255       0.99996077    3.9990339e-06
       3.5174712       0.99996014    4.1229663e-06
       3.5461488       0.99995950    4.2496069e-06
       3.5750602       0.99995885    4.3810892e-06
       3.6042074       0.99995820    4.5153787e-06
       3.6335921       0.99995753    4.6549258e-06
       3.6632164       0.99995685    4.7973923e-06
       3.6930823       0.99995616    4.9454427e-06
       3.7231916       0.99995546    5.0965482e-06
       3.7535465       0.99995476    5.2535733e-06
       3.7841488       0.99995404    5.4138779e-06
       3.8150006       0.99995331    5.5803441e-06
       3.8461039       0.99995257    5.7503650e-06
       3.8774608       0.99995183    5.9268223e-06
       3.9090734       0.99995107    6.1071314e-06
       3.9409437       0.99995030    6.2941875e-06
       3.9730738       0.99994952    6.4853948e-06
       4.0054659       0.99994873    6.6836286e-06
       4.0381221       0.99994792    6.8863499e-06
       4.0710445       0.99994711    7.0963871e-06
       4.1042353       0.99994629    7.3112557e-06
       4.1376967       0.99994546    7.5337009e-06
       4.1714310       0.99994461    7.7613675e-06
       4.2054403       0.99994376    7.9970318e-06
       4.2397268       0.99994289    8.2383516e-06
       4.2742929       0.99994201    8.4881020e-06
       4.3091408       0.99994112    8.7438938e-06
       4.3442728       0.99994023    9.0082995e-06
       4.3796912       0.99993932    9.2791311e-06
       4.4153984       0.99993840    9.5587074e-06
       4.4513967       0.99993746    9.8453836e-06
       4.4876885       0.99993652    1.0141721e-05
       4.5242762       0.99993557    1.0445560e-05
       4.5611622       0.99993461    1.0759157e-05
       4.5983489       0.99993363    1.1080704e-05
       4.6358388       0.99993265    1.1412194e-05
       4.6736344       0.99993165    1.1752433e-05
       4.7117381       0.99993065    1.2103467e-05
       4.7501524       0.99992963    1.2463714e-05
       4.7888799       0.99992861    1.2834885e-05
       4.8279232       0.99992758    1.3215889e-05
       4.8672848       0.99992654    1.3608223e-05
       4.9069673       0.99992549    1.4011141e-05
       4.9469733       0.99992443    1.4425923e-05
       4.9873055       0.99992336    1.4852026e-05
       5.0279666       0.99992229    1.5290479e-05
       5.0689591       0.99992121    1.5741069e-05
       5.1102858       0.99992012    1.6204531e-05
       5.1519495       0.99991903    1.6680758e-05
       5.1939528       0.99991793    1.7170171e-05
       5.2362986       0.99991683    1.7673467e-05
       5.2789897       0.99991572    1.8190731e-05
       5.3220288       0.99991462    1.8722478e-05
       5.3654187       0.99991351    1.9268419e-05
       5.4091625       0.99991241    1.9829626e-05
       5.4532629       0.99991130    2.0405478e-05
       5.4977228       0.99991020    2.0997997e-05
       5.5425452       0.99990910    2.1606027e-05
       5.5877330       0.99990802    2.2231527e-05
       5.6332893       0.99990694    2.2872915e-05
       5.6792169       0.99990588    2.3532902e-05
       5.7255190       0.99990482    2.4209417e-05
       5.7721986       0.99990380    2.4905610e-05
       5.8192588       0.99990278    2.5618892e-05
       5.8667027       0.99990182    2.6352832e-05
       5.9145333       0.99990086    2.7104387e-05
       5.9627539       0.99989997    2.7877857e-05
       6.0113677       0.99989910    2.8669611e-05
       6.0603778       0.99989833    2.9484497e-05
       6.1097875       0.99989759    3.0318271e-05
       6.1596000       0.99989698    3.1162218e-05
       6.2098186       0.99989641    3.2020769e-05
       6.2604467       0.99989603    3.2915163e-05
       6.3114875       0.99989572    3.3832605e-05
       6.3629444       0.99989574    3.4780125e-05
       6.4148209       0.99989586    3.5749316e-05
       6.4671203       0.99989662    3.6750606e-05
       6.5198461       0.99989753    3.7774429e-05
       6.5730018       0.99990001    3.8832304e-05
       6.6265908       0.99990284    3.9913493e-05
       6.6806168       0.99992516    4.1031426e-05
       6.7350832       0.99995080    4.2173615e-05
       6.7899937       0.99993959    3.8390138e-06
       6.8453519       0.99990230    3.9620325e-06
       6.9011614       0.99989409    4.0902105e-06
       6.9574259       0.99988816    4.2212190e-06
       7.0141491       0.99988384    4.3571541e-06
       7.0713348       0.99987953    4.4959830e-06
       7.1289867       0.99987585    4.6398526e-06
       7.1871086       0.99987212    4.7867064e-06
       7.2457044       0.99986873    4.9370210e-06
       7.3047779       0.99986527    5.0904546e-06
       7.3643331       0.99986199    5.2502931e-06
       7.4243738       0.99985866    5.4135237e-06
       7.4849040       0.99985544    5.5834757e-06
       7.5459277       0.99985217    5.7571269e-06
       7.6074489       0.99984895    5.9378444e-06
       7.6694717       0.99984568    6.1225902e-06
       7.7320001       0.99984245    6.3146635e-06
       7.7950384       0.99983917    6.5111195e-06
       7.8585906       0.99983589    6.7153202e-06
       7.9226609       0.99983256    6.9242926e-06
       7.9872536       0.99982923    7.1414229e-06
       8.0523729       0.99982585    7.3636748e-06
       8.1180231       0.99982246    7.5941136e-06
       8.1842086       0.99981901    7.8299996e-06
       8.2509337       0.99981554    8.0740319e-06
       8.3182027       0.99981202    8.3230602e-06
       8.3860202       0.99980846    8.5772946e-06
       8.4543906       0.99980484    8.8378845e-06
       8.5233185       0.99980119    9.1077415e-06
       8.5928082       0.99979748    9.3844847e-06
       8.6628646       0.99979373    9.6709742e-06
       8.7334921       0.99978991    9.9648103e-06
       8.8046954       0.99978605    1.0268587e-05
       8.8764792       0.99978213    1.0580200e-05
       8.9488483       0.99977815    1.0901986e-05
       9.0218074       0.99977412    1.1232314e-05
       9.0953613       0.99977002    1.1573483e-05
       9.1695149       0.99976586    1.1924021e-05
       9.2442731       0.99976164    1.2286148e-05
       9.3196408       0.99975735    1.2658268e-05
       9.3956229       0.99975300    1.3042319e-05
       9.4722245       0.99974857    1.3437145e-05
       9.5494506       0.99974408    1.3844449e-05
       9.6273064       0.99973952    1.4262723e-05
       9.7057969       0.99973489    1.4693160e-05
       9.7849273       0.99973018    1.5136143e-05
       9.8647029       0.99972540    1.5592761e-05
       9.9451288       0.99972053    1.6062987e-05
       10.026211       0.99971559    1.6547592e-05
       10.107953       0.99971057    1.7046824e-05
       10.190362       0.99970547    1.7561029e-05
       10.273443       0.99970028    1.8090571e-05
       10.357202       0.99969501    1.8635326e-05
       10.441643       0.99968966    1.9196705e-05
       10.526773       0.99968421    1.9774062e-05
       10.612597       0.99967867    2.0369410e-05
       10.699120       0.99967304    2.0981518e-05
       10.786349       0.99966731    2.1613189e-05
       10.874289       0.99966149    2.2262519e-05
       10.962946       0.99965557    2.2932997e-05
       11.052326       0.99964955    2.3621987e-05
       11.142434       0.99964341    2.4333137e-05
       11.233278       0.99963716    2.5063209e-05
       11.324861       0.99963080    2.5840139e-05
       11.417192       0.99962431    2.6648923e-05
       11.510275       0.99961774    2.7506004e-05
       11.604117       0.99961107    2.8394832e-05
       11.698725       0.99960430    2.9316906e-05
       11.794103       0.99959742    3.0263873e-05
       11.890259       0.99959043    3.1246966e-05
       11.987199       0.99958333    3.2256055e-05
       12.084930       0.99957611    3.3304071e-05
       12.183457       0.99956878    3.4379256e-05
       12.282788       0.99956133    3.5496802e-05
       12.382928       0.99955376    3.6642686e-05
       12.483885       0.99954606    3.7832872e-05
       12.585665       0.99953825    3.9052265e-05
       12.688275       0.99953030    4.0320294e-05
       12.791721       0.99952223    4.1618891e-05
       12.896010       0.99951403    4.2970404e-05
       13.001150       0.99950570    4.4353822e-05
       13.107148       0.99949724    4.5794618e-05
       13.214009       0.99948864    4.7268685e-05
       13.321741       0.99947992    4.8804571e-05
       13.430352       0.99947105    5.0375074e-05
       13.539849       0.99946207    5.2012534e-05
       13.650238       0.99945294    5.3685920e-05
       13.761527       0.99944370    5.5378342e-05
       13.873723       0.99943431    5.7104474e-05
       13.986834       0.99942476    5.8847591e-05
       14.100868       0.99941505    6.0625702e-05
       14.215831       0.99940515    6.2475436e-05
       14.331731       0.99939507    6.4363533e-05
       14.448576       0.99938484    6.6327061e-05
       14.566374       0.99937443    6.8332291e-05
       14.685132       0.99936385    7.0416168e-05
       14.804859       0.99935310    7.2545257e-05
       14.925561       0.99934218    7.4757088e-05
       15.047248       0.99933107    7.7018002e-05
       15.169927       0.99931980    7.9365458e-05
       15.293606       0.99930835    8.1766256e-05
       15.418293       0.99929677    8.4258191e-05
       15.543997       0.99928500    8.6790307e-05
       15.670725       0.99927306    8.9314657e-05
       15.798487       0.99926091    9.1893746e-05
       15.927291       0.99924850    9.4545365e-05
       16.057145       0.99923587    9.7259168e-05
       16.188057       0.99922302    0.00010006553
       16.320037       0.99920994    0.00010293858
       16.453092       0.99919665    0.00010590776
       16.587233       0.99918312    0.00010894898
       16.722467       0.99916936    0.00011209126
       16.858803       0.99915537    0.00011531088
       16.996252       0.99914113    0.00011863552
       17.134820       0.99912665    0.00012204368
       17.274519       0.99911190    0.00012556196
       17.415356       0.99909690    0.00012917002
       17.557342       0.99908161    0.00013289290
       17.700486       0.99906604    0.00013674279
       17.844796       0.99905018    0.00014077345
       17.990283       0.99903407    0.00014495115
       18.136956       0.99901774    0.00014933003
       18.284825       0.99900115    0.00015383124
       18.433899       0.99898432    0.00015847736
       18.584189       0.99896723    0.00016325549
       18.735704       0.99894988    0.00016818532
       18.888454       0.99893226    0.00017325743
       19.042450       0.99891437    0.00017848786
       19.197701       0.99889619    0.00018387181
       19.354218       0.99887774    0.00018942125
       19.512011       0.99885900    0.00019513620
       19.671091       0.99883997    0.00020102418
       19.831467       0.99882066    0.00020709068
       19.993151       0.99880105    0.00021333771
       20.156154       0.99878115    0.00021977728
       20.320485       0.99876095    0.00022640579
       20.486156       0.99874046    0.00023324171
       20.653177       0.99871967    0.00024027477
       20.821561       0.99869861    0.00024753089
       20.991317       0.99867727    0.00025499274
       21.162457       0.99865566    0.00026258894
       21.334993       0.99863375    0.00027032696
       21.508935       0.99861149    0.00027820756
       21.684295       0.99858887    0.00028623797
       21.861085       0.99856583    0.00029452235
       22.039316       0.99854237    0.00030302235
       22.219001       0.99851856    0.00031179445
       22.400150       0.99849436    0.00032079095
       22.582776       0.99846979    0.00033007920
       22.766892       0.99844483    0.00033960123
       22.952508       0.99841950    0.00034943646
       23.139637       0.99839376    0.00035951523
       23.328293       0.99836765    0.00036992868
       23.518486       0.99834113    0.00038059505
       23.710230       0.99831423    0.00039162193
       23.903537       0.99828690    0.00040291266
       24.098420       0.99825919    0.00041458812
       24.294892       0.99823105    0.00042653763
       24.492966       0.99820253    0.00043890010
       24.692655       0.99817356    0.00045154770
       24.893972       0.99814422    0.00046463826
       25.096930       0.99811442    0.00047802531
       25.301543       0.99808425    0.00049188615
       25.507824       0.99805362    0.00050605513
       25.715787       0.99802263    0.00052073191
       25.925445       0.99799117    0.00053572862
       26.136813       0.99795939    0.00055126874
       26.349904       0.99792712    0.00056714068
       26.564732       0.99789468    0.00058359592
       26.781312       0.99786174    0.00060039604
       26.999657       0.99782840    0.00061729182
       27.219783       0.99779454    0.00063451294
       27.441703       0.99776019    0.00065231752
       27.665432       0.99772530    0.00067048045
       27.890986       0.99768986    0.00068929235
       28.118378       0.99765385    0.00070848734
       28.347625       0.99761742    0.00072836265
       28.578740       0.99758043    0.00074865054
       28.811740       0.99754301    0.00076964886
       29.046639       0.99750502    0.00079109134
       29.283454       0.99746661    0.00081327498
       29.522199       0.99742762    0.00083593671
       29.762891       0.99738823    0.00085937266
       30.005545       0.99734826    0.00088332248
       30.250177       0.99730790    0.00090808390
       30.496804       0.99726696    0.00093339840
       30.745441       0.99722566    0.00095955676
       30.996106       0.99718378    0.00098630968
       31.248814       0.99714156     0.0010139454
       31.503583       0.99709877     0.0010422194
       31.760428       0.99705568     0.0010714146
       32.019368       0.99701205     0.0011012960
       32.280419       0.99696818     0.0011321430
       32.543598       0.99692382     0.0011637263
       32.808923       0.99687932     0.0011963144
       33.076411       0.99683445     0.0012296938
       33.346079       0.99678969     0.0012641213
       33.617947       0.99674488     0.0012990459
       33.892031       0.99670087     0.0013342272
       34.168349       0.99665597     0.0013695247
       34.446920       0.99660991     0.0014043016
       34.727763       0.99656306     0.0014398971
       35.010895       0.99651531     0.0014764575
       35.296335       0.99646681     0.0015138879
       35.584103       0.99641749     0.0015523210
       35.874217       0.99636752     0.0015916797
       36.166696       0.99631693     0.0016320816
       36.461559       0.99626572     0.0016734692
       36.758827       0.99621393     0.0017159423
       37.058518       0.99616153     0.0017594639
       37.360653       0.99610854     0.0018041141
       37.665250       0.99605499     0.0018498768
       37.972331       0.99600091     0.0018968099
       38.281916       0.99594630     0.0019449296
       38.594025       0.99589116     0.0019942712
       38.908678       0.99583553     0.0020448734
       39.225897       0.99577939     0.0020967431
       39.545702       0.99572284     0.0021499517
       39.868114       0.99566581     0.0022044761
       40.193155       0.99560847     0.0022604239
       40.520846       0.99555070     0.0023177397
       40.851209       0.99549292     0.0023765731
       41.184265       0.99543488     0.0024368324
       41.520036       0.99537670     0.0024979653
       41.858545       0.99531814     0.0025600896
       42.199813       0.99525916     0.0026232923
       42.543864       0.99519960     0.0026876386
       42.890720       0.99513931     0.0027536718
       43.240404       0.99507824     0.0028212044
       43.592939       0.99501688     0.0028905268
       43.948348       0.99495492     0.0029614084
       44.306655       0.99489277     0.0030341846
       44.667883       0.99483005     0.0031085760
       45.032056       0.99476729     0.0031849784
       45.399198       0.99470397     0.0032630601
       45.769333       0.99464075     0.0033432723
       46.142486       0.99457698     0.0034252256
       46.518681       0.99451354     0.0035094334
       46.897943       0.99444960     0.0035954457
       47.280298       0.99438617     0.0036838500
       47.665769       0.99432225     0.0037741268
       48.054384       0.99425914     0.0038669368
       48.446167       0.99419557     0.0039616891
       48.841143       0.99413425     0.0040591180
       49.239341       0.99407273     0.0041585606
       49.640784       0.99401300     0.0042594645
       50.045501       0.99395296     0.0043622266
       50.453517       0.99389381     0.0044640218
       50.864859       0.99383419     0.0045674051
       51.279556       0.99377400     0.0046735676
       51.697633       0.99371317     0.0047817929
       52.119119       0.99365320     0.0048929538
       52.544041       0.99359269     0.0050062519
       52.972427       0.99353370     0.0051226405
       53.404306       0.99347419     0.0052412417
       53.839707       0.99341691     0.0053631054
       54.278657       0.99335923     0.0054872845
       54.721185       0.99330682     0.0056148623
       55.167322       0.99325432     0.0057448852
       55.617096       0.99320832     0.0058784306
       56.070536       0.99316223     0.0060133353
       56.527674       0.99311795     0.0061349101
       56.988539       0.99307304     0.0062586889
       57.453161       0.99302350     0.0063849271
       57.921571       0.99297347     0.0065134901
       58.393800       0.99292157     0.0066448496
       58.869879       0.99286922     0.0067786455
       59.349839       0.99281579     0.0069153582
       59.833713       0.99276185     0.0070546182
       60.321531       0.99270675     0.0071968769
       60.813327       0.99265082     0.0073425840
       61.309132       0.99259253     0.0074948320
       61.808979       0.99253489     0.0076508726
       62.312902       0.99248095     0.0078136330
       62.820933       0.99242756     0.0079795945
       63.333106       0.99237671     0.0081493379
       63.849455       0.99232656     0.0083224385
       64.370013       0.99227889     0.0084994437
       64.894816       0.99223185     0.0086799878
       65.423897       0.99218664     0.0088646119
       65.957292       0.99214159     0.0090511158
       66.495035       0.99209699     0.0092376401
       67.037163       0.99205348     0.0094277444
       67.583710       0.99201225     0.0096217129
       68.134713       0.99195250     0.0098079973
       68.690209       0.99185845     0.0099771561
       69.250234       0.99173894      0.010197640
       69.814824       0.99157768      0.010502077
       70.384018       0.99145408      0.010878628
       70.957852       0.99138903      0.011361526
       71.536364       0.99142917      0.011865401
       72.119593       0.99161388      0.012394415
       72.707578       0.99187380      0.012818901
       73.300355       0.99222646      0.013110028
       73.897966       0.99256718      0.013322915
       74.500449       0.99289319      0.013446153
       75.107844       0.99317065      0.013555499
       75.720191       0.99339783      0.013650459
       76.337530       0.99360512      0.013672782
       76.959903       0.99379413      0.013626609
       77.587350       0.99388757      0.013606454
       78.219912       0.99389985      0.013608143
       78.857631       0.99381491      0.013643493
       79.500550       0.99365445      0.013704640
       80.148710       0.99366549      0.014047233
       80.802155       0.99379700      0.014589025
       81.460927       0.99396086      0.014801422
       82.125070       0.99414709      0.014805786
       82.794628       0.99425590      0.014930600
       83.469644       0.99432196      0.015124730
       84.150164       0.99445409      0.015262040
       84.836232       0.99462215      0.015370817
       85.527894       0.99470726      0.015426209
       86.225195       0.99475573      0.015456632
       86.928180       0.99480070      0.015609714
       87.636897       0.99484533      0.015814072
       88.351392       0.99482087      0.015987741
       89.071713       0.99477132      0.016151612
       89.797906       0.99470368      0.016443114
       90.530020       0.99462959      0.016778880
       91.268102       0.99472499      0.017403327
       92.012202       0.99486977      0.018117596
       92.762369       0.99544569      0.018616473
       93.518651       0.99613614      0.019071296
       94.281100       0.99677470      0.019183552
       95.049764       0.99741577      0.019226709
       95.824696       0.99782010      0.019207344
       96.605945       0.99819311      0.019174900
       97.393564       0.99851354      0.019214335
       98.187605       0.99883384      0.019262168
       98.988119       0.99920367      0.019364694
       99.795159       0.99958693      0.019472859
       100.60878        1.0000918      0.019571360
       101.42903        1.0006174      0.019668959
       102.25597        1.0011608      0.019604807
       103.08966        1.0017178      0.019529001
       103.93014        1.0024436      0.019404408
       104.77747        1.0031897      0.019271507
       105.63171        1.0038641      0.018714744
       106.49292        1.0045406      0.018137747
       107.36114        1.0047550      0.017405401
       108.23645        1.0049635      0.016674930
       109.11889        1.0049820      0.016334545
       110.00852        1.0050115      0.015981436
       110.90541        1.0051790      0.015591012
       111.80961        1.0053430      0.015188681
       112.72119        1.0054468      0.014772799
       113.64019        1.0055494      0.014356099
       114.56669        1.0056326      0.014008499
       115.50074        1.0057068      0.013617246
       116.44241        1.0057230      0.013022090
       117.39175        1.0056781      0.012445318
       118.34883        1.0053363      0.012022029
       119.31372        1.0050135      0.011804042
       120.28647        1.0047988      0.012477771
       121.26716        1.0050444      0.013208736
       122.25584        1.0069673      0.014107569
       123.25258        1.0112071      0.015041976
       124.25744        1.0212070     0.0071982737
       125.27050        1.0116822     0.0017721184
       126.29182        1.0086336     0.0017778456
       127.32147        1.0060404     0.0017835422
       128.35951        1.0046845     0.0017893024
       129.40601        1.0034442     0.0017950377
       130.46104        1.0025000     0.0018008336
       131.52468        1.0016098     0.0018066091
       132.59699        1.0008497     0.0018124377
       133.67804        1.0001196     0.0018182517
       134.76790       0.99945732     0.0018241149
       135.86665       0.99881477     0.0018322913
       136.97436       0.99821385     0.0018442702
       138.09110       0.99762823     0.0018575416
       139.21694       0.99707162     0.0018726858
       140.35196       0.99652506     0.0018879526
       141.49624       0.99599605     0.0019033432
       142.64984       0.99547386     0.0019188611
       143.81285       0.99496231     0.0019345071
       144.98535       0.99444794     0.0019502792
       146.16740       0.99393143     0.0019661776
       147.35909       0.99341703     0.0019954036
       148.56049       0.99290455     0.0020379983
       149.77169       0.99240288     0.0020823668
       150.99277       0.99190983     0.0021284362
       152.22379       0.99141719     0.0021755442
       153.46486       0.99092281     0.0022236739
       154.71604       0.99041861     0.0022728941
       155.97743       0.98990410     0.0023231710
       157.24910       0.98939715     0.0024075818
       158.53114       0.98889164     0.0025157423
       159.82362       0.98839839     0.0026300823
       161.12665       0.98790844     0.0027491934
       162.44030       0.98744102     0.0028745888
       163.76466       0.98698154     0.0030046960
       165.09982       0.98654574     0.0031262784
       166.44586       0.98611710     0.0032449561
       167.80288       0.98568473     0.0033481127
       169.17096       0.98524576     0.0034455662
       170.55019       0.98480569     0.0035462525
       171.94067       0.98435976     0.0036494458
       173.34249       0.98392634     0.0037561192
       174.75573       0.98349166     0.0038654065
       176.18050       0.98307514     0.0039522138
       177.61688       0.98265857     0.0040332824
       179.06498       0.98222741     0.0040905618
       180.52487       0.98178627     0.0041421787
       181.99668       0.98133274     0.0041944038
       183.48048       0.98086964     0.0042473322
       184.97638       0.98040066     0.0043008790
       186.48447       0.97992328     0.0043551500
       188.00486       0.97944736     0.0044100591
       189.53765       0.97896407     0.0044657148
       191.08293       0.97847965     0.0044922916
       192.64081       0.97798713     0.0045152804
       194.21139       0.97746932     0.0045361868
       195.79478       0.97694025     0.0045568142
       197.39107       0.97639597     0.0045777001
       199.00038       0.97584108     0.0045985119
       200.62281       0.97527158     0.0046195885
       202.25847       0.97469117     0.0046405811
       203.90747       0.97409383     0.0046618633
       205.56990       0.97348502     0.0046831051
       207.24590       0.97285356     0.0047084600
       208.93555       0.97221017     0.0047349924
       210.63898       0.97155497     0.0047932204
       212.35630       0.97088848     0.0048522345
       214.08762       0.97022619     0.0049119059
       215.83306       0.96955230     0.0049723797
       217.59272       0.96887412     0.0050335331
       219.36674       0.96818389     0.0050955046
       221.15521       0.96748523     0.0051581729
       222.95827       0.96677416     0.0052216761
       224.77603       0.96605321     0.0052859046
       226.60860       0.96531936     0.0053509824
       228.45612       0.96457393     0.0054167941
       230.31870       0.96381461     0.0054834744
       232.19647       0.96304032     0.0055509255
       234.08954       0.96225202     0.0056218324
       235.99805       0.96144990     0.0057037209
       237.92212       0.96063490     0.0057880320
       239.86188       0.95981103     0.0058776331
       241.81745       0.95897491     0.0059686396
       243.78896       0.95813168     0.0060610352
       245.77655       0.95727459     0.0061548822
       247.78034       0.95640559     0.0062501740
       249.80047       0.95552239     0.0063469543
       251.83707       0.95462669     0.0064452105
       253.89028       0.95371633     0.0065450040
       255.96022       0.95279262     0.0066463319
       258.04704       0.95185380     0.0067492409
       260.15087       0.95090083     0.0068537290
       262.27186       0.94993305     0.0069598454
       264.41013       0.94895172     0.0070675924
       266.56584       0.94795465     0.0071761260
       268.73913       0.94694249     0.0072849489
       270.93013       0.94591295     0.0073933578
       273.13900       0.94486594     0.0075004887
       275.36588       0.94380030     0.0076091824
       277.61091       0.94271566     0.0077194417
       279.87424       0.94161234     0.0078313055
       282.15603       0.94049010     0.0079447868
       284.45642       0.93934852     0.0080599146
       286.77557       0.93818729     0.0081767101
       289.11362       0.93700624     0.0082951962
       291.47073       0.93580499     0.0084154040
       293.84707       0.93458322     0.0085373475
       296.24277       0.93334051     0.0086610675
       298.65801       0.93207677     0.0087865697
       301.09294       0.93079150     0.0089139059
       303.54772       0.92948422     0.0090430641
       306.02252       0.92815442     0.0091741099
       308.51749       0.92680979     0.0093070342
       311.03281       0.92544712     0.0094419118
       313.56863       0.92406164     0.0095660586
       316.12512       0.92265262     0.0096845846
       318.70246       0.92120831     0.0097962615
       321.30081       0.91973325     0.0099049740
       323.92034       0.91822418      0.010014851
       326.56123       0.91668494      0.010126011
       329.22366       0.91511231      0.010238326
       331.90778       0.91350951      0.010351948
       334.61380       0.91187210      0.010466771
       337.34187       0.91020363      0.010582948
       340.09219       0.90849893      0.010700333
       342.86492       0.90676209      0.010819102
       345.66027       0.90498716      0.010939095
       348.47840       0.90317895      0.011060513
       351.31951       0.90133012      0.011183182
       354.18379       0.89944668      0.011307318
       357.07142       0.89752034      0.011432713
       359.98258       0.89555811      0.011559615
       362.91749       0.89354967      0.011687811
       365.87632       0.89150385      0.011817556
       368.85927       0.88940852      0.011948602
       371.86655       0.88727434      0.012081237
       374.89834       0.88508635      0.012215220
       377.95485       0.88285794      0.012350842
       381.03628       0.88056967      0.012487792
       384.14283       0.87823922      0.012626423
       387.27471       0.87583563      0.012766438
       390.43212       0.87338757      0.012908181
       393.61528       0.87087315      0.013084839
       396.82438       0.86831332      0.013265051
       400.05966       0.86569420      0.013461943
       403.32130       0.86302829      0.013661940
       406.60954       0.86031108      0.013864780
       409.92459       0.85754450      0.014070758
       413.26667       0.85470657      0.014279665
       419.986      0.853000     0.0142000
       430.038      0.843000     0.0147000
       439.957      0.834000     0.0152000
       450.018      0.824000     0.0158000
       460.037      0.814000     0.0168000
       469.977      0.803000     0.0178000
       479.985      0.792000     0.0192000
       490.040      0.778000     0.0205000
       499.919      0.766000     0.0223000
       509.996      0.752000     0.0243000
       520.050      0.737000     0.0264000
       530.056      0.722000     0.0292000
       539.983      0.706000     0.0325000
       550.044      0.691000     0.0365000
       559.982      0.675000     0.0405000
       570.023      0.659000     0.0455000
       579.888      0.644000     0.0510000
       590.100      0.627000     0.0580000
       600.097      0.610000     0.0650000
       609.838      0.590000     0.0740000
       619.8999     0.567000     0.0835000
       619.9001     0.569000      0.122000
       629.980      0.549000     0.0930000
       635.795      0.542000      0.140000
       640.062      0.530000      0.100000
       650.131      0.513000      0.113000
       652.526      0.514000      0.163000
       670.162      0.485000      0.189000
       688.778      0.455000      0.219000
       698.479      0.440000      0.237000
       708.457      0.426000      0.255000
       718.725      0.411000      0.275000
       729.294      0.397000      0.296000
       737.976      0.386000      0.314000
       742.395      0.379000      0.323000
       746.867      0.374000      0.333000
       751.394      0.369000      0.342000
       762.954      0.357000      0.367000
       774.875      0.345000      0.394000
       787.175      0.333000      0.421000
       799.871      0.323000      0.450000
       812.984      0.313000      0.479000
       826.533      0.304000      0.510000
       840.542      0.296000      0.541000
       855.034      0.288000      0.573000
       870.035      0.281000      0.607000
       885.571      0.275000      0.641000
       901.673      0.269000      0.677000
       918.370      0.265000      0.714000
       935.698      0.261000      0.752000
       953.692      0.258000      0.792000
       972.392      0.256000      0.833000
       991.840      0.255000      0.875000
       1012.08      0.256000      0.918000
       1033.17      0.257000      0.963000
       1055.15      0.259000       1.01000
       1078.09      0.263000       1.06000
       1102.04      0.267000       1.11000
       1127.09      0.272000       1.16000
       1153.30      0.278000       1.21000
       1180.76      0.286000       1.26000
       1209.56      0.295000       1.32000
       1239.80      0.306000       1.38000
       1271.59      0.318000       1.45000
       1305.05      0.332000       1.51000
       1340.32      0.348000       1.58000
       1377.56      0.367000       1.66000
       1416.91      0.389000       1.73000
       1458.59      0.414000       1.82000
       1502.79      0.444000       1.90000
       1549.75      0.478000       2.00000
       1599.74      0.517000       2.10000
       1653.07      0.563000       2.21000
       1710.07      0.618000       2.32000
       1771.14      0.682000       2.45000
       1836.74      0.756000       2.58000
       1907.38      0.847000       2.73000
       1983.68      0.968000       2.89000
       2066.33       1.01000       2.90900
       2073.24       1.03600       2.92800
       2080.20       1.04600       2.94400
       2087.21       1.06600       2.93700
       2094.26       1.07000       2.96300
       2101.36       1.08300       2.98200
       2108.50       1.08800       2.98700
       2115.70       1.10200       3.00500
       2122.95       1.10900       3.01500
       2130.24       1.11900       3.02500
       2137.59       1.13300       3.04500
       2144.98       1.13900       3.06100
       2152.43       1.15500       3.07300
       2159.93       1.16400       3.08600
       2167.48       1.17500       3.10200
       2175.09       1.18000       3.11200
       2182.75       1.19500       3.13500
       2190.46       1.21100       3.15000
       2198.23       1.22200       3.16900
       2206.05       1.23500       3.19000
       2213.93       1.24700       3.20600
       2221.86       1.26500       3.22800
       2229.86       1.28000       3.24500
       2237.91       1.29900       3.26700
       2246.01       1.31900       3.28500
       2254.18       1.34000       3.30200
       2262.41       1.36200       3.31900
       2270.70       1.38900       3.33400
       2279.04       1.41600       3.35000
       2287.45       1.44500       3.35900
       2295.93       1.47100       3.36600
       2304.46       1.50200       3.36800
       2313.06       1.52600       3.36800
       2321.72       1.54800       3.36400
       2330.45       1.56600       3.35800
       2339.25       1.57900       3.35300
       2348.11       1.58500       3.34600
       2357.03       1.59000       3.34400
       2366.03       1.59100       3.34400
       2375.10       1.59200       3.34700
       2384.23       1.58900       3.35400
       2393.44       1.58600       3.36300
       2402.71       1.58200       3.37600
       2412.06       1.57900       3.38900
       2421.48       1.57300       3.40800
       2430.98       1.57100       3.42900
       2440.55       1.57000       3.45100
       2450.20       1.56900       3.47700
       2459.92       1.56800       3.50400
       2469.72       1.56900       3.53300
       2479.60       1.57000       3.56500
       2489.56       1.57500       3.59800
       2499.60       1.58000       3.63200
       2509.72       1.58400       3.67000
       2519.92       1.59100       3.70900
       2530.20       1.59700       3.74900
       2540.57       1.60800       3.78900
       2551.03       1.61800       3.83500
       2561.57       1.62900       3.88000
       2572.20       1.64300       3.92800
       2582.92       1.65800       3.97900
       2593.72       1.67300       4.03100
       2604.62       1.69200       4.08800
       2615.61       1.71300       4.14900
       2626.70       1.73700       4.21100
       2637.87       1.76400       4.27800
       2649.15       1.79400       4.35000
       2660.52       1.83100       4.42600
       2671.98       1.87400       4.50600
       2683.55       1.92700       4.59000
       2695.22       1.98800       4.67800
       2706.99       2.05900       4.76400
       2718.86       2.14000       4.84900
       2730.84       2.23400       4.93300
       2742.92       2.33900       5.01100
       2755.11       2.45100       5.08200
       2767.41       2.57200       5.14800
       2779.82       2.70000       5.20600
       2792.34       2.83300       5.25700
       2804.98       2.97400       5.30400
       2817.73       3.12000       5.34400
       2830.59       3.27700       5.38100
       2843.58       3.44400       5.41400
       2856.68       3.63400       5.43500
       2869.91       3.84900       5.43900
       2883.26       4.08600       5.39500
       2896.73       4.31800       5.30100
       2910.33       4.52500       5.15800
       2924.06       4.68600       4.98900
       2937.91       4.80500       4.81200
       2951.90       4.88800       4.63900
       2966.03       4.94100       4.48000
       2980.29       4.97700       4.33500
       2994.69       4.99900       4.20400
       3009.22       5.01200       4.08600
       3023.90       5.02000       3.97900
       3038.73       5.02100       3.88500
       3053.69       5.02000       3.79800
       3068.81       5.01800       3.72000
       3084.08       5.01500       3.65000
       3099.50       5.01000       3.58700
       3115.08       5.00900       3.52900
       3130.81       5.01000       3.47700
       3146.70       5.00900       3.42900
       3162.76       5.01200       3.38600
       3178.97       5.01600       3.34600
       3195.36       5.02100       3.31000
       3211.92       5.02900       3.27500
       3228.65       5.04000       3.24200
       3245.55       5.05200       3.21100
       3262.63       5.06500       3.18200
       3279.89       5.07900       3.15400
       3297.34       5.09500       3.12800
       3314.97       5.11500       3.10300
       3332.80       5.13400       3.07900
       3350.81       5.15600       3.05800
       3369.02       5.17900       3.03900
       3387.43       5.20400       3.02100
       3406.04       5.23100       3.00700
       3424.86       5.26100       2.99500
       3443.89       5.29600       2.98700
       3463.13       5.33600       2.98300
       3482.58       5.38300       2.98400
       3502.26       5.44200       2.98900
       3522.16       5.51500       2.99900
       3542.29       5.61000       3.01400
       3562.64       5.73300       3.02600
       3583.24       5.89400       3.02300
       3604.07       6.08900       2.98200
       3625.15       6.30800       2.88100
       3646.47       6.52200       2.70500
       3668.05       6.69500       2.45600
       3689.88       6.79600       2.16900
       3711.98       6.82900       1.87000
       3734.34       6.79900       1.57700
       3756.97       6.70900       1.32100
       3779.88       6.58500       1.11000
       3803.07       6.45200      0.945000
       3826.54       6.31600      0.815000
       3850.31       6.18500      0.714000
       3874.38       6.06200      0.630000
       3898.74       5.94800      0.561000
       3923.42       5.84200      0.505000
       3948.41       5.74400      0.456000
       3973.72       5.65400      0.416000
       3999.35       5.57000      0.387000
       4025.32       5.49300      0.355000
       4051.63       5.42000      0.329000
       4078.29       5.34900      0.313000
       4105.30       5.28400      0.291000
       4132.67       5.22200      0.269000
       4160.40       5.16400      0.255000
       4188.51       5.10900      0.244000
       4217.01       5.05800      0.228000
       4245.89       5.00900      0.211000
       4275.17       4.96100      0.203000
       4304.86       4.91600      0.194000
       4334.97       4.87200      0.185000
       4365.49       4.83100      0.185000
       4396.45       4.79100      0.170000
       4427.86       4.75300      0.163000
       4459.71       4.71800      0.149000
       4492.03       4.68200      0.149000
       4524.82       4.64800      0.133000
       4558.09       4.61500      0.131000
       4591.85       4.58300      0.130000
       4626.12       4.55300      0.131000
       4660.90       4.52200      0.134000
       4696.21       4.49500      0.120000
       4732.06       4.46600      0.120000
       4768.46       4.44200     0.0900000
       4805.43       4.41600     0.0940000
       4842.97       4.39100     0.0830000
       4881.10       4.36700     0.0790000
       4919.84       4.34300     0.0770000
       4959.20       4.32000     0.0730000
       4999.19       4.29800     0.0730000
       5039.84       4.27700     0.0660000
       5081.15       4.25500     0.0720000
       5123.14       4.23500     0.0600000
       5165.83       4.21500     0.0600000
       5209.24       4.19600     0.0560000
       5253.39       4.17700     0.0530000
       5298.29       4.15900     0.0430000
       5343.97       4.14000     0.0450000
       5390.44       4.12300     0.0480000
       5437.72       4.10600     0.0440000
       5485.84       4.08900     0.0440000
       5534.82       4.07300     0.0320000
       5584.68       4.05700     0.0380000
       5635.45       4.04200     0.0320000
       5687.16       4.02600     0.0340000
       5739.81       4.01200     0.0300000
       5793.46       3.99700     0.0270000
       5848.11       3.98300     0.0300000
       5903.81       3.96900     0.0300000
       5960.58       3.95600     0.0270000
       6018.45       3.94300     0.0250000
       6077.45       3.93100     0.0250000
       6137.62       3.91800     0.0240000
       6199.00       3.90600     0.0220000
       6261.62       3.89300     0.0220000
       6325.51       3.88200     0.0190000
       6390.72       3.87000     0.0180000
       6457.29       3.85800     0.0170000
       6525.26       3.84700     0.0160000
       6594.68       3.83700     0.0160000
       6665.59       3.82600     0.0150000
       6738.04       3.81500     0.0140000
       6812.09       3.80500     0.0130000
       6887.78       3.79600     0.0130000
       6965.17       3.78700     0.0130000
       7044.32       3.77800     0.0120000
       7125.29       3.76800     0.0110000
       7208.14       3.76100     0.0110000
       7292.94       3.75200     0.0100000
       7379.76       3.74500     0.0100000
       7468.67       3.73600    0.00900000
       7559.76       3.72800    0.00900000
       7653.09       3.72100    0.00800000
       7748.75       3.71400    0.00800000
       7846.84       3.70500    0.00700000
       7947.44       3.69700    0.00700000
       8050.65       3.68800    0.00600000
       8156.58       3.68100    0.00600000
       8265.33       3.67300    0.00500000
       11199.6       3.53610       0.00000
       11437.3       3.52950       0.00000
       12001.9       3.51930       0.00000
       13719.2       3.50070       0.00000
       13999.5       3.48760       0.00000
       15319.4       3.47840       0.00000
       15999.5       3.47100       0.00000
       16960.3       3.46440       0.00000
       17999.4       3.45780       0.00000
       20000.0       3.44900       0.00000
       24371.9       3.44340       0.00000
       25001.0       3.44240       0.00000
       27141.0       3.43930   2.50000e-09
       29997.6       3.43610       0.00000
       33034.9       3.43350       0.00000
       34191.9       3.43270       0.00000
       35002.8       3.43210       0.00000
       39993.5       3.42940       0.00000
       49991.9       3.42610   1.99000e-07
       60009.7       3.42420       0.00000
       70005.6       3.42310       0.00000
       79987.1       3.42240   1.53000e-05
       89971.0       3.42190       0.00000
       99983.9       3.42150   6.76000e-05
       134996.       3.42090       0.00000
       145006.       3.42080       0.00000
       150006.       3.42070       0.00000
       200000.       3.42040   0.000286000
       250010.       3.42010   9.15000e-05
       302981.       3.42000       0.00000
       499919.       3.41970       0.00000
       624899.       3.41950   0.000170000
       714170.       3.41920   0.000190000
       833199.       3.41900   0.000230000
       908944.       3.41880       0.00000
       999839.       3.41850   0.000290000
   1.24992e+06       3.41800   0.000430000
   1.66864e+06       3.41700   0.000720000
   2.00000e+06       3.41650    0.00100000
   2.50010e+06       3.41600    0.00140000
   3.33280e+06       3.41550       0.00000];

lld=AAA(:,1)*1.e-4;nn=AAA(:,2)+i*AAA(:,3);
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 

%------------------------------------------------------

case {'MoSiO2'}
%  Sbastien de Rossi   .Calcul  partir des 'atomic scattering factors' de  
%  MoSiO2'(6.31g/cm3)
%  CXRO <http://www-cxro.lbl.gov> et LLNL <http://www-phys.llnl.gov/V_Div/scattering/asf.html>.

bornes=[0.00012398,.41326];ref='Sbastien de Rossi  CXRO <http://www-cxro.lbl.gov> et LLNL <http://www-phys.llnl.gov/V_Div/scattering/asf.html>.';
AAA=[
      0.12398000       0.99999988    3.5801872e-10
      0.12499080       0.99999988    3.6964513e-10
      0.12600983       0.99999988    3.8166380e-10
      0.12703718       0.99999987    3.9407742e-10
      0.12807290       0.99999987    4.0685864e-10
      0.12911707       0.99999987    4.2008297e-10
      0.13016975       0.99999987    4.3369355e-10
      0.13123101       0.99999986    4.4775584e-10
      0.13230092       0.99999986    4.6227071e-10
      0.13337956       0.99999986    4.7724188e-10
      0.13446699       0.99999986    4.9279934e-10
      0.13556329       0.99999986    5.0869896e-10
      0.13666852       0.99999985    5.2528608e-10
      0.13778277       0.99999985    5.4221971e-10
      0.13890610       0.99999985    5.5979501e-10
      0.14003858       0.99999985    5.7791830e-10
      0.14118030       0.99999984    5.9661402e-10
      0.14233133       0.99999984    6.1598253e-10
      0.14349175       0.99999984    6.3581581e-10
      0.14466162       0.99999984    6.5648973e-10
      0.14584103       0.99999983    6.7758679e-10
      0.14703006       0.99999983    6.9956319e-10
      0.14822878       0.99999983    7.2219956e-10
      0.14943728       0.99999982    7.4551144e-10
      0.15065563       0.99999982    7.6979106e-10
      0.15188391       0.99999982    7.9456706e-10
      0.15312220       0.99999982    8.2039665e-10
      0.15437059       0.99999981    8.4688535e-10
      0.15562916       0.99999981    8.7415217e-10
      0.15689799       0.99999981    9.0235335e-10
      0.15817716       0.99999980    9.3153769e-10
      0.15946677       0.99999980    9.6170843e-10
      0.16076688       0.99999980    9.9249340e-10
      0.16207760       0.99999979    1.0246588e-09
      0.16339900       0.99999979    1.0575901e-09
      0.16473118       0.99999979    1.0917228e-09
      0.16607422       0.99999978    1.1269415e-09
      0.16742821       0.99999978    1.1631900e-09
      0.16879323       0.99999978    1.2006045e-09
      0.17016939       0.99999977    1.2392574e-09
      0.17155676       0.99999977    1.2792257e-09
      0.17295545       0.99999977    1.3204626e-09
      0.17436554       0.99999976    1.3628805e-09
      0.17578712       0.99999976    1.4068801e-09
      0.17722030       0.99999975    1.4519924e-09
      0.17866516       0.99999975    1.4987362e-09
      0.18012180       0.99999975    1.5467333e-09
      0.18159031       0.99999974    1.5964021e-09
      0.18307080       0.99999974    1.6475124e-09
      0.18456336       0.99999973    1.7003515e-09
      0.18606809       0.99999973    1.7549747e-09
      0.18758508       0.99999972    1.8112668e-09
      0.18911444       0.99999972    1.8694345e-09
      0.19065628       0.99999971    1.9292001e-09
      0.19221068       0.99999971    1.9912719e-09
      0.19377775       0.99999971    2.0547221e-09
      0.19535760       0.99999970    2.1208394e-09
      0.19695034       0.99999970    2.1884331e-09
      0.19855605       0.99999969    2.2585790e-09
      0.20017486       0.99999969    2.3310355e-09
      0.20180687       0.99999968    2.4054761e-09
      0.20345218       0.99999967    2.4825633e-09
      0.20511091       0.99999967    2.5622146e-09
      0.20678315       0.99999966    2.6439245e-09
      0.20846904       0.99999966    2.7282599e-09
      0.21016867       0.99999965    2.8155874e-09
      0.21188215       0.99999965    2.9046996e-09
      0.21360961       0.99999964    2.9972263e-09
      0.21535114       0.99999964    3.0926877e-09
      0.21710688       0.99999963    3.1914830e-09
      0.21887693       0.99999962    3.2930652e-09
      0.22066141       0.99999962    3.3979863e-09
      0.22246045       0.99999961    3.5064689e-09
      0.22427414       0.99999960    3.6181597e-09
      0.22610263       0.99999960    3.7332355e-09
      0.22794602       0.99999959    3.8521446e-09
      0.22980444       0.99999958    3.9738546e-09
      0.23167802       0.99999958    4.1008159e-09
      0.23356687       0.99999957    4.2304000e-09
      0.23547111       0.99999956    4.3653492e-09
      0.23739089       0.99999956    4.5037237e-09
      0.23932631       0.99999955    4.6462132e-09
      0.24127752       0.99999954    4.7936480e-09
      0.24324463       0.99999953    4.9456897e-09
      0.24522778       0.99999953    5.1027644e-09
      0.24722710       0.99999952    5.2649116e-09
      0.24924272       0.99999951    5.4318152e-09
      0.25127477       0.99999950    5.6037179e-09
      0.25332339       0.99999950    5.7809302e-09
      0.25538871       0.99999949    5.9638930e-09
      0.25747087       0.99999948    6.1529177e-09
      0.25957000       0.99999947    6.3471510e-09
      0.26168625       0.99999946    6.5469309e-09
      0.26381975       0.99999945    6.7527776e-09
      0.26597065       0.99999944    6.9657777e-09
      0.26813908       0.99999943    7.1853134e-09
      0.27032520       0.99999943    7.4117562e-09
      0.27252913       0.99999942    7.6453682e-09
      0.27475103       0.99999941    7.8860302e-09
      0.27699105       0.99999940    8.1341452e-09
      0.27924933       0.99999939    8.3888335e-09
      0.28152603       0.99999938    8.6526137e-09
      0.28382128       0.99999937    8.9238343e-09
      0.28613525       0.99999936    9.2047991e-09
      0.28846808       0.99999935    9.4918671e-09
      0.29081993       0.99999933    9.7912452e-09
      0.29319096       0.99999932    1.0096997e-08
      0.29558132       0.99999931    1.0416045e-08
      0.29799117       0.99999930    1.0741687e-08
      0.30042066       0.99999929    1.1079427e-08
      0.30286996       0.99999928    1.1426609e-08
      0.30533923       0.99999927    1.1786064e-08
      0.30782863       0.99999925    1.2156468e-08
      0.31033833       0.99999924    1.2535582e-08
      0.31286848       0.99999923    1.2928635e-08
      0.31541927       0.99999922    1.3333764e-08
      0.31799085       0.99999920    1.3750065e-08
      0.32058340       0.99999919    1.4180536e-08
      0.32319709       0.99999918    1.4623087e-08
      0.32583208       0.99999916    1.5078832e-08
      0.32848856       0.99999915    1.5548455e-08
      0.33116670       0.99999914    1.6030223e-08
      0.33386667       0.99999912    1.6528509e-08
      0.33658865       0.99999911    1.7041848e-08
      0.33933282       0.99999909    1.7569672e-08
      0.34209937       0.99999908    1.8114059e-08
      0.34488847       0.99999906    1.8672559e-08
      0.34770032       0.99999905    1.9249312e-08
      0.35053508       0.99999903    1.9844339e-08
      0.35339296       0.99999902    2.0458024e-08
      0.35627414       0.99999900    2.1089442e-08
      0.35917881       0.99999899    2.1739423e-08
      0.36210716       0.99999897    2.2411863e-08
      0.36505938       0.99999895    2.3103057e-08
      0.36803568       0.99999894    2.3816710e-08
      0.37103624       0.99999892    2.4550503e-08
      0.37406126       0.99999890    2.5310067e-08
      0.37711095       0.99999888    2.6092546e-08
      0.38018549       0.99999886    2.6898966e-08
      0.38328511       0.99999885    2.7732608e-08
      0.38641000       0.99999883    2.8593453e-08
      0.38956036       0.99999881    2.9478156e-08
      0.39273641       0.99999879    3.0391025e-08
      0.39593835       0.99999877    3.1328476e-08
      0.39916640       0.99999875    3.2294762e-08
      0.40242076       0.99999873    3.3286809e-08
      0.40570166       0.99999871    3.4315159e-08
      0.40900931       0.99999869    3.5369194e-08
      0.41234392       0.99999867    3.6690669e-08
      0.41570572       0.99999864    3.8024889e-08
      0.41909493       0.99999862    3.9303149e-08
      0.42251177       0.99999860    4.0608347e-08
      0.42595647       0.99999858    4.1940537e-08
      0.42942925       0.99999855    4.3300615e-08
      0.43293034       0.99999853    4.4689362e-08
      0.43645998       0.99999851    4.6106214e-08
      0.44001840       0.99999848    4.7551882e-08
      0.44360583       0.99999846    4.9028259e-08
      0.44722250       0.99999843    5.0534427e-08
      0.45086866       0.99999841    5.2071554e-08
      0.45454455       0.99999838    5.3640303e-08
      0.45825041       0.99999836    5.5241125e-08
      0.46198648       0.99999833    5.6874626e-08
      0.46575301       0.99999830    5.8541350e-08
      0.46955025       0.99999828    6.0241833e-08
      0.47337845       0.99999825    6.1977664e-08
      0.47723786       0.99999822    6.3747454e-08
      0.48112873       0.99999819    6.5553839e-08
      0.48505133       0.99999816    6.7396270e-08
      0.48900590       0.99999813    6.9482067e-08
      0.49299272       0.99999810    7.1706389e-08
      0.49701204       0.99999807    7.3976646e-08
      0.50106413       0.99999804    7.6293881e-08
      0.50514926       0.99999801    7.8658659e-08
      0.50926770       0.99999798    8.1072025e-08
      0.51341971       0.99999795    8.3535473e-08
      0.51760557       0.99999792    8.6049220e-08
      0.52182555       0.99999789    8.8614772e-08
      0.52607995       0.99999785    9.1231415e-08
      0.53036903       0.99999782    9.3902574e-08
      0.53469307       0.99999778    9.6627617e-08
      0.53905238       0.99999775    9.9409081e-08
      0.54344722       0.99999771    1.0224531e-07
      0.54787789       0.99999768    1.0537233e-07
      0.55234469       0.99999765    1.0860267e-07
      0.55684790       0.99999761    1.1190026e-07
      0.56138783       0.99999757    1.1526465e-07
      0.56596477       0.99999754    1.1869858e-07
      0.57057902       0.99999750    1.2220166e-07
      0.57523090       0.99999746    1.2577738e-07
      0.57992070       0.99999743    1.2942469e-07
      0.58464874       0.99999739    1.3331779e-07
      0.58941532       0.99999736    1.3729902e-07
      0.59422077       0.99999732    1.4136224e-07
      0.59906539       0.99999728    1.4550743e-07
      0.60394951       0.99999725    1.4983975e-07
      0.60887346       0.99999722    1.5426178e-07
      0.61383754       0.99999723    1.5884130e-07
      0.61884210       0.99999728    1.6351412e-07
      0.62388746       0.99999729    2.9026290e-08
      0.62897395       0.99999705    2.9941212e-08
      0.63410192       0.99999699    3.0890732e-08
      0.63927169       0.99999692    3.1862753e-08
      0.64448361       0.99999687    3.2872935e-08
      0.64973802       0.99999681    3.3908685e-08
      0.65503527       0.99999675    3.4992470e-08
      0.66037571       0.99999669    3.6100354e-08
      0.66575969       0.99999663    3.7233538e-08
      0.67118757       0.99999657    3.8406234e-08
      0.67665969       0.99999651    3.9677350e-08
      0.68217644       0.99999645    4.0977223e-08
      0.68773815       0.99999639    4.2306350e-08
      0.69334522       0.99999633    4.3665551e-08
      0.69899799       0.99999626    4.5055079e-08
      0.70469686       0.99999620    4.6476019e-08
      0.71044218       0.99999613    4.7928755e-08
      0.71623435       0.99999607    4.9414418e-08
      0.72207374       0.99999600    5.0932678e-08
      0.72796073       0.99999593    5.2485793e-08
      0.73389573       0.99999586    5.4073096e-08
      0.73987911       0.99999579    5.5944487e-08
      0.74591127       0.99999572    5.8409463e-08
      0.75199261       0.99999565    6.0930675e-08
      0.75812354       0.99999557    6.3511714e-08
      0.76430445       0.99999550    6.6151386e-08
      0.77053575       0.99999543    6.8853001e-08
      0.77681785       0.99999535    7.1615868e-08
      0.78315117       0.99999527    7.4438243e-08
      0.78953613       0.99999519    7.7329153e-08
      0.79597314       0.99999511    8.0286465e-08
      0.80246263       0.99999503    8.3305634e-08
      0.80900503       0.99999495    8.6398324e-08
      0.81560077       0.99999486    8.9560675e-08
      0.82225028       0.99999478    9.2788977e-08
      0.82895401       0.99999469    9.6100338e-08
      0.83571239       0.99999460    9.9474797e-08
      0.84252588       0.99999451    1.0293577e-07
      0.84939491       0.99999442    1.0646744e-07
      0.85631994       0.99999432    1.1007865e-07
      0.86330144       0.99999423    1.1377419e-07
      0.87033985       0.99999413    1.1755272e-07
      0.87743565       0.99999403    1.2141270e-07
      0.88458929       0.99999394    1.2535968e-07
      0.89180126       0.99999383    1.2939087e-07
      0.89907203       0.99999373    1.3352495e-07
      0.90640208       0.99999363    1.3773388e-07
      0.91379189       0.99999352    1.4205047e-07
      0.92124195       0.99999341    1.4645089e-07
      0.92875274       0.99999330    1.5096374e-07
      0.93632477       0.99999319    1.5556332e-07
      0.94395854       0.99999308    1.6027409e-07
      0.95165454       0.99999296    1.6508133e-07
      0.95941329       0.99999284    1.7000445e-07
      0.96723529       0.99999272    1.7502732e-07
      0.97512107       0.99999260    1.8016483e-07
      0.98307114       0.99999248    1.8541305e-07
      0.99108602       0.99999235    1.9078781e-07
      0.99916625       0.99999222    1.9627074e-07
       1.0073124       0.99999209    2.0187771e-07
       1.0155249       0.99999196    2.0760526e-07
       1.0238044       0.99999183    2.1346910e-07
       1.0321513       0.99999169    2.1945105e-07
       1.0405664       0.99999155    2.2557524e-07
       1.0490500       0.99999141    2.3182283e-07
       1.0576028       0.99999127    2.3821772e-07
       1.0662253       0.99999112    2.4474159e-07
       1.0749182       0.99999097    2.5141853e-07
       1.0836819       0.99999082    2.5823935e-07
       1.0925170       0.99999067    2.6521956e-07
       1.1014242       0.99999051    2.7233211e-07
       1.1104040       0.99999036    2.7962847e-07
       1.1194570       0.99999019    2.8706360e-07
       1.1285839       0.99999003    2.9466999e-07
       1.1377851       0.99998986    3.0244116e-07
       1.1470613       0.99998969    3.1039088e-07
       1.1564132       0.99998952    3.1850316e-07
       1.1658413       0.99998935    3.2681101e-07
       1.1753463       0.99998917    3.3528936e-07
       1.1849288       0.99998899    3.4395999e-07
       1.1945894       0.99998880    3.5282024e-07
       1.2043288       0.99998861    3.6186980e-07
       1.2141476       0.99998842    3.7112893e-07
       1.2240464       0.99998823    3.8058444e-07
       1.2340259       0.99998803    3.9068002e-07
       1.2440868       0.99998782    4.0618734e-07
       1.2542298       0.99998761    4.2206895e-07
       1.2644554       0.99998740    4.3830479e-07
       1.2747644       0.99998719    4.5491026e-07
       1.2851574       0.99998698    4.7190346e-07
       1.2956351       0.99998677    4.8928295e-07
       1.3061983       0.99998656    5.0706492e-07
       1.3168476       0.99998634    5.2522747e-07
       1.3275838       0.99998611    5.4382835e-07
       1.3384074       0.99998588    5.6282771e-07
       1.3493193       0.99998565    5.8228225e-07
       1.3603202       0.99998541    6.0215247e-07
       1.3714108       0.99998517    6.2249349e-07
       1.3825918       0.99998493    6.4329395e-07
       1.3938639       0.99998468    6.6453332e-07
       1.4052279       0.99998443    6.8627718e-07
       1.4166846       0.99998417    7.0850139e-07
       1.4282347       0.99998391    7.3122758e-07
       1.4398790       0.99998364    7.5445172e-07
       1.4516182       0.99998337    7.7822742e-07
       1.4634531       0.99998309    8.0249167e-07
       1.4753845       0.99998281    8.2733085e-07
       1.4874131       0.99998253    8.5273173e-07
       1.4995399       0.99998224    8.7867946e-07
       1.5117655       0.99998194    9.0520941e-07
       1.5240907       0.99998165    9.3234159e-07
       1.5365165       0.99998134    9.6004668e-07
       1.5490436       0.99998103    9.8841089e-07
       1.5616728       0.99998072    1.0173998e-06
       1.5744049       0.99998040    1.0470178e-06
       1.5872409       0.99998007    1.0772823e-06
       1.6001815       0.99997974    1.1082694e-06
       1.6132277       0.99997941    1.1398930e-06
       1.6263801       0.99997907    1.1722409e-06
       1.6396399       0.99997872    1.2053163e-06
       1.6530077       0.99997837    1.2391178e-06
       1.6664845       0.99997801    1.2736367e-06
       1.6800712       0.99997765    1.3089889e-06
       1.6937687       0.99997728    1.3450478e-06
       1.7075778       0.99997690    1.3819786e-06
       1.7214995       0.99997652    1.4196782e-06
       1.7355347       0.99997613    1.4582569e-06
       1.7496844       0.99997573    1.4976316e-06
       1.7639494       0.99997533    1.5379666e-06
       1.7783307       0.99997492    1.5790857e-06
       1.7928292       0.99997451    1.6212115e-06
       1.8074460       0.99997409    1.6641895e-06
       1.8221819       0.99997366    1.7082278e-06
       1.8370380       0.99997322    1.7531058e-06
       1.8520152       0.99997278    1.8077918e-06
       1.8671145       0.99997233    1.8677605e-06
       1.8823369       0.99997188    1.9292154e-06
       1.8976834       0.99997141    1.9919091e-06
       1.9131550       0.99997094    2.0561685e-06
       1.9287528       0.99997046    2.1217177e-06
       1.9444777       0.99996998    2.1889366e-06
       1.9603309       0.99996948    2.2575326e-06
       1.9763133       0.99996898    2.3277518e-06
       1.9924260       0.99996847    2.3994420e-06
       2.0086700       0.99996795    2.4729082e-06
       2.0250465       0.99996743    2.5478467e-06
       2.0415565       0.99996689    2.6246217e-06
       2.0582011       0.99996634    2.7030055e-06
       2.0749815       0.99996579    2.7832732e-06
       2.0918986       0.99996523    2.8651847e-06
       2.1089536       0.99996466    2.9490669e-06
       2.1261477       0.99996407    3.0347076e-06
       2.1434820       0.99996348    3.1223714e-06
       2.1609576       0.99996288    3.2118763e-06
       2.1785757       0.99996228    3.3189529e-06
       2.1963374       0.99996167    3.4296138e-06
       2.2142439       0.99996104    3.5429779e-06
       2.2322964       0.99996041    3.6587812e-06
       2.2504961       0.99995977    3.7773108e-06
       2.2688442       0.99995912    3.8983946e-06
       2.2873419       0.99995845    4.0223225e-06
       2.3059904       0.99995778    4.1489181e-06
       2.3247909       0.99995709    4.2785283e-06
       2.3437447       0.99995639    4.4108536e-06
       2.3628530       0.99995569    4.5463630e-06
       2.3821171       0.99995497    4.6847248e-06
       2.4015383       0.99995423    4.8263375e-06
       2.4211178       0.99995349    4.9710040e-06
       2.4408569       0.99995273    5.1190348e-06
       2.4607570       0.99995197    5.2702489e-06
       2.4808193       0.99995119    5.4249347e-06
       2.5010452       0.99995039    5.5829683e-06
       2.5214360       0.99994959    5.7446235e-06
       2.5419930       0.99994877    5.9098543e-06
       2.5627176       0.99994794    6.0788227e-06
       2.5836112       0.99994709    6.2514576e-06
       2.6046751       0.99994623    6.4279769e-06
       2.6259108       0.99994536    6.6142370e-06
       2.6473196       0.99994450    6.8267489e-06
       2.6689029       0.99994362    7.0441357e-06
       2.6906622       0.99994273    7.2663844e-06
       2.7125989       0.99994182    7.4936539e-06
       2.7347145       0.99994090    7.7259730e-06
       2.7570104       0.99993996    7.9635253e-06
       2.7794880       0.99993901    8.2063817e-06
       2.8021489       0.99993804    8.4548238e-06
       2.8249946       0.99993706    8.7085850e-06
       2.8480265       0.99993606    8.9682837e-06
       2.8712461       0.99993505    9.2334913e-06
       2.8946551       0.99993402    9.5049111e-06
       2.9182550       0.99993298    9.7821380e-06
       2.9420472       0.99993191    1.0065746e-05
       2.9660335       0.99993083    1.0355467e-05
       2.9902152       0.99992974    1.0651895e-05
       3.0145942       0.99992862    1.0954665e-05
       3.0391719       0.99992751    1.1277597e-05
       3.0639499       0.99992640    1.1625558e-05
       3.0889300       0.99992528    1.1981697e-05
       3.1141138       0.99992414    1.2345484e-05
       3.1395029       0.99992298    1.2717749e-05
       3.1650989       0.99992180    1.3097842e-05
       3.1909037       0.99992060    1.3487008e-05
       3.2169188       0.99991939    1.3884281e-05
       3.2431460       0.99991816    1.4290947e-05
       3.2695871       0.99991690    1.4706101e-05
       3.2962437       0.99991563    1.5131170e-05
       3.3231177       0.99991434    1.5564920e-05
       3.3502107       0.99991306    1.6026765e-05
       3.3775247       0.99991179    1.6511941e-05
       3.4050613       0.99991050    1.7008712e-05
       3.4328224       0.99990918    1.7515719e-05
       3.4608099       0.99990785    1.8034943e-05
       3.4890255       0.99990650    1.8564754e-05
       3.5174712       0.99990513    1.9107376e-05
       3.5461488       0.99990373    1.9660952e-05
       3.5750602       0.99990232    2.0227861e-05
       3.6042074       0.99990089    2.0806321e-05
       3.6335921       0.99989950    2.1419272e-05
       3.6632164       0.99989812    2.2054096e-05
       3.6930823       0.99989672    2.2704462e-05
       3.7231916       0.99989530    2.3367795e-05
       3.7535465       0.99989386    2.4047334e-05
       3.7841488       0.99989240    2.4740391e-05
       3.8150006       0.99989093    2.5450234e-05
       3.8461039       0.99988943    2.6174431e-05
       3.8774608       0.99988800    2.6935564e-05
       3.9090734       0.99988659    2.7717607e-05
       3.9409437       0.99988516    2.8518413e-05
       3.9730738       0.99988371    2.9335486e-05
       4.0054659       0.99988224    3.0171901e-05
       4.0381221       0.99988075    3.1025513e-05
       4.0710445       0.99987940    3.1922970e-05
       4.1042353       0.99987807    3.2842941e-05
       4.1376967       0.99987672    3.3784520e-05
       4.1714310       0.99987536    3.4745509e-05
       4.2054403       0.99987421    3.5745425e-05
       4.2397268       0.99987309    3.6767952e-05
       4.2742929       0.99987263    3.7826335e-05
       4.3091408       0.99987224    3.8907706e-05
       4.3442728       0.99987363    3.6291774e-05
       4.3796912       0.99986844    3.7326225e-05
       4.4153984       0.99986658    3.8390232e-05
       4.4513967       0.99986502    3.9484920e-05
       4.4876885       0.99986361    4.0609527e-05
       4.5242762       0.99986248    4.1766494e-05
       4.5611622       0.99986147    4.2954145e-05
       4.5983489       0.99986094    4.4177677e-05
       4.6358388       0.99986063    4.5433518e-05
       4.6736344       0.99986424    4.6726694e-05
       4.7117381       0.99986942    4.8053421e-05
       4.7501524       0.99986916    3.9393443e-05
       4.7888799       0.99985860    4.0527484e-05
       4.8279232       0.99985745    4.1695190e-05
       4.8672848       0.99986146    4.2898377e-05
       4.9069673       0.99986588    4.4128716e-05
       4.9469733       0.99987201    2.3441680e-05
       4.9873055       0.99984931    2.4126231e-05
       5.0279666       0.99984330    2.4830672e-05
       5.0689591       0.99983861    2.5555572e-05
       5.1102858       0.99983401    2.6299785e-05
       5.1519495       0.99982999    2.7066208e-05
       5.1939528       0.99982600    2.7852841e-05
       5.2362986       0.99982225    2.8663461e-05
       5.2789897       0.99981850    2.9495483e-05
       5.3220288       0.99981489    3.0352077e-05
       5.3654187       0.99981129    3.1232650e-05
       5.4091625       0.99980783    3.2143658e-05
       5.4532629       0.99980433    3.3076280e-05
       5.4977228       0.99980080    3.4032653e-05
       5.5425452       0.99979722    3.5011851e-05
       5.5877330       0.99979360    3.6015839e-05
       5.6332893       0.99979003    3.7051612e-05
       5.6792169       0.99978659    3.8129362e-05
       5.7255190       0.99978311    3.9232012e-05
       5.7721986       0.99977962    4.0362759e-05
       5.8192588       0.99977610    4.1519329e-05
       5.8667027       0.99977257    4.2705348e-05
       5.9145333       0.99976902    4.3918008e-05
       5.9627539       0.99976548    4.5161567e-05
       6.0113677       0.99976200    4.6452817e-05
       6.0603778       0.99975866    4.7802309e-05
       6.1097875       0.99975531    4.9181638e-05
       6.1596000       0.99975205    5.0581865e-05
       6.2098186       0.99974879    5.2007798e-05
       6.2604467       0.99974567    5.3481075e-05
       6.3114875       0.99974256    5.4988663e-05
       6.3629444       0.99973974    5.6538420e-05
       6.4148209       0.99973698    5.8121484e-05
       6.4671203       0.99973479    5.9748914e-05
       6.5198461       0.99973272    6.1411344e-05
       6.5730018       0.99973216    6.3120275e-05
       6.6265908       0.99973189    6.4865396e-05
       6.6806168       0.99975107    6.6660356e-05
       6.7350832       0.99977362    6.8555860e-05
       6.7899937       0.99975929    3.1009463e-05
       6.8453519       0.99971882    3.1955762e-05
       6.9011614       0.99970739    3.2923843e-05
       6.9574259       0.99969819    3.3911651e-05
       7.0141491       0.99969054    3.4921560e-05
       7.0713348       0.99968285    3.5951359e-05
       7.1289867       0.99967573    3.7004389e-05
       7.1871086       0.99966851    3.8078431e-05
       7.2457044       0.99966155    3.9174734e-05
       7.3047779       0.99965447    4.0292300e-05
       7.3643331       0.99964752    4.1435685e-05
       7.4243738       0.99964045    4.2601700e-05
       7.4849040       0.99963343    4.3794495e-05
       7.5459277       0.99962628    4.5010865e-05
       7.6074489       0.99961914    4.6254494e-05
       7.6694717       0.99961187    4.7523740e-05
       7.7320001       0.99960457    4.8820641e-05
       7.7950384       0.99959715    5.0144223e-05
       7.8585906       0.99958967    5.1497106e-05
       7.9226609       0.99958221    5.3050719e-05
       7.9872536       0.99957471    5.4669688e-05
       8.0523729       0.99956708    5.6321199e-05
       8.1180231       0.99955937    5.8009630e-05
       8.1842086       0.99955153    5.9732709e-05
       8.2509337       0.99954360    6.1493614e-05
       8.3182027       0.99953554    6.3290609e-05
       8.3860202       0.99952737    6.5122543e-05
       8.4543906       0.99951907    6.6992942e-05
       8.5233185       0.99951066    6.8904291e-05
       8.5928082       0.99950211    7.0854717e-05
       8.6628646       0.99949343    7.2848609e-05
       8.7334921       0.99948461    7.4883111e-05
       8.8046954       0.99947567    7.6961348e-05
       8.8764792       0.99946657    7.9082815e-05
       8.9488483       0.99945734    8.1250445e-05
       9.0218074       0.99944797    8.3462114e-05
       9.0953613       0.99943844    8.5721798e-05
       9.1695149       0.99942876    8.8027512e-05
       9.2442731       0.99941893    9.0383203e-05
       9.3196408       0.99940894    9.2787913e-05
       9.3956229       0.99939879    9.5244230e-05
       9.4722245       0.99938847    9.7750442e-05
       9.5494506       0.99937802    0.00010035573
       9.6273064       0.99936760    0.00010329116
       9.7057969       0.99935700    0.00010628890
       9.7849273       0.99934623    0.00010934900
       9.8647029       0.99933529    0.00011247567
       9.9451288       0.99932418    0.00011566858
       10.026211       0.99931288    0.00011893058
       10.107953       0.99930140    0.00012226159
       10.190362       0.99928973    0.00012566285
       10.273443       0.99927788    0.00012913687
       10.357202       0.99926583    0.00013268319
       10.441643       0.99925358    0.00013630546
       10.526773       0.99924113    0.00014000268
       10.612597       0.99922848    0.00014378051
       10.699120       0.99921562    0.00014763604
       10.786349       0.99920255    0.00015157444
       10.874289       0.99918926    0.00015559343
       10.962946       0.99917575    0.00015970037
       11.052326       0.99916213    0.00016404344
       11.142434       0.99914848    0.00016874624
       11.233278       0.99913460    0.00017354720
       11.324861       0.99912048    0.00017847414
       11.417192       0.99910611    0.00018351617
       11.510275       0.99909154    0.00018868968
       11.604117       0.99907674    0.00019397947
       11.698725       0.99906170    0.00019938990
       11.794103       0.99904643    0.00020491255
       11.890259       0.99903092    0.00021056162
       11.987199       0.99901516    0.00021632690
       12.084930       0.99899915    0.00022222438
       12.183457       0.99898287    0.00022824219
       12.282788       0.99896634    0.00023439869
       12.382928       0.99894960    0.00024049570
       12.483885       0.99893264    0.00024656075
       12.585665       0.99891540    0.00025274729
       12.688275       0.99889789    0.00025907665
       12.791721       0.99887974    0.00026600794
       12.896010       0.99886104    0.00027345470
       13.001150       0.99884266    0.00028138609
       13.107148       0.99882442    0.00028972965
       13.214009       0.99880589    0.00029824323
       13.321741       0.99878708    0.00030695984
       13.430352       0.99876782    0.00031702611
       13.539849       0.99874869    0.00032483463
       13.650238       0.99872912    0.00033248022
       13.761527       0.99870993    0.00034314182
       13.873723       0.99868912    0.00035182921
       13.986834       0.99866905    0.00036075244
       14.100868       0.99864764    0.00037073676
       14.215831       0.99862569    0.00038108051
       14.331731       0.99860436    0.00039272823
       14.448576       0.99858391    0.00040419571
       14.566374       0.99856408    0.00041551928
       14.685132       0.99854397    0.00042710479
       14.804859       0.99852065    0.00043795400
       14.925561       0.99849901    0.00045074170
       15.047248       0.99847755    0.00046197662
       15.169927       0.99845632    0.00047378799
       15.293606       0.99843277    0.00048448154
       15.418293       0.99840857    0.00049587275
       15.543997       0.99838435    0.00050942076
       15.670725       0.99835977    0.00052317718
       15.798487       0.99833222    0.00053741836
       15.927291       0.99830238    0.00055210697
       16.057145       0.99827916    0.00056995870
       16.188057       0.99825772    0.00058531611
       16.320037       0.99823393    0.00060110289
       16.453092       0.99820977    0.00061723829
       16.587233       0.99818495    0.00063241041
       16.722467       0.99815998    0.00064677492
       16.858803       0.99813550    0.00066230723
       16.996252       0.99811066    0.00067817440
       17.134820       0.99807962    0.00069633383
       17.274519       0.99804820    0.00071242718
       17.415356       0.99801836    0.00073508145
       17.557342       0.99799183    0.00075220489
       17.700486       0.99796231    0.00077478554
       17.844796       0.99793797    0.00079560261
       17.990283       0.99791187    0.00081447175
       18.136956       0.99788521    0.00083342800
       18.284825       0.99785820    0.00085277423
       18.433899       0.99782909    0.00087234612
       18.584189       0.99779689    0.00089202620
       18.735704       0.99776836    0.00091432648
       18.888454       0.99773715    0.00093639596
       19.042450       0.99770641    0.00095789380
       19.197701       0.99767624    0.00098178559
       19.354218       0.99764444     0.0010047126
       19.512011       0.99761219     0.0010281244
       19.671091       0.99757949     0.0010520352
       19.831467       0.99754633     0.0010764543
       19.993151       0.99751271     0.0011013907
       20.156154       0.99747912     0.0011330332
       20.320485       0.99744798     0.0011639647
       20.486156       0.99742299     0.0011921370
       20.653177       0.99738987     0.0012193285
       20.821561       0.99736651     0.0012415236
       20.991317       0.99734423     0.0012629816
       21.162457       0.99730609     0.0012720541
       21.334993       0.99722925     0.0012955626
       21.508935       0.99720296     0.0013465477
       21.684295       0.99719840     0.0014050359
       21.861085       0.99714605     0.0014054083
       22.039316       0.99711264     0.0014444487
       22.219001       0.99706255     0.0014501227
       22.400150       0.99700848     0.0015025204
       22.582776       0.99697129     0.0015528694
       22.766892       0.99693694     0.0015589743
       22.952508       0.99691703     0.0016440162
       23.139637       0.99687650     0.0017211447
       23.328293       0.99684843     0.0017614551
       23.518486       0.99682577     0.0017909207
       23.710230       0.99681323     0.0018291317
       23.903537       0.99679721     0.0018573589
       24.098420       0.99678614     0.0018792772
       24.294892       0.99674447     0.0019058670
       24.492966       0.99670958     0.0019183206
       24.692655       0.99667219     0.0019568137
       24.893972       0.99662558     0.0019697977
       25.096930       0.99657362     0.0019973988
       25.301543       0.99650894     0.0020492601
       25.507824       0.99647978     0.0021122828
       25.715787       0.99642595     0.0021288260
       25.925445       0.99638846     0.0021801028
       26.136813       0.99632063     0.0022162974
       26.349904       0.99627477     0.0022893499
       26.564732       0.99622679     0.0023248419
       26.781312       0.99618080     0.0023927763
       26.999657       0.99613066     0.0024608202
       27.219783       0.99608791     0.0025138411
       27.441703       0.99604781     0.0026045585
       27.665432       0.99604625     0.0026888638
       27.890986       0.99604345     0.0027439876
       28.118378       0.99604561     0.0027734039
       28.347625       0.99601718     0.0027911137
       28.578740       0.99596767     0.0028161355
       28.811740       0.99591915     0.0028554052
       29.046639       0.99584072     0.0029011565
       29.283454       0.99579063     0.0029778241
       29.522199       0.99577300     0.0030177342
       29.762891       0.99563755     0.0030757003
       30.005545       0.99551896     0.0033270439
       30.250177       0.99576897     0.0033322759
       30.496804       0.99565513     0.0033641399
       30.745441       0.99566986     0.0034108232
       30.996106       0.99552256     0.0034352560
       31.248814       0.99533948     0.0037303407
       31.503583       0.99601781     0.0040870977
       31.760428       0.99592017     0.0035660783
       32.019368       0.99574801     0.0035465866
       32.280419       0.99567864     0.0035755489
       32.543598       0.99561791     0.0036031186
       32.808923       0.99551535     0.0036317486
       33.076411       0.99543972     0.0036740911
       33.346079       0.99535997     0.0037388852
       33.617947       0.99529920     0.0038175451
       33.892031       0.99525344     0.0038823435
       34.168349       0.99519801     0.0039428900
       34.446920       0.99515050     0.0040153914
       34.727763       0.99511416     0.0040894109
       35.010895       0.99507819     0.0041649976
       35.296335       0.99503398     0.0042229940
       35.584103       0.99499397     0.0042873815
       35.874217       0.99493475     0.0043578637
       36.166696       0.99488981     0.0044411022
       36.461559       0.99485908     0.0045116413
       36.758827       0.99482780     0.0045806414
       37.058518       0.99479159     0.0046440024
       37.360653       0.99473923     0.0047161091
       37.665250       0.99469313     0.0048010771
       37.972331       0.99465319     0.0048937966
       38.281916       0.99462683     0.0049929604
       38.594025       0.99461937     0.0050863519
       38.908678       0.99462599     0.0051623886
       39.225897       0.99463044     0.0052288858
       39.545702       0.99463508     0.0052928957
       39.868114       0.99463648     0.0053420045
       40.193155       0.99464442     0.0053702603
       40.520846       0.99460891     0.0053926189
       40.851209       0.99459615     0.0054379842
       41.184265       0.99454602     0.0054660726
       41.520036       0.99450719     0.0055049737
       41.858545       0.99445873     0.0055441771
       42.199813       0.99440905     0.0055934722
       42.543864       0.99436202     0.0056533629
       42.890720       0.99432258     0.0057150104
       43.240404       0.99428436     0.0057780182
       43.592939       0.99424890     0.0058429528
       43.948348       0.99421739     0.0059121395
       44.306655       0.99420684     0.0059933155
       44.667883       0.99421692     0.0060758150
       45.032056       0.99425656     0.0061223760
       45.399198       0.99429325     0.0061236499
       45.769333       0.99432083     0.0061105662
       46.142486       0.99434709     0.0060936651
       46.518681       0.99435752     0.0060460673
       46.897943       0.99434181     0.0059378850
       47.280298       0.99429194     0.0058122056
       47.665769       0.99417616     0.0057021317
       48.054384       0.99402281     0.0056004882
       48.446167       0.99382989     0.0055378411
       48.841143       0.99362494     0.0055003241
       49.239341       0.99340877     0.0055004293
       49.640784       0.99319681     0.0055499062
       50.045501       0.99302639     0.0056255587
       50.453517       0.99284954     0.0057048607
       50.864859       0.99270472     0.0057625372
       51.279556       0.99254576     0.0057857665
       51.697633       0.99236955     0.0058134064
       52.119119       0.99215563     0.0058727407
       52.544041       0.99194100     0.0059576473
       52.972427       0.99176052     0.0060699570
       53.404306       0.99158498     0.0061786749
       53.839707       0.99142371     0.0062691485
       54.278657       0.99124792     0.0063974127
       54.721185       0.99107368     0.0064678042
       55.167322       0.99087438     0.0066054839
       55.617096       0.99070112     0.0067494928
       56.070536       0.99053779     0.0068970788
       56.527674       0.99038124     0.0070285584
       56.988539       0.99022615     0.0071638499
       57.453161       0.99006567     0.0072980069
       57.921571       0.98990769     0.0074398609
       58.393800       0.98974901     0.0075793464
       58.869879       0.98959006     0.0077246049
       59.349839       0.98942840     0.0078736815
       59.833713       0.98926400     0.0080254824
       60.321531       0.98910211     0.0081787193
       60.813327       0.98894255     0.0083399289
       61.309132       0.98877876     0.0085043834
       61.808979       0.98861590     0.0086761800
       62.312902       0.98845793     0.0088460339
       62.820933       0.98829159     0.0090255733
       63.333106       0.98813044     0.0092054088
       63.849455       0.98796989     0.0093931279
       64.370013       0.98781250     0.0095875135
       64.894816       0.98765972     0.0097845278
       65.423897       0.98751411     0.0099793175
       65.957292       0.98736612      0.010171266
       66.495035       0.98720212      0.010333128
       67.037163       0.98701893      0.010538904
       67.583710       0.98686110      0.010751528
       68.134713       0.98668050      0.010946838
       68.690209       0.98645960      0.011123632
       69.250234       0.98621211      0.011367462
       69.814824       0.98591902      0.011678669
       70.384018       0.98566107      0.012061646
       70.957852       0.98546920      0.012568135
       71.536364       0.98537444      0.013084670
       72.119593       0.98543189      0.013635391
       72.707578       0.98555498      0.014070299
       73.300355       0.98577737      0.014382045
       73.897966       0.98597966      0.014593668
       74.500449       0.98616409      0.014738748
       75.107844       0.98629373      0.014861187
       75.720191       0.98637033      0.014969382
       76.337530       0.98643214      0.015010838
       76.959903       0.98647215      0.014978077
       77.587350       0.98641449      0.014978409
       78.219912       0.98627702      0.014997470
       78.857631       0.98603960      0.015050453
       79.500550       0.98572402      0.015129109
       80.148710       0.98557986      0.015480425
       80.802155       0.98555472      0.016036312
       81.460927       0.98555891      0.016258912
       82.125070       0.98558605      0.016276137
       82.794628       0.98552244      0.016402139
       83.469644       0.98541499      0.016602370
       84.150164       0.98537430      0.016746606
       84.836232       0.98535376      0.016848290
       85.527894       0.98525408      0.016916520
       86.225195       0.98511442      0.016958677
       86.928180       0.98496504      0.017112550
       87.636897       0.98480960      0.017312422
       88.351392       0.98456860      0.017497004
       89.071713       0.98431416      0.017683060
       89.797906       0.98404464      0.017966723
       90.530020       0.98374406      0.018316126
       91.268102       0.98360966      0.018952575
       92.012202       0.98353006      0.019666088
       92.762369       0.98387046      0.020187755
       93.518651       0.98432624      0.020655404
       94.281100       0.98473036      0.020784923
       95.049764       0.98513399      0.020832088
       95.824696       0.98527835      0.020812539
       96.605945       0.98539079      0.020819884
       97.393564       0.98546445      0.020870244
       98.187605       0.98553847      0.020906035
       98.988119       0.98564246      0.021017673
       99.795159       0.98573855      0.021131982
       100.60878       0.98595531      0.021240294
       101.42903       0.98617937      0.021356753
       102.25597       0.98642900      0.021333604
       103.08966       0.98670688      0.021280229
       103.93014       0.98715661      0.021157961
       104.77747       0.98759625      0.021026703
       105.63171       0.98795505      0.020471625
       106.49292       0.98830323      0.019900216
       107.36114       0.98819443      0.019179188
       108.23645       0.98805707      0.018433264
       109.11889       0.98772527      0.018096139
       110.00852       0.98737973      0.017750752
       110.90541       0.98717406      0.017368549
       111.80961       0.98695456      0.016985441
       112.72119       0.98666541      0.016583527
       113.64019       0.98637721      0.016190105
       114.56669       0.98606671      0.015840579
       115.50074       0.98571642      0.015464925
       116.44241       0.98531484      0.014892358
       117.39175       0.98483651      0.014310284
       118.34883       0.98404009      0.013897250
       119.31372       0.98324481      0.013693371
       120.28647       0.98256832      0.014375934
       121.26716       0.98233305      0.015118988
       122.25584       0.98373804      0.016026766
       123.25258       0.98746000      0.016988839
       124.25744       0.99692796     0.0091498557
       125.27050       0.98684071     0.0037543832
       126.29182       0.98321875     0.0037877321
       127.32147       0.98003996     0.0038306912
       128.35951       0.97808249     0.0038725281
       129.40601       0.97621807     0.0039261131
       130.46104       0.97463073     0.0039813542
       131.52468       0.97309444     0.0040591909
       132.59699       0.97166324     0.0041327627
       133.67804       0.97024681     0.0042105390
       134.76790       0.96888157     0.0043086631
       135.86665       0.96752491     0.0044072282
       136.97436       0.96618698     0.0045047430
       138.09110       0.96483588     0.0046209231
       139.21694       0.96350227     0.0047546313
       140.35196       0.96216248     0.0048779191
       141.49624       0.96082528     0.0050035976
       142.64984       0.95943131     0.0051532109
       143.81285       0.95803404     0.0052958062
       144.98535       0.95658663     0.0054581797
       146.16740       0.95509397     0.0057131758
       147.35909       0.95365438     0.0059233120
       148.56049       0.95214417     0.0061822909
       149.77169       0.95062561     0.0064478287
       150.99277       0.94905890     0.0067547304
       152.22379       0.94746928     0.0071108898
       153.46486       0.94586079     0.0074899588
       154.71604       0.94420519     0.0079199662
       155.97743       0.94251694     0.0083613797
       157.24910       0.94080664     0.0089240618
       158.53114       0.93912265     0.0095204333
       159.82362       0.93739275      0.010172063
       161.12665       0.93569545      0.010856023
       162.44030       0.93403645      0.011594989
       163.76466       0.93239492      0.012341863
       165.09982       0.93079049      0.013082230
       166.44586       0.92916240      0.013942492
       167.80288       0.92735765      0.014523671
       169.17096       0.92554641      0.015231116
       170.55019       0.92381229      0.016112104
       171.94067       0.92184207      0.016958175
       173.34249       0.92005405      0.017631724
       174.75573       0.91765701      0.018805539
       176.18050       0.91576768      0.020177522
       177.61688       0.91394396      0.021076102
       179.06498       0.91170318      0.022368039
       180.52487       0.90944322      0.023663986
       181.99668       0.90732412      0.025363755
       183.48048       0.90559128      0.027078908
       184.97638       0.90376184      0.029005621
       186.48447       0.90206049      0.030799673
       188.00486       0.90049342      0.032277209
       189.53765       0.89901818      0.033198519
       191.08293       0.89582044      0.035017742
       192.64081       0.89561434      0.037412867
       194.21139       0.89288533      0.039211940
       195.79478       0.89129413      0.040513141
       197.39107       0.88863954      0.043728082
       199.00038       0.88836498      0.044457237
       200.62281       0.88544536      0.047556158
       202.25847       0.88342669      0.047304580
       203.90747       0.88219315      0.053144808
       205.56990       0.88090964      0.052773986
       207.24590       0.87864157      0.055135753
       208.93555       0.87640104      0.057574873
       210.63898       0.87440656      0.060205097
       212.35630       0.87243408      0.062937852
       214.08762       0.87064145      0.065896218
       215.83306       0.86882266      0.068918780
       217.59272       0.86698478      0.072006454
       219.36674       0.86520195      0.075224946
       221.15521       0.86362781      0.078692873
       222.95827       0.86211272      0.082288561
       224.77603       0.86078001      0.086093748
       226.60860       0.85968881      0.090044045
       228.45612       0.85916070      0.094219646
       230.31870       0.85880573      0.098505705
       232.19647       0.85879442       0.10292430
       234.08954       0.85891148       0.10693786
       235.99805       0.85924754       0.11014415
       237.92212       0.85970641       0.11343472
       239.86188       0.86035431       0.11682584
       241.81745       0.86103948       0.12028495
       243.78896       0.86176811       0.12381322
       245.77655       0.86256610       0.12699012
       247.78034       0.86344749       0.12966996
       249.80047       0.86437021       0.13239853
       251.83707       0.86533702       0.13517663
       253.89028       0.86634715       0.13800519
       255.96022       0.86740300       0.14088503
       258.04704       0.86855065       0.14344886
       260.15087       0.86979323       0.14569170
       262.27186       0.87097534       0.14777876
       264.41013       0.87210788       0.14972471
       266.56584       0.87311199       0.15169573
       268.73913       0.87401640       0.15369204
       270.93013       0.87496045       0.15571493
       273.13900       0.87594518       0.15776396
       275.36588       0.87704042       0.15983781
       277.61091       0.87822573       0.16193831
       279.87424       0.87967715       0.16390490
       282.15603       0.88131431       0.16579859
       284.45642       0.88294671       0.16723219
       286.77557       0.88460162       0.16841291
       289.11362       0.88588601       0.16960626
       291.47073       0.88700831       0.17080925
       293.84707       0.88817381       0.17201906
       296.24277       0.88938341       0.17323565
       298.65801       0.89063838       0.17445897
       301.09294       0.89193947       0.17568909
       303.54772       0.89357492       0.17693284
       306.02252       0.89536546       0.17818602
       308.51749       0.89753998       0.17884543
       311.03281       0.89988429       0.17931317
       313.56863       0.90211765       0.17849347
       316.12512       0.90437150       0.17728997
       318.70246       0.90573458       0.17612116
       321.30081       0.90692129       0.17491770
       323.92034       0.90756591       0.17361529
       326.56123       0.90812444       0.17224877
       329.22366       0.90835498       0.17076057
       331.90778       0.90854620       0.16920269
       334.61380       0.90835485       0.16772268
       337.34187       0.90811874       0.16620136
       340.09219       0.90758681       0.16475875
       342.86492       0.90702322       0.16327190
       345.66027       0.90627338       0.16186228
       348.47840       0.90550294       0.16040575
       351.31951       0.90503442       0.15903181
       354.18379       0.90457819       0.15760853
       357.07142       0.90394799       0.15619772
       359.98258       0.90384272       0.15264895
       362.91749       0.90251662       0.14845820
       365.87632       0.90016448       0.14381300
       368.85927       0.89456190       0.13935709
       371.86655       0.88751772       0.13753368
       374.89834       0.88262007       0.13915561
       377.95485       0.87900526       0.14112984
       381.03628       0.87782062       0.14153728
       384.14283       0.87658513       0.14082975
       387.27471       0.87523300       0.13888309
       390.43212       0.87341459       0.13585926
       393.61528       0.86980567       0.13269559
       396.82438       0.86608232       0.12942564
       400.05966       0.86182464       0.12570309
       403.32130       0.85609127       0.12258174
       406.60954       0.84927626       0.11970045
       409.92459       0.84222909       0.11676113
       413.26667       0.83411999       0.11555639];

lld=AAA(:,1)/10000;nn=AAA(:,2)+i*AAA(:,3);
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 

%------------------------------------------------------
case {'MgO'}
bornes=[12.2,43.47];ref='Palik transmis par Mondher';

ldn=[12.2000          0.9130+0.0147i;...
  12.2700             0.8930+0.0143i;...
  12.3500             0.8730+0.0146i;...
  12.5000             0.8290+0.0167i;...
  12.6600             0.7800+0.0236i;...
  12.8200             0.7240+0.0295i;...
  12.9900             0.6600+0.0346i;...
  13.3300             0.5030+0.0475i;...
  13.5100             0.3930+0.0913i;...
  13.7000             0.2670+0.1720i;...
  13.8900             0.1560+0.3170i;...
  14.0800             0.1390+0.4780i;...
  14.2900             0.1320+0.5950i;...
  14.4900             0.1340+0.6920i;...
  14.7100             0.1360+0.7870i;...
  14.9300             0.1430+0.8770i;...
  15.1500             0.1540+0.9550i;...
  15.3800             0.1550+1.0300i;...
  15.6300             0.1510+1.0900i;...
  15.8700             0.1350+1.1700i;...
  16.1300             0.1160+1.2600i;...
  16.3900             0.1080+1.3500i;...
  16.6700             0.0980+1.4500i;...
  16.9500             0.0860+1.5600i;...
  17.2400             0.0870+1.6700i;...
  17.5400             0.0860+1.7800i;...
  17.8600             0.0880+1.9000i;...
  18.1800             0.0890+2.0300i;...
  18.5200             0.0910+2.1600i;...
  18.8700             0.0930+2.3100i;...
  19.2300             0.0970+2.4700i;...
  19.6100             0.1020+2.6400i;...
  20.0000             0.1090+2.8400i;...
  20.4100             0.1190+3.0600i;...
  20.8300             0.1310+3.3100i;...
  21.2800             0.1480+3.6000i;...
  21.7400             0.1700+3.9500i;...
  22.2200             0.2070+4.3900i;...
  22.7300             0.2610+4.9500i;...
  23.2600             0.3620+5.7100i;...
  23.8100             0.5430+6.8600i;...
  24.3900             1.0100+8.9700i;...
  25.0000             5.0500+15.2000i;...
  25.6400            13.5000+4.0600i;...
  26.3200             9.2800+1.0200i;...
  27.0300             7.4400+0.5600i;...
  27.7800             6.4600+0.3790i;...
  28.5700             5.8300+0.2660i;...
  29.4100             5.3500+0.2080i;...
  30.3000             5.0300+0.1680i;...
  31.2500             4.7400+0.1290i;...
  32.2600             4.5100+0.1260i;...
  33.3300             4.3600+0.1130i;...
  37.0400             3.9700+0.0290i;...
  38.4600             3.8700+0.0276i;...
  40.0000             3.7800+0.0265i;...
  41.6700             3.7100+0.0254i;...
  43.4700             3.6400+0.0233i];
lld=ldn(:,1);nn=ldn(:,2);
if isempty(lambda);indice=[];else;indice=interp1(lld,nn,lambda,method,NaN*(1+1i));end; 





  otherwise
    error(['Milieu "' milieu '" inconnu!!'])
end
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [indice,bornes,ref,ldr,nr,ldi,ni]=indiceor(ld);
lld=[];nn=[];
bornes=[.55,.95];ref='OR  approximation polynomiale de degre 4 de Corn   (2.04) ';
% ld microns 
indice=(29.0387-140.431*ld+254.394*ld.^2-203.936*ld.^3+61.1781*ld.^4)-i*(5*ld.^2-16.3*ld+4.9253);
indice(ld<bornes(1)|ld>bornes(2))=nan+i*nan;
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [indice,bornes,ref,ldr,nr,ldi,ni]=indiceor_drude(ld);
lld=[];nn=[];
bornes=[0,inf];ref=' modele de Drude  benchmark reseau  (mauvaise approximation)';
% ld microns
c=2.99792458e14;omegap=1.374e16/(2*pi*c);gama=3.21e14/(2*pi*c);
indice=sqrt(1-omegap^2*(ld.^2./(1+i*gama*ld)));
indice(ld<bornes(1)|ld>bornes(2))=nan+i*nan;
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [indice,bornes,ref,ldr,nr,ldi,ni]=indiceSiC_drude(ld);
lld=[];nn=[];
bornes=[];ref='modele de Drude J le Gall M Olivier JJ Greffet Phys Rev B vol 55 number 15 15 april 1997'; 
omegaL=969;omegaT=793;Gama=-4.76;eps_inf=6.7;
indice=sqrt(eps_inf*(1+(omegaL^2-omegaT^2)./(omegaT^2-(10000./ld).^2+i*Gama*(10000./ld))));
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [indice,bornes,ref,ldr,nr,ldi,ni]=indiceag_drude(ld);
lld=[];nn=[];
bornes=[.75,.9];ref=' modele de Drude ''comment'' G Gay,Alloschery,Weiner .Nature Phys (assez mediocre ...) '; 
% ld microns
c=2.99792458e14;omegap=1.3552e16/(2*pi*c);gama=1.9944e14/(2*pi*c);
indice=sqrt(3.2938-omegap^2*(ld.^2./(1+i*gama*ld)));
indice(ld<bornes(1)|ld>bornes(2))=nan+i*nan;
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [indice,bornes,ref,ldr,nr,ldi,ni]=indiceag_drude_benchmark(ld);
lld=[];nn=[];
bornes=[.4,2];ref=' modele de Drude ''benchmark''  ';
% ld microns
c=3e14;omegap=1.3388e16/(2*pi*c);gama=.707592e14/(2*pi*c);
indice=sqrt(3.36174-omegap^2*(ld.^2./(1+i*gama*ld)));

indice(real(ld)<bornes(1)|real(ld)>bornes(2))=nan+i*nan;
ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);


% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [indice,bornes,ref,ldr,nr,ldi,ni]=indice_Si(ld,T,cas);  
if ischar(T);T=293;end;
switch cas;
case 0; %silicium    
bornes=[1.1,5.6];
S1=10.4907-2.0802e-4*T+4.21694e-6*T^2-5.82298e-9*T^3+3.44688e-12*T^4;
S2=-1346.61+29.1664*T-.278724*T^2+1.05939e-3*T^3-1.35089e-6*T^4;
S3=4.42827e7-1.76213e6*T-7.61575e4*T^2+678.414*T^3+103.243*T^4;
ld1=.299713-1.14234e-5*T+1.67134e-7*T^2-2.51049e-10*T^3+2.32484e-14*T^4;
ld2=-3.5171e3+42.3892*T-.357957*T^2+1.17504e-3*T^3-1.13212e-6*T^4;
ld3=1.714e6-1.44984e5*T-6.90744e3*T^2-39.3699*T^3+23.577*T^4;
case 1; %germanium    
bornes=[1.9,5.5];
S1=13.9723+2.52809e-3*T-5.02195e-6*T^2+2.22604e-8*T^3-4.86238e-12*T^4;
S2=.452096-3.09197e-3*T+2.16895e-5*T^2-6.02290e-8*T^3+4.12038e-11*T^4;
S3=751.447-14.2843*T-.238093*T^2+2.96047e-3*T^3-7.73454e-6*T^4;
ld1=.386367+2.01871e-4*T-5.93448e-7*T^2-2.27923e-10*T^3+5.37423e-12*T^4;
ld2=1.08843+1.16510e-3*T-4.97284e-6*T^2+1.12357e-8*T^3+9.40201e-12*T^4;
ld3=-2893.19-.967948*T-.527016*T^2+6.49364e-3*T^3-1.95162e-5*T^4;
end;

indice=sqrt(1+S1*ld.^2./(ld.^2-ld1^2)+S2*ld.^2./(ld.^2-ld2^2)+S3*ld.^2./(ld.^2-ld3^2));

ref='SILICIUM Temperature dependent refractive index of silicon and germanium Frey Leviton Madison,Proc,SPIE Vol,6273,6272J'; 
indice(ld<bornes(1)|ld>bornes(2))=nan+i*nan;
lld=[];nn=[];ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [indice,bornes,ref,ldr,nr,ldi,ni]=indice_algaas(cas,lambda,x)
% Cette fonction retourne l'indice du matriau AlxGa1-xAs en fonction du paramtre x
% et de la longueur d'onde exprime en microns
% Rfrence : "Optolectronique", Emmanuel Rosencher et Borge Vinter, Masson, chapitre 7B2, P264
ref='Optolectronique, Emmanuel Rosencher et Borge Vinter, Masson, chapitre 7B2, P264';
bornes=[0,inf];
E = 1.24./lambda; % E en eV, lambda en microns
lld=[];nn=[];ldr=lld;ldi=lld;nr=real(nn);ni=imag(nn);
if ischar(x);x=0;end;

switch cas;
case 0;ref=['AlxGA1-xAs ',ref];
Eoe = 3.65 + 0.871*x + 0.179*(x^2);
Ed  = 36.1 - 2.45*x;
Eg  = 1.424 + 1.266*x + 0.26*(x^2);
case 1;ref=['PxAs1-xGa ',ref];
Eoe = 3.65 + 0.721*x + 0.139*(x^2);
Ed  = 36.1+0.35*x;
Eg = 1.441 + 1.091*x + 0.21*(x^2);
case 2;ref=['GaxIn1-xP ',ref];
Eoe = 3.391 + 0.524*x + 0.595*(x^2);
Ed = 28.91 + 7.54*x;
Eg = 1.34 + 0.668*x + 0.758*(x^2);
end;


Ef = sqrt((2*(Eoe^2)) - Eg^2);
eta = (pi/2) * (Ed/((Eoe^3)*(Eoe^2 - Eg^2)));
M_1 = (eta/(2*pi))*(Ef^4 - Eg^4);
M_3 = (eta/pi) * (Ef^2 - Eg^2);
epsilon = 1 + M_1 + M_3*(E.^2) + ((eta/pi)*log((Ef^2-E.^2)/(Eg^2 - E.^2)))*E.^4; % Formule d'Afromowitz

indice = sqrt(epsilon);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [n,bornes,ref]=indicefibre(ld,d,n2,n1);
bornes=[nan,nan];ref='formule approchee puis recherche par retmarcuse';
n2=real(n2);n1=real(n1);
% d et ld ont la meme longueur  n2 n1 reels n2/n1 peut varier entre 1.18 et 5
if isempty(ld)|isempty(d);n=[];return;end;
if length(ld)<length(d);ld=ld(1)*ones(size(d));end;
if length(d)<length(ld);d=d(1)*ones(size(ld));end;
   

if n2>5*n1;aa=[-7.3757  ;  5.4352 ;   2.3411  ;  1.1996];
% else;aa=[ -1.2058    5.4116  -11.8752;    0.7901   -3.2581    7.5089;   -0.1713    1.6521   -1.1545;   -0.1635    0.8319    0.2175];
else;aa=[-1.3744    6.4535  -12.8696;    0.8809   -3.8194    8.0447;   -0.1559    1.5568   -1.0635;   -0.2353    1.2757   -0.2061];
end;

a=zeros(1,size(aa,1));for ii=1:size(aa,1);a(ii)=polyval(aa(ii,:),n2/n1);end;

x=n2*d./ld;  % (n2/n1)*(d*n1/ld)
n=exp(-exp(a(1)*x+a(2))).* (1-1./(a(3).*x.^2+a(4)));
n=n1+(n2-n1)*n;
for ii=1:length(d);k0=2*pi/ld(ii);
if abs(n(ii)-n1)<10*eps;n(ii)=n1;
else;prv=real(retmarcuse([n1,n2],d(ii)/2,n(ii)*k0,1,[],[],k0)/k0);
if isnan(prv);if abs(n(ii)-n1)<abs(n(ii)-n2);n(ii)=n1;else;n(ii)=n2;end;else;n(ii)=prv;end;	
end;
end;

%%%%%%%%%%%%%%%  n1=10;n3=n1*5;ld=.95;d=(ld/n3)*linspace(.1,10,1000);neff=retindice(ld,d,n3,n1);
%%%%%%%%%%%%%%%  plot(n1*d./ld,neff/n1)